@@ -44,7 +44,7 @@ using namespace std;
 Backtrace::_lines_status 
        Backtrace::lines_status = Backtrace::LINES_not_checked;
 
-vector<Backtrace::PC_src> Backtrace::pc_2_src;
+Simple_string<Backtrace::PC_src, false> Backtrace::pc_2_src;
 
 //-----------------------------------------------------------------------------
 const char *
@@ -148,7 +148,7 @@ int64_t prev_pc = -1LL;
                       }
 
                    PC_src pcs = { pc, src_line };
-                   pc_2_src.push_back(pcs);
+                   pc_2_src.append(pcs);
                    prev_pc = pc;
                    new_line = false;
                  }
@@ -267,8 +267,8 @@ Backtrace::show(const char * file, int line)
    cerr << "Cannot show function call stack since execinfo.h seems not"
            " to exist on this OS (WINDOWs ?)." << endl;
    return;
-#else
 
+#else
 
    open_lines_file();
 
@@ -303,3 +303,47 @@ char ** strings = backtrace_symbols(buffer, size);
 #endif
 }
 //-----------------------------------------------------------------------------
+#ifdef HAVE_EXECINFO_H
+int
+Backtrace::demangle_line(char * result, size_t result_max, const char * buf)
+{
+Simple_string<char, false> tmp;
+   tmp.reserve(result_max + 1);
+   for (const char * b = buf; *b &&  b < (buf + result_max); ++b)
+       tmp.append(*b);
+   tmp.append(0);
+
+char * e = 0;
+int status = 3;
+char * p = strchr(&tmp[0], '(');
+   if (p == 0)   goto error;
+   else          ++p;
+
+   e = strchr(p, '+');
+   if (e == 0)   goto error;
+   else *e = 0;
+
+// cerr << "mangled fun is: " << p << endl;
+   __cxxabiv1::__cxa_demangle(p, result, &result_max, &status);
+   if (status)   goto error;
+   return 0;
+
+error:
+   strncpy(result, buf, result_max);
+   result[result_max - 1] = 0;
+   return status;
+}
+//-----------------------------------------------------------------------------
+const char *
+Backtrace::caller(int offset)
+{
+void * buffer[200];
+const int size = backtrace(buffer, sizeof(buffer)/sizeof(*buffer));
+char ** strings = backtrace_symbols(buffer, size);
+
+static char demangled[200] = { 0 };
+   demangle_line(demangled, sizeof(demangled), strings[offset]);
+   return demangled;
+}
+//-----------------------------------------------------------------------------
+#endif // HAVE_EXECINFO_H
