@@ -161,7 +161,7 @@ Value_P Z;
       }
    else
       {
-        Workspace::more_error() = UCS_string("Bad left argument of ⍕");
+        MORE_ERROR() = "Bad left argument of ⍕";
         DOMAIN_ERROR;
       }
 
@@ -219,15 +219,16 @@ const ShapeItem rows = B->get_rows();
    // split format into format fields, one per column.
    // If there is only one format field, then repeat it cols times.
    //
-vector<UCS_string> col_formats;
+UCS_string_vector col_formats;
    split_example_into_columns(format, col_formats);
    if (col_formats. size() == 1)
       {
-        UCS_string f = format;
+        const UCS_string f = format;
+        const UCS_string col0 = col_formats[0];   // keep it out of loop!
         loop(c, cols - 1)
            {
              format.append(f);
-             col_formats.push_back(col_formats.front());
+             col_formats.append(col0);
            }
       }
 
@@ -235,9 +236,9 @@ vector<UCS_string> col_formats;
 
    // convert each column format string into a Format_LIFER
    //
-vector<Format_LIFER> col_items;
+Simple_string<Format_LIFER, false> col_items;
    loop(c, col_formats.size())
-       col_items.push_back(Format_LIFER(col_formats[c]));
+       col_items.append(Format_LIFER(col_formats[c]));
 
    Log(LOG_Bif_F12_FORMAT)
       {
@@ -302,7 +303,7 @@ Value_P Z(shape_Z, LOC);
 //-----------------------------------------------------------------------------
 void
 Bif_F12_FORMAT::split_example_into_columns(const UCS_string & format,
-                                   vector<UCS_string> & col_formats)
+                                   UCS_string_vector & col_formats)
 {
 bool fmt_seen = false;
 UCS_string fmt;
@@ -316,8 +317,8 @@ UCS_string fmt;
 
          if ((cc == UNI_ASCII_SPACE) && fmt_seen)   // end of field
             {
-              col_formats.push_back(fmt);
-              fmt.clear();    // start a new field;
+              col_formats.append(fmt);
+              fmt.shrink(0);    // start a new field;
               fmt_seen = false;
               continue;   // next char
             }
@@ -327,8 +328,8 @@ UCS_string fmt;
               ++f;   // next char is right decorator (and end of field)
               if (f < format.size())   fmt.append(format[f]);
 
-              col_formats.push_back(fmt);
-              fmt.clear();    // start a new field;
+              col_formats.append(fmt);
+              fmt.shrink(0);    // start a new field;
               fmt_seen = false;
               continue;   // next char
             }
@@ -336,11 +337,11 @@ UCS_string fmt;
 
    if ((!fmt_seen) && (col_formats.size() > 0))
       {
-        col_formats.back().append(fmt);
+        col_formats.last().append(fmt);
       }
    else
       {
-        col_formats.push_back(fmt);
+        col_formats.append(fmt);
       }
 }
 //-----------------------------------------------------------------------------
@@ -664,7 +665,10 @@ char format[40];
 const int data_buf_len = int_part.out_len + 1        // 123.
                        + fract_part.out_len          // 456
                        + 1 + exponent.out_len + 1;   // E-22
-DynArray(char, data_buf, data_buf_len);
+
+   if (data_buf_len > 100)   DOMAIN_ERROR;
+
+char data_buf[101];
 char * fract_end = 0;
 
    if (exponent.size())
@@ -703,9 +707,9 @@ char * fract_end = 0;
 
         // the int part could be longer than allowed by the exaple string.
         //
-        const char * dot = strchr(data_buf.get_data(), '.');
-        const int ilen = dot ? (dot - data_buf.get_data())
-                             : strlen(data_buf.get_data());
+        const char * dot = strchr(data_buf, '.');
+        const int ilen = dot ? (dot - data_buf)
+                             : strlen(data_buf);
         if (ilen > int_part.out_len)
            {
              if (Workspace::get_FC(3) == UNI_ASCII_0)   DOMAIN_ERROR;
@@ -751,7 +755,7 @@ const int ilen = int_end - &data_buf[0];
    if (  data_int.size() == 1
       && data_int[0] == UNI_ASCII_0
       && int_part.min_len == 0
-      && fract_part.size())   data_int.clear();
+      && fract_part.size())   data_int.shrink(0);
 
    Log(LOG_Bif_F12_FORMAT)
       {
@@ -832,7 +836,7 @@ size_t d = data.size();
          else                             overflow = true;
       }
 
-   if ((flt_mask & BIT_9) && ucs.all_zeroes())   ucs.clear();
+   if ((flt_mask & BIT_9) && ucs.all_zeroes())   ucs.shrink(0);
 
    return ucs.reverse();
 }
@@ -862,7 +866,7 @@ int d = 0;
             }
       }
 
-   if ((flt_mask & BIT_9) && ucs.all_zeroes())   ucs.clear();
+   if ((flt_mask & BIT_9) && ucs.all_zeroes())   ucs.shrink(0);
 
    return ucs;
 }
@@ -1069,21 +1073,21 @@ UCS_string ret = UCS_string::from_double_fixed_prec(value, precision);
         if (ret[0] == UNI_ASCII_0    &&
             ret[1] == UNI_ASCII_FULLSTOP)        // 0.xxx → .xxx
            {
-             ret.remove_front();
+             ret.erase(0);
            }
         else if (ret[0] == UNI_OVERBAR     &&
                  ret[1] == UNI_ASCII_0     &&
                  ret[2] == UNI_ASCII_FULLSTOP)   //  ¯0.x → ¯.x
            {
              ret[1] = UNI_OVERBAR;
-             ret.remove_front();
+             ret.erase(0);
            }
       }
    else if (ret.size() == 2       && 
             ret[0] == UNI_OVERBAR &&
             ret[1] == UNI_ASCII_0)               // ¯0 → 0
            {
-             ret.remove_front();
+             ret.erase(0);
            }
 
    return ret;
