@@ -48,13 +48,13 @@ Shape shape_B = B->get_shape();
 const ShapeItem len_B = shape_B.get_shape_item(axis);
 ShapeItem len_A = A->element_count();
 ShapeItem len_Z = 0;
-vector<ShapeItem> rep_counts;
+Simple_string<ShapeItem, false> rep_counts;
    rep_counts.reserve(len_B);
    if (len_A == 1)   // single a -> a a ... a (len_B times)
       {
         len_A = len_B;
         APL_Integer rep_A = A->get_ravel(0).get_near_int();
-        loop(a, len_A)   rep_counts.push_back(rep_A);
+        loop(a, len_A)   rep_counts.append(rep_A);
         if (rep_A > 0)        len_Z =  rep_A*len_B;
         else if (rep_A < 0)   len_Z = -rep_A*len_B;
       }
@@ -64,7 +64,7 @@ vector<ShapeItem> rep_counts;
         loop(a, len_A)
            {
              APL_Integer rep_A = A->get_ravel(a).get_near_int();
-             rep_counts.push_back(rep_A);
+             rep_counts.append(rep_A);
              if (rep_A > 0)        { len_Z += rep_A;   ++geq_A; }
              else if (rep_A < 0)   len_Z -= rep_A;
              else                  ++geq_A;
