@@ -49,12 +49,12 @@ Shape shape_B = B->get_shape();
 
 const ShapeItem ec_A = A->element_count();
 ShapeItem ones_A = 0;
-vector<ShapeItem> rep_counts;
+Simple_string<ShapeItem, false> rep_counts;
    rep_counts.reserve(ec_A);
    loop(a, ec_A)
       {
         APL_Integer rep_A = A->get_ravel(a).get_near_int();
-        rep_counts.push_back(rep_A);
+        rep_counts.append(rep_A);
         if      (rep_A == 0)        ;
         else if (rep_A == 1)        ++ones_A;
         else                        DOMAIN_ERROR;
