@@ -142,20 +142,19 @@ void
 Bif_OPER2_OUTER::scalar_outer_product() const
 {
 #ifdef PERFORMANCE_COUNTERS_WANTED
-#ifdef HAVE_RDTSC
+# ifdef HAVE_RDTSC
 const uint64_t start_1 = cycle_counter();
-#endif
+# endif
 #endif
 
   // the empty cases have been handled already in eval_ALRB()
 
-const ShapeItem Z_len = job.ZAh * job.ZBl;
    job.ec = E_NO_ERROR;
 
 #if PARALLEL_ENABLED
    if (  Parallel::run_parallel
       && Thread_context::get_active_core_count() > 1
-      && Z_len > get_dyadic_threshold())
+      && job.ZAh * job.ZBl > get_dyadic_threshold())
       {
         job.cores = Thread_context::get_active_core_count();
         Thread_context::do_work = PF_scalar_outer_product;
@@ -173,7 +172,8 @@ const ShapeItem Z_len = job.ZAh * job.ZBl;
 #ifdef PERFORMANCE_COUNTERS_WANTED
 #ifdef HAVE_RDTSC
 const uint64_t end_1 = cycle_counter();
-   Performance::fs_OPER2_OUTER_AB.add_sample(end_1 - start_1, Z_len);
+   Performance::fs_OPER2_OUTER_AB.add_sample(end_1 - start_1,
+                                             job.ZAh * job.ZBl);
 #endif
 #endif
 }
