@@ -2,7 +2,7 @@
     This file is part of GNU APL, a free implementation of the
     ISO/IEC Standard 13751, "Programming Language APL, Extended"
 
-    Copyright (C) 2008-2015  Dr. Jürgen Sauermann
+    Copyright (C) 2008-2016  Dr. Jürgen Sauermann
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 
 #include "Bif_OPER2_RANK.hh"
 #include "IntCell.hh"
+#include "Macro.hh"
 #include "PointerCell.hh"
 #include "Workspace.hh"
 
@@ -37,10 +38,10 @@ Bif_OPER2_RANK::eval_LRB(Token & LO, Token & y, Value_P B)
    if (B->element_count() == 1 && B->get_ravel(0).is_pointer_cell())
       B = B->get_ravel(0).get_pointer_value();
 
-Rank rk_chunk_B = B->get_rank();
-   y123_to_B(y.get_apl_val(), rk_chunk_B);
+Rank rank_chunk_B = B->get_rank();
+   y123_to_B(y.get_apl_val(), rank_chunk_B);
 
-   return do_LyXB(LO.get_function(), 0, B, rk_chunk_B);
+   return do_LyXB(LO, Value_P(), B, rank_chunk_B);
 }
 //-----------------------------------------------------------------------------
 Token
@@ -49,86 +50,56 @@ Bif_OPER2_RANK::eval_LRXB(Token & LO, Token & y, Value_P X, Value_P B)
    if (B->element_count() == 1 && B->get_ravel(0).is_pointer_cell())
       B = B->get_ravel(0).get_pointer_value();
 
-Rank rk_chunk_B = B->get_rank();
+Rank rank_chunk_B = B->get_rank();
 
-   y123_to_B(y.get_apl_val(), rk_chunk_B);
+   y123_to_B(y.get_apl_val(), rank_chunk_B);
 
-Shape sh_X(X, Workspace::get_CT(), Workspace::get_IO());
-   return do_LyXB(LO.get_function(), &sh_X, B, rk_chunk_B);
+   return do_LyXB(LO, X, B, rank_chunk_B);
 }
 //-----------------------------------------------------------------------------
 Token
-Bif_OPER2_RANK::do_LyXB(Function * LO, const Shape * axes,
-                        Value_P B, Rank rk_chunk_B)
+Bif_OPER2_RANK::do_LyXB(Token & _LO, Value_P X, Value_P B, Rank rank_chunk_B)
 {
+Function * LO = _LO.get_function();
+   Assert(LO);
    if (!LO->has_result())   DOMAIN_ERROR;
 
-   // split shape of B into high and low shapes.
+   // split shape of B into high (=frame) and low (= chunk) shapes.
    //
-const Shape shape_Z = B->get_shape().high_shape(B->get_rank() - rk_chunk_B);
-Value_P Z(shape_Z, LOC);
-EOC_arg arg(Z, B);
-RANK & _arg = arg.u.u_RANK;
-
-   _arg.get_sh_chunk_B() = B->get_shape().low_shape(rk_chunk_B);
-   _arg.get_sh_frame() = shape_Z;
-   _arg.ec_frame   = shape_Z.get_volume();
-   if (!_arg.ec_frame)   _arg.ec_frame = 1;
-   _arg.ec_chunk_B = _arg.get_sh_chunk_B().get_volume();
-   _arg.cB = &B->get_ravel(0);
-   _arg.LO = LO;
-   _arg.axes_valid = (axes != 0);
-   if (axes)   _arg.get_axes() = *axes;
-   _arg.h = -1;
-
-   return finish_LyXB(arg, true);
-}
-//-----------------------------------------------------------------------------
-Token
-Bif_OPER2_RANK::finish_LyXB(EOC_arg & arg, bool first)
-{
-RANK & _arg = arg.u.u_RANK;
-
-   while (++_arg.h < _arg.ec_frame)
+const Shape shape_Z = B->get_shape().high_shape(B->get_rank() - rank_chunk_B);
+   if (shape_Z.is_empty())
       {
-        Value_P BB(_arg.get_sh_chunk_B(), LOC);
-        Assert1(_arg.ec_chunk_B == _arg.get_sh_chunk_B().get_volume());
-        loop(l, _arg.ec_chunk_B)
-            BB->next_ravel()->init(*_arg.cB++, BB.getref());
-        BB->check_value(LOC);
-
-        Token result = _arg.LO->eval_B(BB);
-        if (result.get_tag() == TOK_SI_PUSHED)
-           {
-             // LO was a user defined function or ⍎
-             //
-             if (first)   // first call
-                Workspace::SI_top()->add_eoc_handler(eoc_RANK, arg, LOC);
-             else           // subsequent call
-                Workspace::SI_top()->move_eoc_handler(eoc_RANK, &arg, LOC);
-
-             return result;   // continue in user defined function...
-           }
-
-        if (result.get_tag() == TOK_ERROR)   return result;
-
-        if (result.get_Class() == TC_VALUE)
-           {
-             Value_P ZZ = result.get_apl_val();
-             Cell * cZ = arg.Z->next_ravel();
-             if (cZ == 0)           cZ = &arg.Z->get_ravel(0);   // empty Z
-             if (ZZ->is_scalar())   cZ->init(ZZ->get_ravel(0), arg.Z.getref());
-             else                   new (cZ)   PointerCell(ZZ, arg.Z.getref());
-             continue;
-           }
-
-        Q1(result);   FIXME;
+        Value_P Fill_B = Bif_F12_TAKE::first(B);
+        Token tZ = LO->eval_fill_B(Fill_B);
+        Value_P Z = tZ.get_apl_val();
+        Z->set_shape(B->get_shape());
+        Z->check_value(LOC);
+        return Token(TOK_APL_VALUE1, Z);
       }
 
-   arg.Z->check_value(LOC);
-   if (!_arg.axes_valid)   return Bif_F12_PICK::disclose(arg.Z, true);
+const Shape shape_B = B->get_shape().low_shape(rank_chunk_B);
 
-   return Bif_F12_PICK::disclose_with_axis(_arg.get_axes(), arg.Z, true);
+Value_P vsh_B(shape_B.get_rank(), LOC);
+   new (&vsh_B->get_ravel(0)) IntCell(0);   // prototype
+   loop(sh, shape_B.get_rank())
+            new (vsh_B->next_ravel()) IntCell(shape_B.get_shape_item(sh));
+   vsh_B->check_value(LOC);
+
+Value_P vsh_Z(shape_Z.get_rank(), LOC);
+   loop(sh, shape_Z.get_rank())
+            new (vsh_Z->next_ravel()) IntCell(shape_Z.get_shape_item(sh));
+   vsh_Z->check_value(LOC);
+
+Value_P X5(5, LOC);
+   if (!X)   new (X5->next_ravel())   IntCell(-1);                // no X
+   else      new (X5->next_ravel())   PointerCell(X, X5.getref());   // X
+
+   new (X5->next_ravel())   IntCell(shape_B.get_volume());        // LB
+   new (X5->next_ravel())   PointerCell(vsh_B, X5.getref());      // rho_B
+   new (X5->next_ravel())   IntCell(shape_Z.get_volume());        // N_max
+   new (X5->next_ravel())   PointerCell(vsh_Z, X5.getref());      // rho_Z
+   X5->check_value(LOC);
+   return Macro::Z__LO_RANK_X5_B->eval_LXB(_LO, X5, B);
 }
 //-----------------------------------------------------------------------------
 Token
@@ -137,11 +108,11 @@ Bif_OPER2_RANK::eval_ALRB(Value_P A, Token & LO, Token & y, Value_P B)
    if (B->element_count() == 1 && B->get_ravel(0).is_pointer_cell())
       B = B->get_ravel(0).get_pointer_value();
 
-Rank rk_chunk_A = A->get_rank();
-Rank rk_chunk_B = B->get_rank();
-   y123_to_AB(y.get_apl_val(), rk_chunk_A, rk_chunk_B);
+Rank rank_chunk_A = A->get_rank();
+Rank rank_chunk_B = B->get_rank();
+   y123_to_AB(y.get_apl_val(), rank_chunk_A, rank_chunk_B);
 
-   return do_ALyXB(A, rk_chunk_A, LO.get_function(), 0, B, rk_chunk_B);
+   return do_ALyXB(A, rank_chunk_A, LO, Value_P(), B, rank_chunk_B);
 }
 //-----------------------------------------------------------------------------
 Token
@@ -151,23 +122,24 @@ Bif_OPER2_RANK::eval_ALRXB(Value_P A, Token & LO, Token & y,
    if (B->element_count() == 1 && B->get_ravel(0).is_pointer_cell())
       B = B->get_ravel(0).get_pointer_value();
 
-Rank rk_chunk_A = A->get_rank();
-Rank rk_chunk_B = B->get_rank();
+Rank rank_chunk_A = A->get_rank();
+Rank rank_chunk_B = B->get_rank();
 
-   y123_to_AB(y.get_apl_val(), rk_chunk_A, rk_chunk_B);
+   y123_to_AB(y.get_apl_val(), rank_chunk_A, rank_chunk_B);
 
-Shape sh_X(X, Workspace::get_CT(), Workspace::get_IO());
-   return do_ALyXB(A, rk_chunk_A, LO.get_function(), &sh_X, B, rk_chunk_B);
+   return do_ALyXB(A, rank_chunk_A, LO, X, B, rank_chunk_B);
 }
 //-----------------------------------------------------------------------------
 Token
-Bif_OPER2_RANK::do_ALyXB(Value_P A, Rank rk_chunk_A, Function *  LO,
-                         const Shape * axes, Value_P B, Rank rk_chunk_B)
+Bif_OPER2_RANK::do_ALyXB(Value_P A, Rank rank_chunk_A, Token & _LO,
+                         Value_P X, Value_P B, Rank rank_chunk_B)
 {
+Function * LO = _LO.get_function();
+   Assert(LO);
    if (!LO->has_result())   DOMAIN_ERROR;
 
-Rank rk_A_frame = A->get_rank() - rk_chunk_A;   // rk_A_frame is y8
-Rank rk_B_frame = B->get_rank() - rk_chunk_B;   // rk_B_frame is y9
+Rank rk_A_frame = A->get_rank() - rank_chunk_A;   // rk_A_frame is y8
+Rank rk_B_frame = B->get_rank() - rank_chunk_B;   // rk_B_frame is y9
 
    // if both high-ranks are 0, then return A LO B.
    //
@@ -177,155 +149,54 @@ Rank rk_B_frame = B->get_rank() - rk_chunk_B;   // rk_B_frame is y9
    // Even though A and B have the same shape, rk_A_frame and rk_B_frame
    // could be different, leading to different split shapes for A and B
    //
-bool repeat_A = false;
-bool repeat_B = false;
-Shape sh_A_frame = A->get_shape().high_shape(A->get_rank() - rk_chunk_A);
-const Shape sh_B_frame = B->get_shape().high_shape(B->get_rank() - rk_chunk_B);
-Shape shape_Z;
-   if (rk_A_frame == 0)   // "conform" A to B
+const Shape shape_Z = rk_B_frame ? B->get_shape().high_shape(rk_B_frame)
+                                 : A->get_shape().high_shape(rk_A_frame);
+
+   if (rk_A_frame && rk_B_frame)   // A and B frames non-scalar
       {
-        shape_Z = sh_B_frame;
-        repeat_A = true;
-      }
-   else if (rk_B_frame == 0)   // "conform" B to A
-      {
-        shape_Z = sh_A_frame;
-        repeat_B = true;
-      }
-   else
-      {
-        if (rk_A_frame != rk_B_frame)   RANK_ERROR;
-        if (sh_A_frame != sh_B_frame)   LENGTH_ERROR;
-        shape_Z = sh_B_frame;
+        if (rk_A_frame != rk_B_frame)                           RANK_ERROR;
+        if (shape_Z != A->get_shape().high_shape(rk_A_frame))   LENGTH_ERROR;
       }
 
-Value_P Z(shape_Z, LOC);
-EOC_arg arg(Z, B, A);
-RANK & _arg = arg.u.u_RANK;
-
-   _arg.get_sh_chunk_A() = A->get_shape().low_shape(rk_chunk_A);
-   _arg.get_sh_chunk_B() = B->get_shape().low_shape(rk_chunk_B);
-   _arg.get_sh_frame() = B->get_shape().high_shape(B->get_rank() - rk_chunk_B);
-
-   // a trick to avoid conforming A to B or B to A. If A or B
-   // needs to be conformed, then we set the corresponding repeat_A or 
-   // repeat_B to true and copy the same A or B again and again
-   //
-   _arg.repeat_A = repeat_A;
-   _arg.repeat_B = repeat_B;
-
-   if (_arg.repeat_A)   // "conform" A to B
+   if (shape_Z.is_empty())
       {
-         rk_A_frame = rk_B_frame;
-         sh_A_frame = _arg.get_sh_frame();
-      }
-   else if (_arg.repeat_B)   // "conform" B to A
-      {
-         rk_B_frame = rk_A_frame;
-         _arg.get_sh_frame() = sh_A_frame;
-      }
- 
-   // at this point sh_A_frame == sh_B_frame, so we can use either of them
-   // to compute ec_chunk_A
-   //
-   _arg.ec_frame = _arg.get_sh_frame().get_volume();
-   if (!_arg.ec_frame)   _arg.ec_frame = 1;
-   _arg.ec_chunk_A = _arg.get_sh_chunk_A().get_volume();
-   _arg.ec_chunk_B = _arg.get_sh_chunk_B().get_volume();
+        Value_P Fill_A = Bif_F12_TAKE::first(A);
+        Value_P Fill_B = Bif_F12_TAKE::first(B);
+        Shape shape_Z;
 
-   _arg.cA = &A->get_ravel(0);
-   _arg.LO = LO;
-   _arg.axes_valid = (axes != 0);
-   if (axes)   _arg.get_axes() = *axes;
-   _arg.cB = &B->get_ravel(0);
-   _arg.h = -1;
+        if (A->is_empty())          shape_Z = A->get_shape();
+        else if (!A->is_scalar())   DOMAIN_ERROR;
 
-   return finish_ALyXB(arg, true);
-}
-//-----------------------------------------------------------------------------
-Token
-Bif_OPER2_RANK::finish_ALyXB(EOC_arg & arg, bool first)
-{
-RANK & _arg = arg.u.u_RANK;
+        if (B->is_empty())          shape_Z = B->get_shape();
+        else if (!B->is_scalar())   DOMAIN_ERROR;
 
-   while (++_arg.h < _arg.ec_frame)
-      {
-        Value_P AA(_arg.get_sh_chunk_A(), LOC);
-        Value_P BB(_arg.get_sh_chunk_B(), LOC);
+        Value_P Z1 = LO->eval_fill_AB(Fill_A, Fill_B).get_apl_val();
 
-        loop(l, _arg.ec_chunk_A)  AA->get_ravel(l).init(*_arg.cA++,AA.getref());
-        loop(l, _arg.ec_chunk_B)  BB->get_ravel(l).init(*_arg.cB++,BB.getref());
-
-        if (_arg.repeat_A)   _arg.cA = &arg.A->get_ravel(0);
-        if (_arg.repeat_B)   _arg.cB = &arg.B->get_ravel(0);
-
-        AA->check_value(LOC);
-        BB->check_value(LOC);
-
-        Token result = _arg.LO->eval_AB(AA, BB);
-        if (result.get_tag() == TOK_SI_PUSHED)
-           {
-             // LO was a user defined function or ⍎
-             //
-             if (first)   // first call
-                Workspace::SI_top()->add_eoc_handler(eoc_RANK, arg, LOC);
-             else           // subsequent call
-                Workspace::SI_top()->move_eoc_handler(eoc_RANK, &arg, LOC);
-
-             return result;   // continue in user defined function...
-           }
-
-        if (result.get_tag() == TOK_ERROR)   return result;
-
-        if (result.get_Class() == TC_VALUE)
-           {
-             Value_P ZZ = result.get_apl_val();
-             Cell * cZ = arg.Z->is_empty() ? &arg.Z->get_ravel(0)
-                                           : arg.Z->next_ravel();
-             if (ZZ->is_scalar())   cZ->init(ZZ->get_ravel(0), arg.Z.getref());
-             else                   new (cZ) PointerCell(ZZ, arg.Z.getref());
-
-             continue;
-          }
-
-        Q1(result);   FIXME;
+        Value_P Z(shape_Z, LOC);
+        new (&Z->get_ravel(0)) PointerCell(Z1, Z.getref());
+        Z->check_value(LOC);
+        return Token(TOK_APL_VALUE1, Z);
       }
 
-   arg.Z->check_value(LOC);
-   if (!_arg.axes_valid)   return Bif_F12_PICK::disclose(arg.Z, true);
+const Shape low_A = A->get_shape().low_shape(rank_chunk_A);
+const Shape low_B = B->get_shape().low_shape(rank_chunk_B);
 
-   return Bif_F12_PICK::disclose_with_axis(_arg.get_axes(), arg.Z, true);
-}
-//-----------------------------------------------------------------------------
-bool
-Bif_OPER2_RANK::eoc_RANK(Token & token)
-{
-EOC_arg * arg = Workspace::SI_top()->remove_eoc_handlers();
-EOC_arg * next = arg->next;
-RANK & _arg = arg->u.u_RANK;
+Value_P vsh_A(LOC, &low_A);
+Value_P vsh_B(LOC, &low_B);
+Value_P vsh_Z(LOC, &shape_Z);
 
-   if (token.get_Class() != TC_VALUE)  return false;   // stop it
+Value_P X7(7, LOC);
+   if (!X)   new (X7->next_ravel())   IntCell(-1);                // no X
+   else      new (X7->next_ravel())   PointerCell(X, X7.getref());   // X
 
-   // the user defined function has returned a value. Store it.
-   //
-Value_P ZZ = token.get_apl_val();
-   if (ZZ->is_scalar())
-      arg->Z->next_ravel()->init(ZZ->get_ravel(0), arg->Z.getref());
-   else
-      new (arg->Z->next_ravel())   PointerCell(ZZ, arg->Z.getref());
-
-   if (_arg.h < (_arg.ec_frame - 1))   Workspace::pop_SI(LOC);
-
-   if (!arg->A)   copy_1(token, finish_LyXB (*arg, false), LOC);
-   else           copy_1(token, finish_ALyXB(*arg, false), LOC);
-
-   if (token.get_tag() == TOK_SI_PUSHED)   return true;   // continue
-
-   delete arg;
-   Workspace::SI_top()->set_eoc_handlers(next);
-   if (next)   return next->handler(token);
-
-   return false;   // stop it
+   new (X7->next_ravel())   IntCell(low_A.get_volume());        // LA
+   new (X7->next_ravel())   PointerCell(vsh_A, X7.getref());      // rho_A
+   new (X7->next_ravel())   IntCell(low_B.get_volume());        // LB
+   new (X7->next_ravel())   PointerCell(vsh_B, X7.getref());      // rho_B
+   new (X7->next_ravel())   IntCell(shape_Z.get_volume());        // N_max
+   new (X7->next_ravel())   PointerCell(vsh_Z, X7.getref());      // rho_Z
+   X7->check_value(LOC);
+   return Macro::Z__A_LO_RANK_X7_B->eval_ALXB(A, _LO, X7, B);
 }
 //-----------------------------------------------------------------------------
 void
@@ -365,7 +236,7 @@ const Rank rk_B = rank_B;
                          y123->get_ravel(1).get_near_int();
                          y123->get_ravel(2).get_near_int();   break;
 
-             default: LENGTH_ERROR;
+        default: LENGTH_ERROR;
       }
 
    // 3. adjust rank_B if they exceed its initial value or
@@ -480,14 +351,15 @@ const ShapeItem length = y123_B->element_count();
               else
                  {
                    B = Value_P(LOC);
-                   B->next_ravel()->init(B0, B.getref());
+                   B->next_ravel()->init(B0, B.getref(), LOC);
                  }
             }
          else                    // vector B
             {
               B = Value_P(length - 1, LOC);
               loop(l, length - 1)
-                  B->next_ravel()->init(y123_B->get_ravel(l + 1), B.getref());
+                  B->next_ravel()->init(y123_B->get_ravel(l + 1),
+                                                          B.getref(), LOC);
             }
          return;
       }
@@ -517,7 +389,7 @@ int y123_len = 0;
       {
         y123 = Value_P(y123_len, LOC);
         loop(yy, y123_len)
-            y123->next_ravel()->init(y123_B->get_ravel(yy), y123.getref());
+            y123->next_ravel()->init(y123_B->get_ravel(yy), y123.getref(), LOC);
         B = y123_B->get_ravel(y123_len).get_pointer_value();
         return;
       }
@@ -526,10 +398,10 @@ int y123_len = 0;
    //
    y123 = Value_P(y123_len, LOC);
    loop(yy, y123_len)
-       y123->next_ravel()->init(y123_B->get_ravel(yy), y123.getref());
+       y123->next_ravel()->init(y123_B->get_ravel(yy), y123.getref(), LOC);
 
    B = Value_P(length - y123_len, LOC);
    loop(bb, (length - y123_len))
-       B->next_ravel()->init(y123_B->get_ravel(bb + y123_len), B.getref());
+       B->next_ravel()->init(y123_B->get_ravel(bb + y123_len), B.getref(), LOC);
 }
 //-----------------------------------------------------------------------------
