@@ -2,7 +2,7 @@
     This file is part of GNU APL, a free implementation of the
     ISO/IEC Standard 13751, "Programming Language APL, Extended"
 
-    Copyright (C) 2008-2015  Dr. Jürgen Sauermann
+    Copyright (C) 2008-2016  Dr. Jürgen Sauermann
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@ CDR::to_CDR(CDR_string & result, const Value & value)
 const CDR_type type = value.get_CDR_type();
 const int len = value.total_size_brutto(type);
    result.reserve(len + 1);
-   result.clear();
+   result.shrink(0);
 
    fill(result, type, len, value);
 }
@@ -140,7 +140,8 @@ const uint32_t nelm = val.element_count();
         loop(e, nelm)
            {
              const double v = val.get_ravel(e).get_real_value();
-             uint64_t i = *(uint64_t *)&v;
+             const double * dv = &v;
+             uint64_t i = *(const uint64_t *)dv;
              result.append(Unicode(i & 0xFF));   i >>= 8;
              result.append(Unicode(i & 0xFF));   i >>= 8;
              result.append(Unicode(i & 0xFF));   i >>= 8;
@@ -156,7 +157,8 @@ const uint32_t nelm = val.element_count();
         loop(e, nelm)
            {
              double v = val.get_ravel(e).get_real_value();
-             uint64_t i = *(uint64_t *)&v;
+             const double * dv = &v;
+             uint64_t i = *(uint64_t *)dv;
              result.append(Unicode(i & 0xFF));   i >>= 8;
              result.append(Unicode(i & 0xFF));   i >>= 8;
              result.append(Unicode(i & 0xFF));   i >>= 8;
@@ -167,7 +169,8 @@ const uint32_t nelm = val.element_count();
              result.append(Unicode(i & 0xFF));   i >>= 8;
 
              v = val.get_ravel(e).get_imag_value();
-             i = *(uint64_t *)&v;
+             dv = &v;
+             i = *(uint64_t *)dv;
              result.append(Unicode(i & 0xFF));   i >>= 8;
              result.append(Unicode(i & 0xFF));   i >>= 8;
              result.append(Unicode(i & 0xFF));   i >>= 8;
