@@ -54,15 +54,15 @@ Error::print(ostream & out) const
              out << endl;
            }
 
-        if (parser_loc)   out << "   Parser LOC: " << parser_loc << endl;
-        if (print_loc)    out << "   Print LOC:  " << print_loc  << endl;
+        if (parser_loc)   out << "   Parser LOC: " << parser_loc  << endl;
+        if (print_loc)    out << "   Print LOC:  " << print_loc   << endl;
 
         if (symbol_name.size())
-                          out << "   Symbol:     " << symbol_name << endl;
+           out                << "   Symbol:     " << symbol_name << endl;
 
-                          out << "   Thrown at:  " << throw_loc << endl
-                              << "--------------------------"   << endl
-                              << endl;
+        out <<                   "   Thrown at:  " << throw_loc   << endl
+            <<                   "--------------------------"     << endl
+                                                                  << endl;
       }
 }
 //-----------------------------------------------------------------------------
@@ -109,8 +109,8 @@ Error::init(ErrorCode ec, const char * loc)
    throw_loc = loc;
    error_message_1 = error_name(error_code);
    if (Workspace::more_error().size())   error_message_1.append(UNI_ASCII_PLUS);
-   error_message_2.clear();
-   symbol_name.clear();
+   error_message_2.shrink(0);
+   symbol_name.shrink(0);
    parser_loc = 0;
    show_locked = false;
    left_caret = -1;
@@ -196,7 +196,7 @@ throw_tokenize_error(ErrorCode code, const UCS_string & more_error_info)
            << "throwing " << Error::error_name(code)
            << " in  Tokenizer" << endl;
 
-   Workspace::more_error() = more_error_info;
+   MORE_ERROR() << more_error_info;
 
 Error error(code, LOC);
 
@@ -212,7 +212,7 @@ throw_parse_error(ErrorCode code, const char * par_loc, const char *loc)
 
    Log(LOG_verbose_error)   Backtrace::show(__FILE__, __LINE__);
 
-   Workspace::more_error() = Error::error_name(code);
+   MORE_ERROR() << Error::error_name(code);
 
 Error error(code, loc);
    error.parser_loc = par_loc;
