@@ -48,17 +48,17 @@ Executable::Executable(const UCS_string & ucs,  bool multi_line,
              switch (uni)
                 {
                   case UNI_ASCII_CR:                         break;
-                  case UNI_ASCII_LF: text.push_back(line);
-                                     line.clear();           break;
+                  case UNI_ASCII_LF: text.append(line);
+                                     line.shrink(0);         break;
                   default:           line.append(uni);
                }
            }
 
-        if (line.size())   text.push_back(line);
+        if (line.size())   text.append(line);
       }
    else
       {
-        text.push_back(ucs);
+        text.append(ucs);
       }
 }
 //-----------------------------------------------------------------------------
@@ -87,17 +87,17 @@ ShapeItem last_semi = -1;
       {
         for (ShapeItem t = last_semi; t < lambda_text.size(); ++t)
             header.append(lambda_text[t]);
-        text.push_back(header);
+        text.append(header);
 
         UCS_string new_body(lambda_text);
         new_body.shrink(last_semi);
         new_body.remove_trailing_whitespaces();
-        text.push_back(new_body);
+        text.append(new_body);
       }
    else
       {
-        text.push_back(header);
-        text.push_back(lambda_text);
+        text.append(header);
+        text.append(lambda_text);
      }
 }
 //-----------------------------------------------------------------------------
@@ -106,7 +106,7 @@ Executable::~Executable()
    Log(LOG_UserFunction__fix)
       {
         CERR << "deleting Executable " << (const void *)this
-             << " (body size=" <<  body.size() << ")" << endl;
+             << " (body size=" << body.size() << ")" << endl;
       }
 
    clear_body();
@@ -121,13 +121,14 @@ Executable::clear_body()
 
         if (body[b].is_function())
            {
-              Function * fun = body[b].get_function();
-              UserFunction * ufun = fun->get_ufun1();
-              if (ufun && ufun->is_lambda())   ufun->decrement_refcount(LOC);
+             Function * fun = body[b].get_function();
+             UserFunction * ufun = fun->get_ufun1();
+             if (ufun && ufun->is_lambda())   ufun->decrement_refcount(LOC);
+             new (&body[b]) Token();
            }
       }
 
-   body.clear();   // resize to 0
+   body.shrink(0);
 }
 //-----------------------------------------------------------------------------
 ErrorCode
@@ -161,7 +162,7 @@ ErrorCode
 Executable::parse_body_line(Function_Line line, const Token_string & in,
                             bool trace,  bool tolerant, const char * loc)
 {
-Source<Token> src(in);
+Source<Token, true> src(in);
 
    // handle labels (if any)
    //
@@ -309,7 +310,7 @@ int tidx = 0;
    while (stats_before)
       {
         Assert(tidx < line_txt.size());
-        if (line_txt[tidx++] == UNI_DIAMOND)   --stats_before;
+        if (Avec::is_diamond(line_txt[tidx++]))   --stats_before;
       }
 
    // skip leading spaces
@@ -319,7 +320,7 @@ int tidx = 0;
 UCS_string ret;
    while (tidx < line_txt.size())
       {
-        if (line_txt[tidx] == UNI_DIAMOND)    break;
+        if (Avec::is_diamond(line_txt[tidx]))    break;
         ret.append(line_txt[tidx++], LOC);
       }
 
@@ -591,7 +592,7 @@ UserFunction * ufun = new UserFunction(signature, lambda_num,
    // A←{ ... }   by:
    // A←UFUN      UFUN being a user-defined function with body { ... }
    //
-   move_2(body[bend], ufun->get_token(), LOC);
+   body[bend].move_2(ufun->get_token(), LOC);
 
    return bend + 1;
 }
@@ -606,7 +607,7 @@ int level = 0;
    for (; b < bend; ++b)
        {
          Token t;
-         move_1(t, body[b], LOC);
+         t.move_1(body[b], LOC);
          body[b].clear(LOC);   // invalidate in main body
 
          // figure the signature by looking for ⍺, ⍶, ⍵, ⍹, and χ
@@ -642,7 +643,15 @@ int level = 0;
          lambda_body.append(t, LOC);
        }
 
-   // if the lambda has at least one token, then it (is supposed to) return λ.
+   if ((signature & SIG_B) == 0 &&   // niladic
+       (signature & (SIG_A | SIG_LO | SIG_RO | SIG_X)))
+      {
+        MORE_ERROR() <<
+           "niladic lambda with axis. left argument, or function argument(s)";
+        DEFN_ERROR;
+      }
+
+   // if the lambda has at least one token, then it is supposedly returns λ.
    // Otherwise the lambda is empty (and result-less)
    //
    if (signature & SIG_Z)
@@ -670,8 +679,8 @@ UCS_string lambda_text;
 
 int level = 0;   // {/} nesting level
 bool copying = false;
-size_t tidx = 0;   // the current line in text[]
-int tcol = 0;      // the current column in text[tidx];
+int tidx = 0;    // the current line in text[]
+int tcol = 0;    // the current column in text[tidx];
 bool in_single_quotes = false;
 bool in_double_quotes = false;
 
@@ -681,7 +690,7 @@ bool in_double_quotes = false;
        {
          if (tidx >= text.size())
             {
-              Q1(copying)   Q1(tidx)   Q(skip)   FIXME;
+              Q1(copying)   Q1(tidx)   Q1(skip)   FIXME;
             }
 
          const UCS_string & line = text[tidx];
@@ -806,15 +815,16 @@ Executable::increment_refcount(const char * loc)
 void
 Executable::decrement_refcount(const char * loc)
 {
-   Assert1(get_ufun());
-   Assert(get_ufun()->is_lambda());
+UserFunction * ufun = get_ufun();
+   Assert1(ufun);
+   Assert(ufun->is_lambda());
 
    if (refcount <= 0)
       {
         CERR << "*** Warning: refcount of " << get_name() << " is " << refcount
              << ":" << endl;
         print_text(CERR);
-//     FIXME;
+        FIXME;
       }
 
    --refcount;
@@ -826,7 +836,8 @@ Executable::decrement_refcount(const char * loc)
    if (refcount <= 0)
       {
 //      CERR << "*** lambda died" << endl;
-        clear_body();
+//      clear_body();
+        delete ufun;
       }
 }
 //=============================================================================
