@@ -2,7 +2,7 @@
     This file is part of GNU APL, a free implementation of the
     ISO/IEC Standard 13751, "Programming Language APL, Extended"
 
-    Copyright (C) 2008-2015  Dr. Jürgen Sauermann
+    Copyright (C) 2008-2016  Dr. Jürgen Sauermann
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -35,7 +35,9 @@
 #include "PrimitiveFunction.hh"
 #include "PrintOperator.hh"
 #include "QuadFunction.hh"
+#include "Quad_DLX.hh"
 #include "Quad_FX.hh"
+#include "Quad_SQL.hh"
 #include "Quad_SVx.hh"
 #include "Quad_TF.hh"
 #include "ScalarFunction.hh"
@@ -43,29 +45,68 @@
 #include "Workspace.hh"
 
 //-----------------------------------------------------------------------------
-const char *
-ID::name(ID::Id id)
+/// an Id and how it looks like in APL
+struct Id_name
 {
-   switch(id)
-      {
-#define pp(i, _u, _v) case i:          return #i;
-#define qf(i,  u, _v)  case Quad_ ## i: return u;
-#define qv(i,  u, _v)  case Quad_ ## i: return u;
-#define sf(i,  u, _v)  case i:          return u;
-#define st(i,  u, _v)  case i:          return u;
+  /// compare \b key with \b item (for bsearch())
+  static int compare(const void * key, const void * item)
+     {
+       return *(const ID::Id *)key - ((const Id_name *)item)->id;
+     }
+
+  /// the ID
+  ID::Id id;
+
+   /// how \b id is being printed
+  const UCS_string * ucs_name;
+};
+
+static Id_name id2ucs[] =
+{
+#define pp(i, _u, _v) { ID::i, 0}, 
+#define qf(i,  _u, _v) {ID::Quad_ ## i, 0}, 
+#define qv(i,  _u, _v) {ID::Quad_ ## i, 0}, 
+#define sf(i,  _u, _v) {ID::i, 0}, 
+#define st(i,  _u, _v) {ID::i, 0},
 
 #include "Id.def"
+};
 
-        default: break;
-      }
+const UCS_string &
+ID::get_name(Id id)
+{
+const void * result =
+    bsearch(&id, id2ucs, sizeof(id2ucs) / sizeof(Id_name),
+            sizeof(Id_name), Id_name::compare);
 
-   return "Unknown-ID";
+   Assert(result);
+Id_name * idn = (Id_name *) result;
+   if (const UCS_string * ucs = idn->ucs_name)   return *ucs; 
+
+   // the name was not yet constructed. Do it now
+   //
+const char * name = "unknown ID";
+   switch(id)
+       {
+#define pp(i, _u, _v) case ID::i:          name = #i;   break;
+#define qf(i,  u, _v) case ID::Quad_ ## i: name = u;   break;
+#define qv(i,  u, _v) case ID::Quad_ ## i: name = u;   break;
+#define sf(i,  u, _v) case ID::i:          name = u;   break;
+#define st(i,  u, _v) case ID::i:          name = u;   break;
+
+#include "Id.def"
+       }
+
+UTF8_string utf(name);
+UCS_string * ucs = new UCS_string(utf);
+   idn->ucs_name = ucs;
+   return *ucs;
 }
 //-----------------------------------------------------------------------------
 ostream &
 operator << (ostream & out, ID::Id id)
 {
-   return out << ID::name(id);
+   return out << ID::get_name(id);
 }
 //-----------------------------------------------------------------------------
 Function *
