@@ -195,7 +195,7 @@ ofstream outf(filename);
       }
 
 int count = 0;
-   for (size_t p = put + 1; p < hist_lines.size(); ++p)
+   for (int p = put + 1; p < hist_lines.size(); ++p)
       {
         outf << hist_lines[p] << endl;
         ++count;
@@ -215,7 +215,7 @@ LineHistory::clear_history(ostream & out)
 {
    current_line = 0;
    put = 0;
-   hist_lines.resize(0);
+   hist_lines.shrink(0);
 UCS_string u("xxx");
    add_line(u);
 }
@@ -223,7 +223,7 @@ UCS_string u("xxx");
 void
 LineHistory::print_history(ostream & out)
 {
-   for (size_t p = put + 1; p < hist_lines.size(); ++p)
+   for (int p = put + 1; p < hist_lines.size(); ++p)
       {
         out << "      " << hist_lines[p] << endl;
       }
@@ -240,7 +240,7 @@ LineHistory::add_line(const UCS_string & line)
 
    if ((int)hist_lines.size() < max_lines)   // append
       {
-        hist_lines.push_back(line);
+        hist_lines.append(line);
         put = 0;
       }
    else                            // override
@@ -256,7 +256,7 @@ void
 LineHistory::replace_line(const UCS_string & line)
 {
    if (put > 0)   hist_lines[put - 1] = line;
-   else           hist_lines.back() = line;
+   else           hist_lines.last() = line;
 }
 //-----------------------------------------------------------------------------
 const UCS_string *
@@ -411,7 +411,7 @@ LineEditContext::delete_char()
       }
    else
       {
-        user_line.erase(uidx, 1);
+        user_line.erase(uidx);
         refresh_from_cursor();
       }
 
@@ -430,7 +430,7 @@ LineEditContext::insert_char(Unicode uni)
       }
    else if (ins_mode)              // insert char
       {
-        user_line.insert(uidx, 1, uni);
+        user_line.insert_before(uidx, uni);
         adjust_allocated_height();
         refresh_wrapped_cursor();
         refresh_from_cursor();
@@ -504,8 +504,9 @@ LineEditContext::tab_expansion(LineInputMode mode)
 
 int replace_count = 0;
 UCS_string line = user_line;
+const ExpandResult expand_result = Command::expand_tab(line, replace_count);
 
-   switch(Command::expand_tab(line, replace_count))
+   switch(expand_result)
       {
         case ER_IGNORE: return;
 
@@ -601,8 +602,7 @@ LineInput::LineInput(bool do_read_history)
    //
    current_termios.c_iflag &= ~( ISTRIP | // don't strip off bit 8
                                  INLCR  | // don't NL → CR
-                                 IGNCR  | // don't ignore CR
-                                 IXON);   // don't enable XON/XOFF on output
+                                 IGNCR);  // don't ignore CR
    current_termios.c_iflag |=    IGNBRK | // ignore break
                                  IGNPAR |
                                  ICRNL  ; // CR → NL
@@ -678,7 +678,7 @@ InputMux::get_line(LineInputMode mode, const UCS_string & prompt,
                               file_line.size() &&
                               file_line[0] == UNI_ASCII_BS)
                              {
-                               file_line.drop_leading(1);
+                               file_line.erase(0);
                                line.pop();
                              }
                        line.append_utf8(file_line.c_str());
@@ -830,7 +830,7 @@ LineInput::edit_line(LineInputMode mode, const UCS_string & prompt,
    tcsetattr(STDIN_FILENO, TCSANOW, &the_line_input->current_termios);
 #endif // WANT_LIBAPL
 
-   user_line.clear();
+   user_line.shrink(0);
 
 LineEditContext lec(mode, 24, Workspace::get_PW(), hist, prompt);
 
@@ -962,7 +962,16 @@ LineInput::get_uni()
 again:
 
 const int b0 = fgetc(stdin);
-   if (b0 == EOF)   return UNI_EOF;
+   if (b0 == EOF)
+      {
+       if (got_WINCH)
+          {
+            got_WINCH = false;
+            goto again;
+          }
+
+         return UNI_EOF;
+      }
 
    if (b0 & 0x80)   // non-ASCII unicode
       {
