@@ -55,7 +55,7 @@ Nabla::Nabla(const UCS_string & cmd)
      current_line(1),
      first_command(cmd)
 {
-   Workspace::more_error().clear();
+   Workspace::more_error().shrink(0);
 }
 //-----------------------------------------------------------------------------
 void
@@ -68,7 +68,7 @@ Nabla::throw_edit_error(const char * why)
 
    if (Workspace::more_error().size() == 0)
       {
-        Workspace::more_error() = UCS_string(why);
+        MORE_ERROR() << why;
       }
 
    throw_define_error(fun_header, first_command, why);
@@ -158,10 +158,9 @@ UCS_string fun_text;
           line_0.append(lines[0].text);
           line_0.remove_trailing_whitespaces();
           LineInput::replace_history_line(line_0);
-
         }
 
-        for (size_t l = 1; l < lines.size(); ++l)
+        for (int l = 1; l < lines.size(); ++l)
             {
               UCS_string line_l("[");
               line_l.append_number(l);
@@ -196,18 +195,16 @@ UserFunction * ufun = UserFunction::fix(fun_text, error_line, false,
 
    // set stop and trace vectors
    //
-DynArray(Function_Line, stop_vec,  lines.size());
-DynArray(Function_Line, trace_vec, lines.size());
-int stop_count = 0;
-int trace_count = 0;
+Simple_string<Function_Line, false> stop_vec;
+Simple_string<Function_Line, false> trace_vec;
    loop(l, lines.size())
        {
-         if (lines[l].stop_flag)    stop_vec [stop_count++]  = Function_Line(l);
-         if (lines[l].trace_flag)   trace_vec[trace_count++] = Function_Line(l);
+         if (lines[l].stop_flag)    stop_vec.append(Function_Line(l));
+         if (lines[l].trace_flag)   trace_vec.append(Function_Line(l));
        }
 
-   ufun->set_trace_stop(stop_vec.get_data(),  stop_count,  true);
-   ufun->set_trace_stop(trace_vec.get_data(), trace_count, false);
+   ufun->set_trace_stop(stop_vec,  true);
+   ufun->set_trace_stop(trace_vec, false);
 }
 //-----------------------------------------------------------------------------
 const char *
@@ -371,7 +368,7 @@ Nabla::parse_oper(UCS_string & oper, bool initial)
         while (oper.size() > 0 && oper.last() <= ' ')   oper.pop();
       }
 
-   current_text.clear();
+   current_text.shrink(0);
    ecmd = ECMD_NOP;
 
    if (oper.size() == 0 && do_close)   return 0;
@@ -572,7 +569,7 @@ Nabla::open_new_function()
       UERR << "creating new function '" << fun_symbol->get_name() 
            << "' with header '" << fun_header << "'" << endl;
 
-   lines.push_back(FunLine(0, fun_header));
+   lines.append(FunLine(0, fun_header));
    return 0;
 }
 //-----------------------------------------------------------------------------
@@ -585,7 +582,8 @@ Nabla::open_existing_function()
 
    // this function must only be called when editing functions interactively
    //
-   Assert(!InputFile::running_script());
+   if (InputFile::running_script())
+      return "∇-edit existing function from a script";
 
 Function * function = fun_symbol->get_function();
    Assert(function);
@@ -600,7 +598,7 @@ Function * function = fun_symbol->get_function();
       return "function is a lambda";
 
    if (Workspace::is_called(fun_symbol->get_name()))
-      return "function is pendent or suspended";
+      return "function is used, pendent or suspended";
 
 const UserFunction * ufun = function->get_ufun1();
    if (ufun == 0)
@@ -609,7 +607,7 @@ const UserFunction * ufun = function->get_ufun1();
 const UCS_string ftxt = function->canonical(false);
    Log(LOG_nabla)   UERR << "existing function is:\n" << ftxt << endl;
 
-vector<UCS_string> tlines;
+UCS_string_vector tlines;
    ftxt.to_vector(tlines);
 
    Assert(tlines.size());
@@ -638,7 +636,7 @@ vector<UCS_string> tlines;
                   }
              }
 
-         lines.push_back(fl);
+         lines.append(fl);
        }
 
    current_line = LineLabel(tlines.size());
@@ -698,7 +696,7 @@ const LineLabel user_edit_to = edit_to;
 
    if (idx_from == 0)   COUT << "    ∇" << endl;               // if header line
    for (int e = idx_from; e <= idx_to; ++e)   lines[e].print(COUT);
-   if ((size_t)idx_to == lines.size() - 1)   COUT << "    ∇" << endl;  // if last line
+   if (idx_to == lines.size() - 1)   COUT << "    ∇" << endl;  // if last line
 
    if (user_edit_to.valid())   // eg. [⎕42] or [2⎕42]
       {
@@ -715,7 +713,7 @@ const LineLabel user_edit_to = edit_to;
       }
    else                      // eg. [42⎕] or [⎕]
       {
-        current_line = lines.back().label;
+        current_line = lines.last().label;
         current_line.next();
       }
 
@@ -738,7 +736,7 @@ const int idx_to = find_line(LineLabel(edit_to));
 
    if (edit_from == -1)   // delete single line
       {
-        lines.erase(lines.begin() + idx_to);
+        lines.erase(idx_to);
         return 0;
       }
 
@@ -748,8 +746,8 @@ const int idx_from = find_line(LineLabel(edit_from));
    if (idx_from == -1)       return "Bad line number M in [M∆N] ";
    if (idx_from >= idx_to)   return "M ≥ N in [M∆N] ";
 
-   lines.erase(lines.begin() + idx_from, lines.begin() + idx_to);
-   current_line = lines.back().label;
+   loop(j, idx_to - idx_from)   lines.erase(idx_from);
+   current_line = lines.last().label;
    return 0;
 }
 //-----------------------------------------------------------------------------
@@ -839,14 +837,14 @@ const int idx_from = find_line(edit_from);
         // find the largest label before edit_from (if any)
         //
         int before_idx = -1;
-        for (size_t i = 0; i < lines.size(); ++i)
+        for (int i = 0; i < lines.size(); ++i)
             {
               if (lines[i].label < edit_from)   before_idx = i;
               else                                 break;
             }
 
         FunLine fl(edit_from, current_text);
-        lines.insert(lines.begin() + before_idx + 1, fl);
+        lines.insert_before(before_idx + 1, fl);
       }
    else
       {
@@ -856,14 +854,15 @@ const int idx_from = find_line(edit_from);
       }
 
    current_line.next();
+
    return 0;
 }
 //-----------------------------------------------------------------------------
 const char *
 Nabla::execute_escape()
 {
-   lines.clear();
-   lines.push_back(FunLine(0, fun_header));
+   lines.shrink(0);
+   lines.append(FunLine(0, fun_header));
    return 0;
 }
 //-----------------------------------------------------------------------------
