@@ -2,7 +2,7 @@
     This file is part of GNU APL, a free implementation of the
     ISO/IEC Standard 13751, "Programming Language APL, Extended"
 
-    Copyright (C) 2008-2013  Dr. Jürgen Sauermann
+    Copyright (C) 2008-2015  Dr. Jürgen Sauermann
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,21 +28,30 @@
 NameClass
 NamedObject::get_nc() const
 {
-   if (id == ID_USER_SYMBOL)   // user defined object
+   if (id == ID::USER_SYMBOL ||   // this named object is a user defined object
+       id == ID::ALPHA       ||   // ⍺
+       id == ID::ALPHA_U     ||   // ⍶
+       id == ID::LAMBDA      ||   // λ
+       id == ID::OMEGA       ||   // ⍵
+       id == ID::OMEGA_U     ||   // ⍹
+       id == ID::CHI)             // χ
       {
         const Symbol * sym = get_symbol();
-        if (sym == 0)   return NC_UNUSED_USER_NAME;
-        const ValueStackItem * tos = sym->top_of_stack();
-        if (tos)   return tos->name_class;
+        if (sym)
+           {
+             const ValueStackItem * tos = sym->top_of_stack();
+             if (tos)   return tos->name_class;
+           }
+
         return NC_UNUSED_USER_NAME;
       }
 
    // Distinguished name.
    //
-   Assert(Avec::is_quad(idname[0]));
+   Assert(Avec::is_quad(get_name()[0]));
 
 int len;
-Token tok = Workspace::get_quad(idname, len);
+Token tok = Workspace::get_quad(get_name(), len);
    if (len == 1)   return NC_INVALID;
 
    if (tok.get_Class() == TC_SYMBOL)   return NC_VARIABLE;
