@@ -30,7 +30,7 @@
 #include "Symbol.hh"
 #include "Workspace.hh"
 
-vector<NativeFunction *> NativeFunction::valid_functions;
+Simple_string<NativeFunction *, false> NativeFunction::valid_functions;
 
 //-----------------------------------------------------------------------------
 NativeFunction::NativeFunction(const UCS_string & so_name,
@@ -47,13 +47,12 @@ UCS_string t4;
    handle = open_so_file(t4, so_path);
    if (handle == 0)
       {
-         Workspace::more_error() = t4;
+         MORE_ERROR() << t4;
          return;
       }
 
    t4 = UCS_string("shared library ");
-   t4.append(so_name);
-   t4.append_utf8(" ");
+   t4 << so_name << " ";
 
    // get the function multiplexer
    //
@@ -62,8 +61,8 @@ void * fmux = dlsym(handle, "get_function_mux");
         {
           CERR << "shared library " << so_name << " is lacking the mandatory "
                   "function get_function_mux() !" << endl;
-          t4.append_utf8("is invalid (no get_function_mux())");
-          Workspace::more_error() = t4;
+          t4 << "is invalid (no get_function_mux())";
+          MORE_ERROR() = t4;
           return;
         }
 
@@ -78,8 +77,8 @@ void * (*get_function_mux)(const char *) = (void * (*)(const char *))fmux;
         {
           CERR << "shared library is lacking the mandatory "
                   "function signature() !" << endl;
-          t4.append_utf8("is invalid (no get_signature())");
-          Workspace::more_error() = t4;
+          t4 << "is invalid (no get_signature())";
+          MORE_ERROR() << t4;
           return;
         }
 
@@ -100,10 +99,10 @@ void * (*get_function_mux)(const char *) = (void * (*)(const char *))fmux;
 Symbol * sym = Workspace::lookup_symbol(apl_name);
    Assert(sym);
 
-const char * why_not = sym->cant_be_defined();
-   if (why_not)
+const char * why = sym->cant_be_defined();
+   if (why)
       {
-        Workspace::more_error() = UCS_string(why_not);
+        MORE_ERROR() << why;
         return;
       }
 
@@ -145,21 +144,21 @@ const char * why_not = sym->cant_be_defined();
    if (is_operator())   sym->set_nc(NC_OPERATOR, this);
    else                 sym->set_nc(NC_FUNCTION, this);
 
-   Workspace::more_error().clear();
+   Workspace::more_error().shrink(0);
    valid = true;
-   valid_functions.push_back(this);
+   valid_functions.append(this);
 }
 //-----------------------------------------------------------------------------
 NativeFunction::~NativeFunction()
 {
   Log(LOG_UserFunction__enter_leave)
-      CERR << "Native function " << get_name() << " deleted." << endl;
+      get_CERR() << "Native function " << get_name() << " deleted." << endl;
 
    loop(v, valid_functions.size())
       {
         if (valid_functions[v] == this)
            {
-             valid_functions.erase(valid_functions.begin() + v);
+             valid_functions.erase(v);
            }
       }
 }
@@ -200,7 +199,7 @@ NativeFunction::open_so_file(UCS_string & t4, UCS_string & so_path)
 {
    // prepare a )MORE error message containing the file names tried.
    //
-   t4.clear();
+   t4.shrink(0);
    t4.append_utf8("Could not find shared library '");
    t4.append(so_path);
    t4.append_utf8("'\n"
@@ -309,19 +308,23 @@ NativeFunction::cleanup()
 {
    // delete in reverse construction order
    //
-   loop(v, valid_functions.size())
+   while(valid_functions.size())
       {
-        NativeFunction & fun = *valid_functions.back();
-        if (fun.close_fun && fun.handle)
+        NativeFunction * fun = valid_functions.last();
+        valid_functions.pop();
+        
+        if (fun->close_fun && fun->handle)
            {
-             const bool do_dlclose = (*fun.close_fun)(CAUSE_SHUTDOWN, &fun);
+             const bool do_dlclose = (*fun->close_fun)(CAUSE_SHUTDOWN, fun);
              if (do_dlclose)
                 {
-                  dlclose(fun.handle);
-                  fun.handle = 0;
+                  dlclose(fun->handle);
+                  fun->handle = 0;
                 }
            }
-        delete valid_functions[v];
+
+        // don't delete fun since this will be done when the symbol for the
+        // fun is being deleted
       }
 }
 //-----------------------------------------------------------------------------
@@ -358,10 +361,10 @@ NativeFunction::fix(const UCS_string & so_name,
              Symbol * sym = Workspace::lookup_symbol(fun->get_name());
              Assert(sym);
 
-             const char * why_not = sym->cant_be_defined();
-             if (why_not)
+             const char * why = sym->cant_be_defined();
+             if (why)
                 {
-                  Workspace::more_error() = UCS_string(why_not);
+                  MORE_ERROR() << why;
                   return 0;
                 }
 
@@ -372,9 +375,14 @@ NativeFunction::fix(const UCS_string & so_name,
       }
 
 NativeFunction * new_function = new NativeFunction(so_name, function_name);
+   Log(LOG_delete)
+      CERR << "new    " << (const void *)new_function << " at " LOC << endl;
+
 
    if (!new_function->valid)   // something went wrong
       {
+        Log(LOG_delete)
+          CERR << "delete " << (const void *)new_function << " at " LOC << endl;
         delete new_function;
         return 0;
       }
@@ -457,7 +465,7 @@ const int error = ((int (*)(const char *, const char *))emacs_start)
         return t4;
       }
 
-   t4.clear();   // success
+   t4.shrink(0);   // success
    return t4;
 }
 //-----------------------------------------------------------------------------
