@@ -1348,6 +1348,9 @@ double z  = tgamma(N + 1);          // N!
 ErrorCode
 NumericCell::bif_binomial(Cell * Z, const Cell * A) const
 {
+   if (!A->is_numeric())            return E_DOMAIN_ERROR;
+   if (A->get_real_value() > 170)   return E_DOMAIN_ERROR;
+
    if (!is_near_real() || !A->is_near_real())   // complex result
       {
         const APL_Float r_1_a    = A->get_real_value();
@@ -1376,12 +1379,23 @@ NumericCell::bif_binomial(Cell * Z, const Cell * A) const
         if (r_1_b < 0    && is_near_int(r_1_b))      return E_DOMAIN_ERROR;
         if (r_1_b__a < 0 && is_near_int(r_1_b__a))   return E_DOMAIN_ERROR;
 
-        new (Z) FloatCell(  tgamma(r_1_b) / (tgamma(r_1_a) * tgamma(r_1_b__a)));
-        return E_NO_ERROR;
+        const double rb = tgamma(r_1_b);
+        if (!isfinite(rb))   return E_DOMAIN_ERROR;
+
+        const double ra = tgamma(r_1_a);
+        if (!isfinite(ra))   return E_DOMAIN_ERROR;
+
+        const double r_b__a = tgamma(r_1_b__a);
+        if (!isfinite(r_b__a))   return E_DOMAIN_ERROR;
+
+        return FloatCell::zv(Z, (rb / ra) / r_b__a);
       }
 
    if (!A->is_near_int())   return E_DOMAIN_ERROR;
    if (!is_near_int())      return E_DOMAIN_ERROR;
+   if (is_float_cell() && get_real_value() > 0x7FFFFFFFFFFFFFFFLL)
+      return E_DOMAIN_ERROR;
+
 const APL_Integer a = A->get_checked_near_int();
 const APL_Integer b = get_checked_near_int();
 
