@@ -103,6 +103,22 @@ Prefix::syntax_error(const char * loc)
 }
 //-----------------------------------------------------------------------------
 bool
+Prefix::uses_function(const UserFunction * ufun) const
+{
+   loop (s, size())
+      {
+        const Token & tok = at(s).tok;
+        if (tok.get_ValueType() == TV_FUN &&
+            tok.get_function() == ufun)   return true;
+      }
+
+   if (saved_lookahead.tok.get_ValueType() == TV_FUN &&
+            saved_lookahead.tok.get_function() == ufun)   return true;
+
+   return false;
+}
+//-----------------------------------------------------------------------------
+bool
 Prefix::is_value_parent(int pc) const
 {
    // we have ) XXX with XXX on the stack and need to know if the evaluation
@@ -466,7 +482,7 @@ grow:
                     if (!!val && !left_sym)
                        {
                          Token tok(TOK_APL_VALUE1, val);
-                         move_1(tl.tok, tok, LOC);
+                         tl.tok.move_1(tok, LOC);
                          resolved = true;
                        }
                   }
@@ -708,8 +724,8 @@ Prefix::dont_reduce(TokenClass next) const
 bool
 Prefix::replace_AB(Value_P old_value, Value_P new_value)
 {
-   Assert(old_value);
-   Assert(new_value);
+   Assert(!!old_value);
+   Assert(!!new_value);
 
    loop(s, size())
      {
@@ -839,8 +855,8 @@ Prefix::reduce_LPAR_F_C_RPAR()
    if (at2().get_ValueType() != TV_VAL)   SYNTAX_ERROR;
    if (!at2().get_apl_val())              SYNTAX_ERROR;
 
-   move_1(at3(), at2(), LOC);
-   move_1(at2(), at1(), LOC);
+   at3().move_1(at2(), LOC);
+   at2().move_1(at1(), LOC);
    pop();    // pop C
    pop();    // pop RPAR
    action = RA_CONTINUE;
@@ -904,8 +920,7 @@ Token result = at0().get_function()->eval_B(at1().get_apl_val());
                   Token exec = Bif_F1_EXECUTE::execute_statement(stat);
                   if (exec.get_Class() == TC_VALUE)   // ⍎ literal
                      {
-                       move_1(Workspace::SI_top()->get_prefix().at0(),
-                              exec, LOC);
+                       Workspace::SI_top()->get_prefix().at0().move_1(exec,LOC);
                        return;
                      }
 
@@ -975,6 +990,21 @@ Prefix::reduce_MISC_F_C_B()
       }
    if (!at1().get_apl_val())              SYNTAX_ERROR;
 
+   if (at0().get_tag() == TOK_Quad_FIO &&
+       saved_lookahead.tok.get_Class() == TC_FUN12)
+      {
+        DerivedFunction * derived =
+                          Workspace::SI_top()->fun_oper_cache.get(LOC);
+        new (derived)   DerivedFunction(saved_lookahead.tok,
+                                        at0().get_function(),
+                                        at1().get_apl_val(),  LOC);
+        saved_lookahead.tok.clear(LOC);
+        prefix_len = 2;   // only f ⎕FIO
+        pop_args_push_result(Token(TOK_FUN2, derived));
+        action = RA_CONTINUE;
+        return;
+      }
+
 Token result = at0().get_function()->eval_XB(at1().get_apl_val(),
                                              at2().get_apl_val());
    if (result.get_tag() == TOK_ERROR)
@@ -1396,7 +1426,8 @@ void
 Prefix::reduce_V_C__()
 {
 Symbol * V = at0().get_sym_ptr();
-   copy_1(at0(), V->resolve_lv(LOC), LOC);
+Token tok = V->resolve_lv(LOC);
+   at0().move_1(tok, LOC);
    set_assign_state(ASS_var_seen);
    action = RA_CONTINUE;
 }
@@ -1461,7 +1492,8 @@ Prefix::reduce_F_V__()
    // turn V into a (left-) value
    //
 Symbol * V = at1().get_sym_ptr();
-   copy_1(at1(), V->resolve_lv(LOC), LOC);
+Token tok = V->resolve_lv(LOC);
+   at1().move_1(tok, LOC);
    set_assign_state(ASS_var_seen);
    action = RA_CONTINUE;
 }
@@ -1591,8 +1623,8 @@ Token result = at1();
       {
         assign_state = idx.get_assign_state();
 
-        if (idx.is_axis()) move_2(result, Token(TOK_AXES, idx.values[0]), LOC);
-        else               move_2(result, Token(TOK_INDEX, idx), LOC);
+        if (idx.is_axis()) result.move_2(Token(TOK_AXES, idx.values[0]), LOC);
+        else               result.move_2(Token(TOK_INDEX, idx), LOC);
       }
    else
       {
@@ -1624,14 +1656,14 @@ const bool last_index = (at0().get_tag() == TOK_L_BRACK);   // ; vs. [
            {
              Value_P X = idx.extract_value(0);
              Assert1(!!X);
-             move_2(I, Token(TOK_AXES, X), LOC);
+             I.move_2(Token(TOK_AXES, X), LOC);
              Log(LOG_delete)
                 CERR << "delete " HEX(&idx) << " at " LOC << endl;
              delete &idx;
            }
         else
            {
-             move_2(I, Token(TOK_INDEX, idx), LOC);
+             I.move_2(Token(TOK_INDEX, idx), LOC);
            }
       }
    else
@@ -1676,26 +1708,26 @@ const int count = vector_ass_count();
         Symbol * V = at0().get_sym_ptr();
         Token result = V->resolve_lv(LOC);
         set_assign_state(ASS_var_seen);
-        move_1(at0(), result, LOC);
+        at0().move_1(result, LOC);
         action = RA_CONTINUE;
         return;
       }
 
    // vector assignment.
    //
-DynArray(Symbol *, symbols, count + 1);
-   symbols[0] = at0().get_sym_ptr();
+Simple_string<Symbol *, false> symbols;
+   symbols.append(at0().get_sym_ptr());
    loop(c, count)
       {
         Token_loc tl = lookahead();
         Assert1(tl.tok.get_tag() == TOK_LSYMB2);   // by vector_ass_count()
         Symbol * V = tl.tok.get_sym_ptr();
         Assert(V);
-        symbols[c + 1] = V;
+        symbols.append(V);
       }
 
 Value_P B = at3().get_apl_val();
-   Symbol::vector_assignment(symbols.get_data(), count + 1, B);
+   Symbol::vector_assignment(symbols, B);
 
    set_assign_state(ASS_none);
 
@@ -1955,6 +1987,7 @@ Prefix::reduce_RETC___()
 
              {
                const UserFunction * ufun = si.get_executable()->get_ufun();
+               if (ufun)   { /* do nothing, needed for -Wall */ }
                Assert1(ufun);
                at0().clear(LOC);
              }
