@@ -35,6 +35,7 @@
 #include "IndexIterator.hh"
 #include "IntCell.hh"
 #include "LvalCell.hh"
+#include "Macro.hh"
 #include "Output.hh"
 #include "PointerCell.hh"
 #include "PrimitiveFunction.hh"
@@ -101,6 +102,8 @@ Bif_F2_RIGHT      * Bif_F2_RIGHT     ::fun = &Bif_F2_RIGHT     ::_fun;
 const CharCell PrimitiveFunction::c_filler(UNI_ASCII_SPACE);
 const IntCell  PrimitiveFunction::n_filler(0);
 
+int Bif_F1_EXECUTE::copy_pending = 0;
+
 //-----------------------------------------------------------------------------
 Token
 PrimitiveFunction::eval_fill_AB(Value_P A, Value_P B)
@@ -334,18 +337,16 @@ Value_P Z(B->get_shape(), LOC);
    loop(bz, len_BZ)
        {
          const Cell & cell_B = B->get_ravel(bz);
-         bool found = false;
+         APL_Integer z = len_A;   // assume cell_B is not in A
          loop(a, len_A)
              {
                if (cell_B.equal(A->get_ravel(a), qct))
                   {
-                    new (Z->next_ravel()) IntCell(qio + a);
-                    found = true;
+                    z = a;
                     break;
                   }
              }
-         if (!found)
-            new (Z->next_ravel()) IntCell(qio + len_A);
+         new (Z->next_ravel()) IntCell(qio + z);
        }
 
    Z->check_value(LOC);
@@ -982,7 +983,7 @@ ShapeItem cols_B = 1;
    if (rows_B <  cols_B)   LENGTH_ERROR;
    if (rows_A != rows_B)   LENGTH_ERROR;
 
-const bool need_complex = A->is_complex() || B->is_complex();
+const bool need_complex = A->is_complex(true) || B->is_complex(true);
 Value_P Z(shape_Z, LOC);
    divide_matrix(&Z->get_ravel(0), need_complex,
                  rows_A, cols_A, &A->get_ravel(0),
@@ -1743,14 +1744,21 @@ const Shape weight_B = B->get_shape().reverse_scan();
             }
        }
 
+int X_axes_used = 0;
    loop(r, shape_X.get_rank())   // the axes in shape_X
        {
          const ShapeItem x_r = shape_X.get_shape_item(r);
 
          // check that X∈⍳⍴⍴B
          //
-         if (x_r < 0)                AXIS_ERROR;
-         if (x_r >= B->get_rank())   AXIS_ERROR;
+         if (x_r < 0)                  AXIS_ERROR;
+         if (x_r >= B->get_rank())     AXIS_ERROR;
+         if (X_axes_used & 1 << x_r)
+            {
+              MORE_ERROR() = "Duplicate axis";
+              AXIS_ERROR;
+            }
+         X_axes_used |= 1 << x_r;
 
          item_shape.add_shape_item(B->get_shape_item(x_r));
          it_weight.add_shape_item(weight_B.get_shape_item(x_r));
@@ -2776,6 +2784,19 @@ ExecuteList * fun = ExecuteList::fix(statement.no_pad(), LOC);
 Token
 Bif_F1_EXECUTE::execute_command(UCS_string & command)
 {
+   if (copy_pending &&
+       (
+//      command.starts_iwith(")COPY")    ||
+        command.starts_iwith(")ERASE")   ||
+        command.starts_iwith(")FNS")     ||
+        command.starts_iwith(")NMS")     ||
+        command.starts_iwith(")QLOAD")   ||
+        command.starts_iwith(")SYMBOLS") ||
+        command.starts_iwith(")VARS")))
+      {
+        throw_apl_error(E_COPY_PENDING, LOC);
+      }
+
    if (command.starts_iwith(")LOAD")  ||
        command.starts_iwith(")QLOAD") ||
        command.starts_iwith(")CLEAR") ||
@@ -2799,11 +2820,11 @@ UTF8_string result_utf8 = out.get_data();
        result_utf8.last() != UNI_ASCII_LF)
       result_utf8.append(UNI_ASCII_LF);
 
-vector<ShapeItem> line_starts;
-   line_starts.push_back(0);
+Simple_string<ShapeItem, false> line_starts;
+   line_starts.append(0);
    loop(r, result_utf8.size())
       {
-        if (result_utf8[r] == UNI_ASCII_LF)   line_starts.push_back(r + 1);
+        if (result_utf8[r] == UNI_ASCII_LF)   line_starts.append(r + 1);
       }
 
 Value_P Z((ShapeItem)line_starts.size() - 1, LOC);
@@ -2833,21 +2854,25 @@ Bif_F12_UNION::eval_B(Value_P B)
 const APL_Float qct = Workspace::get_CT();
 const ShapeItem len_B = B->element_count();
    if (len_B <= 1)   return Token(TOK_APL_VALUE1, B->clone(LOC));
+   if (len_B >= SHORT_VALUE_LENGTH_WANTED && !B->is_complex(false))
+      return Macro::Z__UNIQUE_B->eval_B(B);
 
-DynArray(const Cell *, items_Z, len_B);
-ShapeItem len_Z = 0;
+   // B is small, so an itertive search of unique elements is faster
+   //
+Simple_string<const Cell *, false> items_Z;
+   items_Z.reserve(len_B);
 
    loop(b, len_B)
       {
         const Cell & cell = B->get_ravel(b);
-        if (is_unique(cell, items_Z.get_data(), len_Z, qct))
-           items_Z[len_Z++] = &cell;
+        if (is_unique(cell, items_Z, qct))   items_Z.append(&cell);
       }
 
    // build result value Z
    //
-Value_P Z(len_Z, LOC);
-   loop(z, len_Z)   Z->next_ravel()->init(*(items_Z[z]), Z.getref(), LOC);
+Value_P Z(items_Z.size(), LOC);
+   loop(z, items_Z.size())
+       Z->next_ravel()->init(*(items_Z[z]), Z.getref(), LOC);
    Z->check_value(LOC);
    return Token(TOK_APL_VALUE1, Z);
 }
