@@ -2,7 +2,7 @@
     This file is part of GNU APL, a free implementation of the
     ISO/IEC Standard 13751, "Programming Language APL, Extended"
 
-    Copyright (C) 2008-2015  Dr. Jürgen Sauermann
+    Copyright (C) 2008-2016  Dr. Jürgen Sauermann
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include "PrintBuffer.hh"
 #include "PrintOperator.hh"
 #include "Value.icc"
+#include "Workspace.hh"
 
 /// max sizes for arrays on the stack. Larger values are allocated with new()
 enum
@@ -48,7 +49,7 @@ PrintBuffer::PrintBuffer(const UCS_string & ucs, const ColInfo & ci)
    : col_info(ci),
      complete(true)
 {
-   buffer.push_back(ucs);
+   buffer.append(ucs);
 }
 //-----------------------------------------------------------------------------
 PrintBuffer::PrintBuffer(const Value & value, const PrintContext & _pctx,
@@ -148,46 +149,27 @@ const ShapeItem ec = value.element_count();
    //
 const ShapeItem cols = value.get_last_shape_item();
 
-   if (cols <= PB_MAX_COLS)   // few columns
+Simple_string<bool, false> scaling;
+   scaling.reserve(cols);
+   loop(c, cols)   scaling.append(false);
+
+Simple_string<PrintBuffer, true> pcols;
+   pcols.reserve(cols);
+   loop(c, cols)   pcols.append(PrintBuffer());
+
+PrintBuffer * item_matrix = 0;
+   try { item_matrix = new PrintBuffer[ec]; }
+   catch (...)
       {
-        DynArray(bool, scaling, cols);
-        DynArray(PrintBuffer, pcols, cols);
-        if (ec <= PB_MAX_ITEMS)
-           {
-             DynArray(PrintBuffer, item_matrix, ec);
-             do_PrintBuffer(value, pctx, out, outer_style,
-                            scaling.get_data(), pcols.get_data(),
-                            item_matrix.get_data());
-           }
-        else
-           {
-             PrintBuffer * item_matrix = new PrintBuffer[ec];
-             do_PrintBuffer(value, pctx, out, outer_style,
-                            scaling.get_data(), pcols.get_data(),
+        MORE_ERROR() << "value too large to print ("
+                     << cols << " columns, " << ec << " items)";
+        WS_FULL;
+      }
+
+   do_PrintBuffer(value, pctx, out, outer_style,
+                            &scaling[0], &pcols[0],
                             item_matrix);
-             delete [] item_matrix;
-           }
-      }
-   else                      // many columns
-      {
-        bool * scaling = new bool[cols];
-        PrintBuffer * pcols = new PrintBuffer[cols];
-        if (ec <= PB_MAX_ITEMS)
-           {
-             DynArray(PrintBuffer, item_matrix, ec);
-             do_PrintBuffer(value, pctx, out, outer_style,
-                            scaling, pcols, item_matrix.get_data());
-           }
-        else
-           {
-             PrintBuffer * item_matrix = new PrintBuffer[ec];
-             do_PrintBuffer(value, pctx, out, outer_style,
-                            scaling, pcols, item_matrix);
-             delete [] item_matrix;
-           }
-        delete [] pcols;
-        delete [] scaling;
-      }
+   delete [] item_matrix;
 
    PERFORMANCE_END(fs_PrintBuffer_B, start_0, ec)
 }
@@ -231,12 +213,12 @@ const bool nested = !value.is_simple();
      //    therefore we have (⍴,value) items. Items are rectangular.
      //
      PERFORMANCE_START(start_2)
-     vector<Rank> max_row_ranks;
+     Simple_string<Rank, false> max_row_ranks;
      max_row_ranks.reserve(rows);
      loop(y, rows)
         {
           ShapeItem max_row_height = 0;
-          max_row_ranks.push_back(0);
+          max_row_ranks.append(0);
 
           loop(x, cols)
               {
@@ -252,8 +234,8 @@ const bool nested = !value.is_simple();
                    {
                      Value_P sub_val = cell.get_pointer_value();
                      const Rank sub_rank = sub_val->get_rank();
-                     if (max_row_ranks.back() < sub_rank)
-                        max_row_ranks.back() = sub_rank;
+                     if (max_row_ranks.last() < sub_rank)
+                        max_row_ranks.last() = sub_rank;
                    }
 
                 if (max_row_height < item.get_height())
@@ -637,7 +619,15 @@ PrintBuffer::set_char(int x, int y, Unicode uc)
 void
 PrintBuffer::pad_l(Unicode pad, ShapeItem count)
 {
-   loop(y, get_height())   buffer[y].insert(0, count, pad);
+   if (count == 1)
+      {
+        loop(y, get_height())   buffer[y].prepend(pad);
+      }
+   else
+      {
+        UCS_string pads(count, pad);
+        loop(y, get_height())   buffer[y] = pads + buffer[y];
+      }
 }
 //-----------------------------------------------------------------------------
 void
@@ -653,7 +643,7 @@ PrintBuffer::pad_height(Unicode pad, ShapeItem height)
    if (height > get_height())
       {
         UCS_string ucs(get_width(0), pad);
-        while (height > get_height())   buffer.push_back(ucs);
+        while (height > get_height())   buffer.append(ucs);
       }
 }
 //-----------------------------------------------------------------------------
@@ -671,7 +661,7 @@ PrintBuffer::pad_height_above(Unicode pad, ShapeItem height)
    if (height > get_height())
       {
         UCS_string ucs(get_width(0), pad);
-        while (height > get_height())   buffer.insert(buffer.begin(), ucs);
+        while (height > get_height())   buffer.insert_before(0, ucs);
       }
 }
 //-----------------------------------------------------------------------------
@@ -743,12 +733,12 @@ Unicode HORI, VERT, NW, NE, SE, SW;
         UCS_string upper;
         upper.append(NE);
         upper.append(NW);
-        buffer.push_back(upper);
+        buffer.append(upper);
 
         UCS_string lower;
         lower.append(SE);
         lower.append(SW);
-        buffer.push_back(lower);
+        buffer.append(lower);
 
         Assert(is_rectangular());
         return;
@@ -758,7 +748,7 @@ Unicode HORI, VERT, NW, NE, SE, SW;
    //
    loop(y, get_height())
       {
-        buffer[y].insert(0, 1, VERT);
+        buffer[y].prepend(VERT);
         buffer[y].append(VERT);
 
         // change internal pad characters to ASCII_SPACE so that they will
@@ -772,8 +762,8 @@ Unicode HORI, VERT, NW, NE, SE, SW;
    //
 UCS_string hori(get_width(0), HORI);
 
-   buffer.insert(buffer.begin(), hori);
-   buffer.push_back(hori);
+   buffer.insert_before(0, hori);
+   buffer.append(hori);
 
    // draw the corners
    //
@@ -850,12 +840,12 @@ Unicode HORI, VERT, NW, NE, SE, SW;
         UCS_string upper;
         upper.append(NE);
         upper.append(NW);
-        buffer.push_back(upper);
+        buffer.append(upper);
 
         UCS_string lower;
         lower.append(SE);
         lower.append(SW);
-        buffer.push_back(lower);
+        buffer.append(lower);
 
         Assert(is_rectangular());
         return;
@@ -865,7 +855,7 @@ Unicode HORI, VERT, NW, NE, SE, SW;
    //
    loop(y, get_height())
       {
-        buffer[y].insert(0, 1, VERT);
+        buffer[y].prepend(VERT);
         buffer[y].append(VERT);
 
         // change internal pad characters to ASCII_SPACE so that they will
@@ -878,8 +868,8 @@ Unicode HORI, VERT, NW, NE, SE, SW;
    //
 UCS_string hori(get_width(0), HORI);
 
-   buffer.insert(buffer.begin(), hori);
-   buffer.push_back(hori);
+   buffer.insert_before(0, hori);
+   buffer.append(hori);
 
    // draw the corners
    //
@@ -939,24 +929,24 @@ PrintBuffer::append_ucs(const UCS_string & ucs)
 {
    if (buffer.size() == 0)   // empty
       {
-        buffer.push_back(ucs);
+        buffer.append(ucs);
       }
    else if (ucs.size() < get_width(0))
       {
         UCS_string ucs1(ucs);
         UCS_string pad(get_width(0) - ucs.size(), UNI_iPAD_L1);
         ucs1.append(pad);
-        buffer.push_back(ucs1);
+        buffer.append(ucs1);
       }
    else if (ucs.size() > get_width(0))
       {
         UCS_string pad(ucs.size() - get_width(0), UNI_iPAD_L2);
         loop(h, get_height())   buffer[h].append(pad);
-        buffer.push_back(ucs);
+        buffer.append(ucs);
       }
    else
       {
-        buffer.push_back(ucs);
+        buffer.append(ucs);
       }
 }
 //-----------------------------------------------------------------------------
@@ -1058,7 +1048,7 @@ UCS_string ucs1;
    if (this_l > 0)   pad_l(UNI_ASCII_SPACE, this_l);
    if (this_r > 0)   pad_r(UNI_ASCII_SPACE, this_r);
 
-   buffer.push_back(ucs1);
+   buffer.append(ucs1);
 
    Assert(is_rectangular());
 }
@@ -1086,7 +1076,7 @@ PrintBuffer::add_column(Unicode pad, int32_t pad_count, const PrintBuffer & pb)
 void PrintBuffer::add_row(const PrintBuffer & pb)
 {
    buffer.reserve(buffer.size() + pb.get_height());
-   loop(h, pb.get_height())   buffer.push_back(pb.buffer[h]);
+   loop(h, pb.get_height())   buffer.append(pb.buffer[h]);
 }
 //-----------------------------------------------------------------------------
 void
@@ -1309,7 +1299,7 @@ PrintBuffer::align_left(ColInfo & COL_INFO)
 const size_t diff = COL_INFO.int_len - col_info.int_len;
 
    if (buffer.size())   pad_r(UNI_iPAD_L3, diff);
-   else                 buffer.push_back(UCS_string(diff, UNI_iPAD_L3));
+   else                 buffer.append(UCS_string(diff, UNI_iPAD_L3));
 
    col_info.int_len = COL_INFO.int_len;
 
