@@ -2,7 +2,7 @@
     This file is part of GNU APL, a free implementation of the
     ISO/IEC Standard 13751, "Programming Language APL, Extended"
 
-    Copyright (C) 2008-2015  Dr. Jürgen Sauermann
+    Copyright (C) 2008-2016  Dr. Jürgen Sauermann
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@
 #include "PrintOperator.hh"
 #include "QuadFunction.hh"
 #include "Quad_FX.hh"
+#include "Quad_SQL.hh"
 #include "Quad_TF.hh"
 #include "Tokenizer.hh"
 #include "UserFunction.hh"
@@ -364,7 +365,7 @@ Value_P Z(3, LOC);
         new (Z->next_ravel()) PointerCell(Z3, Z.getref());
 
         Z->check_value(LOC);
-        move_2(result, Token(TOK_APL_VALUE1, Z), LOC);
+        result.move_2(Token(TOK_APL_VALUE1, Z), LOC);
         return;
       }
 
@@ -415,7 +416,7 @@ Value_P Z2(2, LOC);
       }
 
    Z->check_value(LOC);
-   move_2(result, Token(TOK_APL_VALUE1, Z), LOC);
+   result.move_2(Token(TOK_APL_VALUE1, Z), LOC);
 }
 //=============================================================================
 Token
@@ -425,7 +426,7 @@ Quad_ENV::eval_B(Value_P B)
 
 const ShapeItem ec_B = B->element_count();
 
-vector<const char *> evars;
+Simple_string<const char *, false> evars;
 
    for (char **e = environ; *e; ++e)
        {
@@ -443,7 +444,7 @@ vector<const char *> evars;
                  }
             }
 
-         if (match)   evars.push_back(env);
+         if (match)   evars.append(env);
        }
 
 const Shape sh_Z(evars.size(), 2);
@@ -462,7 +463,7 @@ Value_P Z(sh_Z, LOC);
 
         Value_P varname(ucs, LOC);
 
-        ucs.clear();
+        ucs.shrink(0);
         while (*env)   ucs.append(Unicode(*env++));
 
         Value_P varval(ucs, LOC);
@@ -539,7 +540,7 @@ const ErrorCode ec = get_error_code(B);
       }
    else                                   //  ⎕ES B with unknown major/minor B
       {
-        error.error_message_1.clear();
+        error.error_message_1.shrink(0);
       }
 
    error.show_locked = true;
@@ -601,8 +602,7 @@ Quad_EX::eval_B(Value_P B)
    if (B->get_rank() > 2)   RANK_ERROR;
 
 const ShapeItem var_count = B->get_rows();
-vector<UCS_string> vars(var_count);
-   B->to_varnames(vars, false);
+const UCS_string_vector vars(B.getref(), false);
 
 Shape sh_Z;
    if (var_count > 1)   sh_Z.add_shape_item(var_count);
@@ -629,7 +629,7 @@ Quad_INP::eval_AB(Value_P A, Value_P B)
 {
    if (Quad_INP_running)
       {
-        Workspace::more_error() = UCS_string("⎕INP called recursively");
+        MORE_ERROR() = "⎕INP called recursively";
         SYNTAX_ERROR;
       }
 
@@ -650,9 +650,9 @@ UCS_string e2;
    esc1 = e1;
    esc2 = e2;
 
-   prefixes.clear();
-   escapes.clear();
-   suffixes.clear();
+   prefixes.shrink(0);
+   escapes.shrink(0);
+   suffixes.shrink(0);
 
    read_strings();    // read lines from file or stdin
    split_strings();   // split lines into prefixes, escapes, and suffixes
@@ -709,7 +709,7 @@ Quad_INP::eval_B(Value_P B)
 {
    if (Quad_INP_running)
       {
-        Workspace::more_error() = UCS_string("⎕INP called recursively");
+        MORE_ERROR() << "⎕INP called recursively";
         SYNTAX_ERROR;
       }
 
@@ -758,7 +758,7 @@ APL_Integer x = X->get_ravel(0).get_near_int();
 
 UCS_string end_marker(B->get_UCS_ravel());
 
-vector<UCS_string> lines;
+UCS_string_vector lines;
 Parser parser(PM_EXECUTE, LOC, false);
 
    for (;;)
@@ -797,7 +797,7 @@ Parser parser(PM_EXECUTE, LOC, false);
                  }
 
             }
-         lines.push_back(line);
+         lines.append(line);
       }
 
 Value_P Z(lines.size(), LOC);
@@ -869,7 +869,7 @@ Quad_INP::read_strings()
 {
    // read lines until an end-maker is detected
    //
-   raw_lines.clear();
+   raw_lines.shrink(0);
    for (;;)
       {
         bool eof = false;
@@ -882,12 +882,12 @@ Quad_INP::read_strings()
         if (end != -1)   // end marker found
            {
              line.shrink(end);
-             if (line.size())   raw_lines.push_back(line);
+             if (line.size())   raw_lines.append(line);
              break;
            }
 
         if (eof && !line.size())   break;
-        raw_lines.push_back(line);
+        raw_lines.append(line);
         if (eof)   break;
       }
 }
@@ -902,9 +902,9 @@ UCS_string empty;
         const ShapeItem epos = line.substr_pos(esc1);
         if (esc1.size() == 0 || epos == -1)   // no escape in this line
            {
-             prefixes.push_back(line);
-             escapes.push_back(empty);
-             suffixes.push_back(empty);
+             prefixes.append(line);
+             escapes.append(empty);
+             suffixes.append(empty);
              continue;
            }
 
@@ -912,7 +912,7 @@ UCS_string empty;
         //
         UCS_string pref = line;
         pref.shrink(epos);
-        prefixes.push_back(pref);
+        prefixes.append(pref);
 
         line = line.drop(epos + esc1.size());   // skip prefix and esc1
         if (esc2.size())   // end defined
@@ -920,24 +920,24 @@ UCS_string empty;
              const ShapeItem eend = line.substr_pos(esc2);
              if (eend == -1)   // no exec end in this line
                 {
-                  escapes.push_back(line);
-                  suffixes.push_back(empty);
+                  escapes.append(line);
+                  suffixes.append(empty);
                   continue;
                 }
              else              // found an exec end in this line
                 {
                   UCS_string exec = line;
                   exec.shrink(eend);
-                  escapes.push_back(exec);
+                  escapes.append(exec);
                   line = line.drop(eend + esc2.size());   // skip exec and esc2
-                  suffixes.push_back(line);
+                  suffixes.append(line);
                   continue;
                 }
            }
         else               // no end defined
            {
-             escapes.push_back(line);
-             suffixes.push_back(empty);
+             escapes.append(line);
+             suffixes.append(empty);
            }
       }
 
@@ -952,8 +952,7 @@ Quad_NC::eval_B(Value_P B)
    if (B->get_rank() > 2)   RANK_ERROR;
 
 const ShapeItem var_count = B->get_rows();
-vector<UCS_string> vars(var_count);
-   B->to_varnames(vars, false);
+const UCS_string_vector vars(B.getref(), false);
 
 Shape sh_Z;
    if (var_count > 1)   sh_Z.add_shape_item(var_count);
@@ -1048,16 +1047,14 @@ int requested_NCs = 0;
 
    // 2, build a name table, starting with user defined names
    //
-vector<UCS_string> names;
+UCS_string_vector names;
    {
-     const ShapeItem symbol_count = Workspace::symbols_allocated();
-     DynArray(Symbol *, table, symbol_count);
-     Workspace::get_all_symbols(&table[0], symbol_count);
+     Simple_string<const Symbol *, false> symbols = Workspace::get_all_symbols();
 
-     loop(s, symbol_count)
+     loop(s, symbols.size())
         {
-          Symbol * symbol = table[s];
-          if (symbol->is_erased())   continue;   // erased symbol
+          const Symbol * symbol = symbols[s];
+          if (symbol->is_erased())   continue;
 
           NameClass nc = symbol->get_nc();
           if (nc == NC_SHARED_VAR)   nc = NC_VARIABLE;
@@ -1070,7 +1067,7 @@ vector<UCS_string> names;
                if (!first_chars.contains(first_char))   continue;
              }
 
-          names.push_back(symbol->get_name());
+          names.append(symbol->get_name());
         }
    }
 
@@ -1081,15 +1078,15 @@ vector<UCS_string> names;
 #define ro_sv_def(x, _str, _txt)                                   \
    { Symbol * symbol = &Workspace::get_v_ ## x();           \
      if ((requested_NCs & 1 << 5) && symbol->get_nc() != 0) \
-        names.push_back(symbol->get_name()); }
+        names.append(symbol->get_name()); }
 
 #define rw_sv_def(x, _str, _txt)                                   \
    { Symbol * symbol = &Workspace::get_v_ ## x();           \
      if ((requested_NCs & 1 << 5) && symbol->get_nc() != 0) \
-        names.push_back(symbol->get_name()); }
+        names.append(symbol->get_name()); }
 
 #define sf_def(x, _str, _txt)                                      \
-   { if (requested_NCs & 1 << 6)   names.push_back((x::fun->get_name())); }
+   { if (requested_NCs & 1 << 6)   names.append((x::fun->get_name())); }
 #include "SystemVariable.def"
       }
 
@@ -1363,7 +1360,8 @@ UserFunction * ufun = fun->get_ufun1();
 }
 //-----------------------------------------------------------------------------
 Token
-Stop_Trace::reference(const Simple_string<Function_Line> & lines, bool assigned)
+Stop_Trace::reference(const Simple_string<Function_Line, false> & lines,
+                      bool assigned)
 {
 Value_P Z(lines.size(), LOC);
 
@@ -1376,17 +1374,17 @@ Value_P Z(lines.size(), LOC);
 void
 Stop_Trace::assign(UserFunction * ufun, const Value & new_value, bool stop)
 {
-DynArray(Function_Line, lines, new_value.element_count());
-int line_count = 0;
+Simple_string<Function_Line, false> lines;
+   lines.reserve(new_value.element_count());
 
    loop(l, new_value.element_count())
       {
          APL_Integer line = new_value.get_ravel(l).get_near_int();
          if (line < 1)   continue;
-         lines[line_count++] = (Function_Line)line;
+         lines.append((Function_Line)line);
       }
 
-   ufun->set_trace_stop(lines.get_data(), line_count, stop);
+   ufun->set_trace_stop(lines, stop);
 }
 //=============================================================================
 Token
