@@ -76,7 +76,7 @@ const Function * function = 0;
    // show the function...
    //
 const UCS_string ucs = function->canonical(false);
-vector<UCS_string> tlines;
+UCS_string_vector tlines;
    ucs.to_vector(tlines);
 int max_len = 0;
    loop(row, tlines.size())
@@ -136,7 +136,7 @@ Quad_CR::do_CR(APL_Integer a, const Value * B, PrintContext pctx)
         case -17: a = 16;   break;
         case -18: a = 19;   break;
         case -19: a = 18;   break;
-        default: Workspace::more_error() = "A ⎕CR B with invalid A < 0";
+        default: MORE_ERROR() << "A ⎕CR B with invalid A < 0";
                  DOMAIN_ERROR;
       }
   
@@ -188,7 +188,7 @@ bool extra_frame = false;
         case 31: return do_CR31_32(true, *B);    // ⎕INP helper
         case 32: return do_CR31_32(false, *B);   // ⎕INP helper
 
-        default: Workspace::more_error() = "A ⎕CR B with invalid A";
+        default: MORE_ERROR() << "A ⎕CR B with invalid A";
                  DOMAIN_ERROR;
       }
 
@@ -247,7 +247,7 @@ Quad_CR::do_CR10(const Value & B)
    // cannot use PrintBuffer because the lines in ucs_vec
    // have different lengths
    //
-vector<UCS_string> ucs_vec;
+UCS_string_vector ucs_vec;
    do_CR10(ucs_vec, B);
 
 Value_P Z(ucs_vec.size(), LOC);
@@ -263,7 +263,7 @@ Value_P Z(ucs_vec.size(), LOC);
 }
 //-----------------------------------------------------------------------------
 void
-Quad_CR::do_CR10(vector<UCS_string> & result, const Value & B)
+Quad_CR::do_CR10(UCS_string_vector & result, const Value & B)
 {
 const UCS_string symbol_name(B);
 const Symbol * symbol = Workspace::lookup_existing_symbol(symbol_name);
@@ -304,7 +304,7 @@ const Symbol * symbol = Workspace::lookup_existing_symbol(symbol_name);
                         }
                     res.append(UNI_ASCII_R_CURLY);
 
-                    result.push_back(res);
+                    result.append(res);
                   }
                else
                   {
@@ -314,8 +314,8 @@ const Symbol * symbol = Workspace::lookup_existing_symbol(symbol_name);
                        {
                          if (text[u] == '\n')
                              {
-                               result.push_back(res);
-                               res.clear();
+                               result.append(res);
+                               res.shrink(0);
                                UCS_string next(text, u+1, text.size()-(u+1));
                                if (!next.is_comment_or_label() &&
                                    u < (text.size() - 1))
@@ -328,7 +328,7 @@ const Symbol * symbol = Workspace::lookup_existing_symbol(symbol_name);
                        }
                     res.append(ufun.get_exec_properties()[0]
                                ? UNI_DEL_TILDE : UNI_NABLA);
-                    result.push_back(res);
+                    result.append(res);
                   }
                return;
              }
@@ -338,7 +338,7 @@ const Symbol * symbol = Workspace::lookup_existing_symbol(symbol_name);
 }
 //-----------------------------------------------------------------------------
 void
-Quad_CR::do_CR10_var(vector<UCS_string> & result, const UCS_string & var_name,
+Quad_CR::do_CR10_var(UCS_string_vector & result, const UCS_string & var_name,
                      const Value & value)
 {
 Picker picker(var_name);
@@ -346,7 +346,7 @@ Picker picker(var_name);
 }
 //-----------------------------------------------------------------------------
 void
-Quad_CR::do_CR10_rec(vector<UCS_string> & result, const Value & value, 
+Quad_CR::do_CR10_rec(UCS_string_vector & result, const Value & value, 
                      Picker & picker, ShapeItem pidx)
 {
    /*
@@ -390,21 +390,24 @@ UCS_string shape_rho;
 
    if (value.element_count() == 0)   // empty value
       {
+        UCS_string reshape(2*(picker.get_level() - 1), UNI_ASCII_SPACE);
+        Value_P proto = value.prototype(LOC);
+
+        // emit one line for the prototype
         UCS_string proto_name;
         picker.get(proto_name);
         picker.pop();
+        do_CR10_rec(result, *proto, picker, pidx);
+        result.last().append_utf8(" ⍝ proto 1");
 
-        Value_P proto = value.prototype(LOC);
-        do_CR10_rec(result, *proto, picker, 0);
-        result.back().append_utf8(" ⍝ prototype...");
-
-        // reshape the prototype
+        // and then another line to reshape the prototype
         //
-        UCS_string reshape = proto_name;
+        reshape.append(proto_name);
         reshape.append_utf8("←");
         reshape.append(shape_rho);
         reshape.append(proto_name);
-        result.push_back(reshape);
+        result.append(reshape);
+        result.last().append_utf8(" ⍝ proto 2");
         return;
       }
 
@@ -415,7 +418,7 @@ bool nested = false;
    //
    if (short_ravel(result, nested, value, picker, left, shape_rho))
       {
-        result.push_back(compute_prolog(picker.get_level(), left, value));
+        result.append(compute_prolog(picker.get_level(), left, value));
 
         ShapeItem pos = 0;
         V_mode mode = Vm_NONE;
@@ -450,8 +453,8 @@ bool nested = false;
                    pos += count;
                    count = 0;
 
-                   result.push_back(line);
-                   rhs.clear();
+                   result.append(line);
+                   rhs.shrink(0);
                    mode = Vm_NONE;
                  }
 
@@ -471,7 +474,7 @@ bool nested = false;
      line.append_utf8("←");
      line.append(rhs);
 
-     result.push_back(line);
+     result.append(line);
    }
 
    // step 2: nested items
@@ -492,7 +495,7 @@ bool nested = false;
 }
 //-----------------------------------------------------------------------------
 bool
-Quad_CR::short_ravel(vector<UCS_string> & result, bool & nested,
+Quad_CR::short_ravel(UCS_string_vector & result, bool & nested,
                      const Value & value, const Picker & picker,
                      const UCS_string & left, const UCS_string & shape_rho)
 {
@@ -546,7 +549,7 @@ V_mode mode = Vm_NONE;
    //
    close_mode(line, mode);
 
-   result.push_back(line);
+   result.append(line);
    return false;
 }
 //-----------------------------------------------------------------------------
@@ -639,7 +642,7 @@ const bool need_disclose = get_level() > 1;
 
    // , is only needed if the picked item is not a true vector
    //
-const bool need_comma = shapes.back().get_rank() != 1;
+const bool need_comma = shapes.last().get_rank() != 1;
 
    result.append_utf8("(");
    if (need_comma)      result.append_utf8(",");
@@ -679,7 +682,13 @@ Quad_CR::do_CR10_item(UCS_string & item, const Cell & cell, V_mode mode,
 
    if (cell.is_integer_cell())
       {
-        item.append_number(cell.get_int_value());
+        APL_Integer aint = cell.get_int_value();
+        if (aint < 0)
+           {
+             item.append(UNI_OVERBAR);
+             aint = -aint;
+           }
+        item.append_number(aint);
         return Vm_NUM;
       }
 
