@@ -34,6 +34,8 @@
 
 #include "Bif_OPER2_INNER.hh"
 #include "Bif_OPER2_OUTER.hh"
+#include "Bif_OPER1_EACH.hh"
+#include "Common.hh"
 #include "FloatCell.hh"
 #include "IntCell.hh"
 #include "PointerCell.hh"
@@ -42,6 +44,8 @@
 #include "Tokenizer.hh"
 #include "Workspace.hh"
 
+extern long long top_of_memory();
+
 Quad_FIO  Quad_FIO::_fun;
 Quad_FIO * Quad_FIO::fun = &Quad_FIO::_fun;
 
@@ -59,11 +63,11 @@ Quad_FIO::Quad_FIO()
    // init stdin, stdout, and stderr
    //
 file_entry f0(stdin, STDIN_FILENO);
-   open_files.push_back(f0);
+   open_files.append(f0);
 file_entry f1(stdout, STDOUT_FILENO);
-   open_files.push_back(f1);
+   open_files.append(f1);
 file_entry f2(stderr, STDERR_FILENO);
-   open_files.push_back(f2);
+   open_files.append(f2);
 }
 //-----------------------------------------------------------------------------
 void
@@ -72,13 +76,13 @@ Quad_FIO::clear()
    // close open files, but leave stdin, stdout, and stderr open
    while(open_files.size() > 3)
       {
-         file_entry & fe = open_files.back();
+         file_entry & fe = open_files.last();
          if (fe.fe_FILE)   fclose(fe.fe_FILE);   // also closes fe.fe_fd
          else              close(fe.fe_fd);
          CERR << "WARNING: File " << fe.path << " still open - closing it"
               << endl;
-        ::close(open_files.back().fe_fd);
-        open_files.pop_back();
+        ::close(open_files.last().fe_fd);
+        open_files.pop();
       }
 }
 //-----------------------------------------------------------------------------
@@ -290,12 +294,9 @@ const Value & format = *A->get_ravel(a++).get_pointer_value();
 
                      default:
                         {
-                          UCS_string & t4 = Workspace::more_error();
-                          t4.clear();
-                          t4.append_utf8("invalid format character ");
-                          t4.append(un1, LOC);
-                          t4.append_utf8(" in function 22 (aka. printf())"
-                                         " in module file_io:: ");
+                          MORE_ERROR() << "invalid format character " << un1
+                                       << " in function 22 (aka. printf())"
+                                         " in module file_io:: ";
                           DOMAIN_ERROR;   // bad format char
                         }
                   }
@@ -426,8 +427,8 @@ Unicode lookahead = fget_utf8(file, fget_count);
 
         if (conv == Unicode_0)
            {
-             Workspace::more_error() = UCS_string(
-    "expecting conversion character %, c, d, f, i, n, o, u, s, or x after %");
+             MORE_ERROR() << "expecting conversion character "
+                             "%, c, d, f, i, n, o, u, s, or x after %";
              DOMAIN_ERROR;
            }
 
@@ -526,6 +527,12 @@ Quad_FIO::list_functions(ostream & out)
 "           i - integer\n"
 "           n - names (nested vector of strings)\n"
 "           s - string\n"
+"           u - time divisor: 1       - second\n"
+"                             1000    - milli second\n"
+"                             1000000 - micro second\n"
+"           y4 - seconds, wday, yday, dst (\n"
+"           y67- year, mon, day, hour, minute, second, [dst]\n"
+"           y9 - year, mon, day, hour, minute, second, wday, yday, dst\n"
 "           A1, A2, ...  nested vector with elements A1, A2, ...\n"
 "\n"
 "           ⎕FIO     ''    print this text on stderr\n"
@@ -594,6 +601,13 @@ Quad_FIO::list_functions(ostream & out)
 "   Zi ← Ai ⎕FIO[46] Bh    getsockopt(Bh, A_level, A_optname, Zi)\n"
 "   Ze ← Ai ⎕FIO[47] Bh    setsockopt(Bh, A_level, A_optname, A_optval)\n"
 "   Ze ← As ⎕FIO[48] Bh    fscanf(Bh, As)\n"
+"   Zs ←    ⎕FIO[49] Bs    return entire file Bs as nested lines\n"
+"   Zi ←    ⎕FIO[50] Bu    gettimeofday()\n"
+"   Zy4←    ⎕FIO[51] By67  mktime(By67)  Note: Jan 2, 2017 is: 2017 1 2 ...\n"
+"   Zy9←    ⎕FIO[52] Bi    localtime(Bi) Note: Jan 2, 2017 is: 2017 1 2 ...\n"
+"   Zy9←    ⎕FIO[53] Bi    gmtime(Bi)    Note: Jan 2, 2017 is: 2017 1 2 ...\n"
+"   Zi ←    ⎕FIO[54] Bs    chdir(Bs)\n"
+
 "\n"
 "Benchmarking functions:\n"
 "\n"
@@ -617,11 +631,23 @@ Quad_FIO::eval_B(Value_P B)
 const APL_Integer what = B->get_ravel(0).get_int_value();
    switch(what)
       {
-        // what < 0 are "hacker functions" that should no be used by
+        // what < 0 are "hacker functions" that should not be used by
         // normal mortals.
         //
+        case -14: // print stacks. SI_top is the ⎕FIO call, so dont show it.
+                  for (StateIndicator * si = Workspace::SI_top()->get_parent();
+                       si; si = si->get_parent())
+                      {
+                        CERR << "[" << si->get_level() << "]: ";
+                        si->get_prefix().print_stack(CERR, ".");
+                      }
+                   return Token(TOK_APL_VALUE1, IntScalar(0, LOC));
+
+        case -13: // total number of UCS strings
+             return Token(TOK_APL_VALUE1,
+                          IntScalar(UCS_string::get_total_count(), LOC));
         case -12: // sbrk()
-             return Token(TOK_APL_VALUE1, IntScalar((uint64_t)(sbrk(0)), LOC));
+             return Token(TOK_APL_VALUE1, IntScalar(top_of_memory(), LOC));
         case -11: // fast new
              return Token(TOK_APL_VALUE1, IntScalar(Value::fast_new, LOC));
         case -10: // slow new
@@ -760,12 +786,22 @@ const APL_Integer what = B->get_ravel(0).get_int_value();
 }
 //-----------------------------------------------------------------------------
 Token
+Quad_FIO::eval_LXB(Token & LO, Value_P X, Value_P B)
+{
+const APL_Integer function_number = X->get_ravel(0).get_int_value();
+   if (function_number != 49)   DOMAIN_ERROR;
+
+Token lines_B = eval_XB(X, B);
+   return Bif_OPER1_EACH::fun->eval_LB(LO, lines_B.get_apl_val());
+}
+//-----------------------------------------------------------------------------
+Token
 Quad_FIO::eval_XB(Value_P X, Value_P B)
 {
    if (B->get_rank() > 1)   RANK_ERROR;
    if (X->get_rank() > 1)   RANK_ERROR;
 
-const int function_number = X->get_ravel(0).get_near_int();
+const APL_Integer function_number = X->get_ravel(0).get_near_int();
 
    switch(function_number)
       {
@@ -777,7 +813,8 @@ const int function_number = X->get_ravel(0).get_near_int();
 
          case 2:   // return strerror(B)
               {
-                const int b = B->get_ravel(0).get_near_int();
+                int b = B->get_ravel(0).get_near_int();
+                if (b < 0)   b = -b;
                 const char * text = strerror(b);
                 const int len = strlen(text);
                 Value_P Z(len, LOC);
@@ -798,7 +835,7 @@ const int function_number = X->get_ravel(0).get_near_int();
                 file_entry fe(f, fileno(f));
                 fe.path = path;
                 fe.fe_may_read = true;
-                open_files.push_back(fe);
+                open_files.append(fe);
                 return Token(TOK_APL_VALUE1, IntScalar(fe.fe_fd, LOC));
               }
 
@@ -813,8 +850,8 @@ const int function_number = X->get_ravel(0).get_near_int();
                 if (fe.fe_FILE)   fclose(fe.fe_FILE);   // also closes fe.fe_fd
                 else              close(fe.fe_fd);
 
-                fe = open_files.back();       // move last file to fe
-                open_files.pop_back();        // erase last file
+                fe = open_files.last();       // move last file to fe
+                open_files.pop();        // erase last file
                 goto out_errno;
               }
 
@@ -958,7 +995,7 @@ const int function_number = X->get_ravel(0).get_near_int();
 
                 file_entry fe(f, fileno(f));
                 fe.fe_may_read = true;
-                open_files.push_back(fe);
+                open_files.append(fe);
                 return Token(TOK_APL_VALUE1, IntScalar(fe.fe_fd,LOC));
               }
 
@@ -969,8 +1006,8 @@ const int function_number = X->get_ravel(0).get_near_int();
                 int err = EBADF;   /* Bad file number */
                 if (fe.fe_FILE)   err = pclose(fe.fe_FILE);
 
-                fe = open_files.back();       // move last file to fe
-                open_files.pop_back();        // erase last file
+                fe = open_files.last();       // move last file to fe
+                open_files.pop();        // erase last file
 
                 if (err == -1)   goto out_errno;   // pclose() failed
                 return Token(TOK_APL_VALUE1, IntScalar(err,LOC));
@@ -993,15 +1030,13 @@ const int function_number = X->get_ravel(0).get_near_int();
                 if (!S_ISREG(st.st_mode))
                    {
                      close(fd);
-                     Workspace::more_error() = UCS_string(path);
-                     Workspace::more_error().append_utf8(
-                                " is not a regular file");
+                     MORE_ERROR() << path << " is not a regular file";
                      DOMAIN_ERROR;
                    }
 
-                ShapeItem len = st.st_size;
-                unsigned char * data = (unsigned char *)mmap(0, len, PROT_READ,
-                                MAP_SHARED, fd, 0);
+                const ShapeItem len = st.st_size;
+                uint8_t * data = (uint8_t *)mmap(0, len, PROT_READ,
+                                  MAP_SHARED, fd, 0);
                 close(fd);
                 if (data == 0)   goto out_errno;
 
@@ -1023,7 +1058,7 @@ const int function_number = X->get_ravel(0).get_near_int();
                 DIR * dir = opendir(path.c_str());
                 if (dir == 0)   goto out_errno;
 
-                vector<struct dirent> entries;
+                Simple_string<struct dirent, false> entries;
                 for (;;)
                     {
                       dirent * entry = readdir(dir);
@@ -1039,7 +1074,7 @@ const int function_number = X->get_ravel(0).get_near_int();
                                continue;
                           }
 
-                      entries.push_back(*entry);
+                      entries.append(*entry);
                     }
                 closedir(dir);
 
@@ -1110,7 +1145,7 @@ const int function_number = X->get_ravel(0).get_near_int();
                 file_entry fe(0, sock);
                 fe.fe_may_read = true;
                 fe.fe_may_write = true;
-                open_files.push_back(fe);
+                open_files.append(fe);
                 return Token(TOK_APL_VALUE1, IntScalar(fe.fe_fd, LOC));
 	      }
 
@@ -1132,7 +1167,7 @@ const int function_number = X->get_ravel(0).get_near_int();
                 if (sock == -1)   goto out_errno;
 
                 file_entry nfe (0, sock);
-                open_files.push_back(nfe);
+                open_files.append(nfe);
 
                 Value_P Z(4, LOC);
                 new (Z->next_ravel())   IntCell(nfe.fe_fd);
@@ -1190,8 +1225,8 @@ const int function_number = X->get_ravel(0).get_near_int();
                           {
                             const APL_Integer fd =
                                   vex->get_ravel(l).get_int_value();
-                            if (fd < 0)                  DOMAIN_ERROR;
-                            if (fd > 8*sizeof(fd_set))   DOMAIN_ERROR;
+                            if (fd < 0)                         DOMAIN_ERROR;
+                            if (fd > (int)(8*sizeof(fd_set)))   DOMAIN_ERROR;
                             FD_SET(fd, &exceptfds);
                             if (max_fd < fd)   max_fd = fd;
                             ex = &exceptfds;
@@ -1205,8 +1240,8 @@ const int function_number = X->get_ravel(0).get_near_int();
                           {
                             const APL_Integer fd =
                                   vwr->get_ravel(l).get_int_value();
-                            if (fd < 0)                  DOMAIN_ERROR;
-                            if (fd > 8*sizeof(fd_set))   DOMAIN_ERROR;
+                            if (fd < 0)                         DOMAIN_ERROR;
+                            if (fd > (int)(8*sizeof(fd_set)))   DOMAIN_ERROR;
                             FD_SET(fd, &writefds);
                             if (max_fd < fd)   max_fd = fd;
                             wr = &writefds;
@@ -1220,8 +1255,8 @@ const int function_number = X->get_ravel(0).get_near_int();
                           {
                             const APL_Integer fd =
                                   vrd->get_ravel(l).get_int_value();
-                            if (fd < 0)                  DOMAIN_ERROR;
-                            if (fd > 8*sizeof(fd_set))   DOMAIN_ERROR;
+                            if (fd < 0)                         DOMAIN_ERROR;
+                            if (fd > (int)(8*sizeof(fd_set)))   DOMAIN_ERROR;
                             FD_SET(fd, &readfds);
                             if (max_fd < fd)   max_fd = fd;
                             rd = &readfds;
@@ -1295,7 +1330,6 @@ const int function_number = X->get_ravel(0).get_near_int();
                 return Token(TOK_APL_VALUE1, Z);
               }
 
-
          case 200:   // clear statistics Bi
          case 201:   // get statistics Bi
               {
@@ -1344,8 +1378,158 @@ const int function_number = X->get_ravel(0).get_near_int();
                     }
               }
 
+         case 49:   // read entire file as nested lines
+              {
+                errno = 0;
+                UTF8_string path(*B.get());
+                int fd = open(path.c_str(), O_RDONLY);
+                if (fd == -1)   goto out_errno;
+
+                struct stat st;
+                if (fstat(fd, &st))
+                   {
+                     close(fd);
+                     goto out_errno;
+                   }
+
+                if (!S_ISREG(st.st_mode))
+                   {
+                     close(fd);
+                     MORE_ERROR() << path << " is not a regular file";
+                     DOMAIN_ERROR;
+                   }
+
+                const ShapeItem len = st.st_size;
+                uint8_t * data = (uint8_t *)mmap(0, len, PROT_READ,
+                                  MAP_SHARED, fd, 0);
+                close(fd);
+                if (data == 0)   goto out_errno;
+
+                // count number of LFs in the file
+                //
+                ShapeItem line_count = 0;
+                loop(l, len)
+                    {
+                      if (data[l] == '\n')   ++line_count;
+                    }
+                if (data[len - 1] != '\n')   ++line_count;
+
+                Value_P Z(line_count, LOC);
+                Z->set_proto_Spc();
+
+                uint8_t * from = data;
+                loop(l, len)
+                    {
+                      if (data[l] != '\n')   continue;
+
+                      uint8_t * end = data + l;
+                      if (end[-1] == '\r')   --end;   // discard trailing CR
+                      UTF8_string utf(from, end - from);
+                      UCS_string ucs(utf);
+                      Value_P ZZ(ucs, LOC);
+                      new (Z->next_ravel())  PointerCell(ZZ, Z.getref());
+                      from = data + l + 1;
+                    }
+
+                if (data[len - 1] != '\n')   // incomplete final line
+                   {
+                      uint8_t * end = data + len;
+                      if (end[-1] == '\r')   --end;   // discard trailing CR
+                      UTF8_string utf(from, end - from);
+                      UCS_string ucs(utf);
+                      Value_P ZZ(ucs, LOC);
+                      new (Z->next_ravel())  PointerCell(ZZ, Z.getref());
+                   }
+
+                munmap((char *)data, len);
+
+                Z->set_default_Spc();
+                Z->check_value(LOC);
+                return Token(TOK_APL_VALUE1, Z);
+              }
+
+         case 50:   // gettimeofday
+              {
+                const APL_Integer unit = B->get_ravel(0).get_near_int();
+                timeval tv;
+                gettimeofday(&tv, 0);
+                int64_t usec = tv.tv_sec;
+                usec *= 1000000;
+                usec += tv.tv_usec;
+                APL_Integer z = 0;
+                if      (unit == 1)         z = usec/1000000;
+                else if (unit == 1000)      z = usec/1000;
+                else if (unit == 1000000)   z = usec;
+                else
+                   {
+                     MORE_ERROR() <<
+       "Invalid time unit (use 1 for seconds, 1000 for ms, or 1000000 for μs)";
+                     DOMAIN_ERROR;
+                   }
+                return Token(TOK_APL_VALUE1, IntScalar(z, LOC));
+              }
+
+         case 51:   // mktime
+              {
+                if (B->element_count() < 6 || B->element_count() > 9)
+                   LENGTH_ERROR;
+
+                tm t;
+                t.tm_year = B->get_ravel(0).get_int_value() - 1900;
+                t.tm_mon  = B->get_ravel(1).get_int_value() - 1;
+                t.tm_mday = B->get_ravel(2).get_int_value();
+                t.tm_hour = B->get_ravel(3).get_int_value();
+                t.tm_min  = B->get_ravel(4).get_int_value();
+                t.tm_sec  = B->get_ravel(5).get_int_value();
+                if (B->element_count() > 6)   // dst provided
+                   t.tm_isdst = B->get_ravel(6).get_int_value();
+                else
+                   t.tm_isdst = -1;
+
+               const time_t seconds = mktime(&t);
+               if (seconds == (time_t)-1)   DOMAIN_ERROR;
+
+                Value_P Z(4, LOC);
+                new (Z->next_ravel()) IntCell(seconds);
+                new (Z->next_ravel()) IntCell(t.tm_wday);
+                new (Z->next_ravel()) IntCell(t.tm_yday);
+                new (Z->next_ravel()) IntCell(t.tm_isdst);
+                Z->check_value(LOC);
+                return Token(TOK_APL_VALUE1, Z);
+              }
+
+         case 52:   // localtime
+         case 53:   // gmtime
+              {
+                if (B->element_count() != 1)   LENGTH_ERROR;
+                const time_t t = B->get_ravel(0).get_int_value();
+                const tm * tmp = (function_number == 52) ? localtime(&t)
+                                                         : gmtime(&t);
+                if (tmp == 0)   DOMAIN_ERROR;
+
+                Value_P Z(9, LOC);
+                new (Z->next_ravel()) IntCell(tmp->tm_year + 1900);
+                new (Z->next_ravel()) IntCell(tmp->tm_mon + 1);
+                new (Z->next_ravel()) IntCell(tmp->tm_mday);
+                new (Z->next_ravel()) IntCell(tmp->tm_hour);
+                new (Z->next_ravel()) IntCell(tmp->tm_min);
+                new (Z->next_ravel()) IntCell(tmp->tm_sec);
+                new (Z->next_ravel()) IntCell(tmp->tm_wday);
+                new (Z->next_ravel()) IntCell(tmp->tm_yday);
+                new (Z->next_ravel()) IntCell(tmp->tm_isdst);
+                Z->check_value(LOC);
+                return Token(TOK_APL_VALUE1, Z);
+              }
+
+         case 54:    // chdir
+              {
+                UTF8_string path(*B.get());
+                errno = chdir(path.c_str());
+                goto out_errno;
+              }
+ 
          case 202:   // get monadic parallel threshold
-         case 203:   // get dyadicadic parallel threshold
+         case 203:   // get dyadic  parallel threshold
               {
                 const Function * fun = 0;
                 if (B->element_count() == 3)   // dyadic operator
@@ -1384,6 +1568,9 @@ const int function_number = X->get_ravel(0).get_near_int();
         default: break;
       }
 
+   MORE_ERROR() << "bad function number " << function_number <<
+                   " in Quad_FIO::eval_XB()";
+
    CERR << "Bad eval_XB() function number: " << function_number << endl;
    DOMAIN_ERROR;
 
@@ -1429,7 +1616,7 @@ const int function_number = X->get_ravel(0).get_near_int();
                 fe.path = path;
                 fe.fe_may_read = read;
                 fe.fe_may_write = write;
-                open_files.push_back(fe);
+                open_files.append(fe);
                 return Token(TOK_APL_VALUE1, IntScalar(fe.fe_fd,LOC));
               }
 
@@ -1574,7 +1761,7 @@ const int function_number = X->get_ravel(0).get_near_int();
                 file_entry fe(f, fileno(f));
                 fe.fe_may_read = read;
                 fe.fe_may_write = write;
-                open_files.push_back(fe);
+                open_files.append(fe);
                 return Token(TOK_APL_VALUE1, IntScalar(fe.fe_fd, LOC));
               }
 
@@ -1830,6 +2017,9 @@ const int function_number = X->get_ravel(0).get_near_int();
         default: break;
       }
 
+   MORE_ERROR() << "bad function number " << function_number <<
+                   " in Quad_FIO::eval_AXB()";
+
    CERR << "eval_AXB() function number: " << function_number << endl;
    DOMAIN_ERROR;
 
