@@ -189,8 +189,7 @@ const ShapeItem ec = val->element_count();
                 }
              else
                 {
-                  Workspace::more_error() =
-                             UCS_string("Non-number in 1 ⎕TF N record");
+                  MORE_ERROR() << "Non-number in 1 ⎕TF N record";
                   return Value_P();
                 }
            }
@@ -203,7 +202,7 @@ Value_P
 Quad_TF::tf1(const UCS_string & fun_name, const Function & fun)
 {
 const UCS_string text = fun.canonical(false);
-vector<UCS_string> lines;
+UCS_string_vector lines;
 const size_t max_len = text.to_vector(lines);
 
 UCS_string ucs;
@@ -229,12 +228,11 @@ UCS_string ucs;
 Value_P
 Quad_TF::tf1_inv(const UCS_string & ravel)
 {
-UCS_string & more = Workspace::more_error();
-const size_t len = ravel.size();
+const int len = ravel.size();
 
    if (len < 2)
       {
-        more = UCS_string("1 ⎕TF record too short");
+        MORE_ERROR() << "1 ⎕TF record too short";
         return Value_P();
       }
 
@@ -242,14 +240,14 @@ const size_t len = ravel.size();
 const Unicode mode = ravel[0];
    if (mode != 'C' && mode != 'F' && mode != 'N')
       {
-        more = UCS_string("1 ⎕TF record type not F, N, or C");
+        MORE_ERROR() << "1 ⎕TF record type not F, N, or C";
         return Value_P();
       }
 
 UCS_string name(ravel[1]);
    if (!Avec::is_quad(name[0]) && !Avec::is_first_symbol_char(name[0]))
       {
-        more = UCS_string("Bad variable name in 1 ⎕TF record");
+        MORE_ERROR() << "Bad variable name in 1 ⎕TF record";
         return Value_P();
       }
 
@@ -259,13 +257,13 @@ ShapeItem idx = 2;
 
    if (Avec::is_quad(name[0]) && name.size() == 1)
       {
-        more = UCS_string("Bad variable name in 1 ⎕TF record");
+        MORE_ERROR() << "Bad variable name in 1 ⎕TF record";
         return Value_P();
       }
 
    if (ravel[idx++] != UNI_ASCII_SPACE)
       {
-        more = UCS_string("missing space (before rank) in 1 ⎕TF record");
+        MORE_ERROR() << "missing space (before rank) in 1 ⎕TF record";
         return Value_P();
       }
 
@@ -276,19 +274,19 @@ Rank rank = 0;
 
    if (rank == 0 && idx0 == idx)
       {
-        more = UCS_string("missing rank in 1 ⎕TF record");
+        MORE_ERROR() << "missing rank in 1 ⎕TF record";
         return Value_P();
       }
 
    if (ravel[idx++] != UNI_ASCII_SPACE)
       {
-        more = UCS_string("missing space (after rank) in 1 ⎕TF record");
+        MORE_ERROR() << "missing space (after rank) in 1 ⎕TF record";
         return Value_P();
       }
 
    if (rank > MAX_RANK)
       {
-        more = UCS_string("max. rank exceeded in 1 ⎕TF record");
+        MORE_ERROR() << "max. rank exceeded in 1 ⎕TF record";
         return Value_P();
       }
 
@@ -301,13 +299,13 @@ Shape shape;
            sh = 10 * sh + ravel[idx++] - UNI_ASCII_0;
         if (sh == 0 && idx0 == idx)   // no shape
            {
-             more = UCS_string("too few shape items in 1 ⎕TF record");
+             MORE_ERROR() << "too few shape items in 1 ⎕TF record";
              return Value_P();
            }
 
         if (ravel[idx++] != UNI_ASCII_SPACE)
            {
-             more = UCS_string("missing space (in shape) in 1 ⎕TF record");
+             MORE_ERROR() << "missing space (in shape) in 1 ⎕TF record";
              return Value_P();
            }
         shape.add_shape_item(sh);
@@ -322,31 +320,30 @@ NamedObject * sym_or_fun = Workspace::lookup_existing_name(name);
         nc = sym_or_fun->get_nc();
         if (symbol && symbol->is_readonly())
            {
-             more = UCS_string("symbol cannot be modified in 1 ⎕TF record");
+             MORE_ERROR() << "symbol cannot be modified in 1 ⎕TF record";
              return Value_P();
            }
      }
 
-const size_t data_chars = len - idx;
+const int data_chars = len - idx;
 
    if (mode == UNI_ASCII_F)   // function
       {
         if (rank != 2)
            {
-             more = UCS_string("function text is not a matrix in 1 ⎕TF record");
+             MORE_ERROR() << "function text is not a matrix in 1 ⎕TF record";
              return Value_P();
            }
 
         if (nc != NC_UNUSED_USER_NAME && nc != NC_FUNCTION && nc != NC_OPERATOR)
            {
-             more = UCS_string(
-                    "symbol is an existing variable in 1 ⎕TF F record");
+             MORE_ERROR() << "symbol is an existing variable in 1 ⎕TF F record";
              return Value_P();
            }
 
         if (data_chars != shape.get_volume())
            {
-             more = UCS_string("item count mismatch in 1 ⎕TF N record");
+             MORE_ERROR() << "item count mismatch in 1 ⎕TF N record";
              return Value_P();
            }
 
@@ -361,13 +358,13 @@ const size_t data_chars = len - idx;
       {
         if (data_chars != shape.get_volume())
            {
-             more = UCS_string("item count mismatch in 1 ⎕TF C record");
+             MORE_ERROR() << "item count mismatch in 1 ⎕TF C record";
              return Value_P();
            }
         if (nc != NC_UNUSED_USER_NAME && nc != NC_VARIABLE)
            {
-             more = UCS_string(
-                    "symbol is existing and not a variable in 1 ⎕TF C record");
+             MORE_ERROR() <<
+             "symbol is existing and not a variable in 1 ⎕TF C record";
              return Value_P();
            }
 
@@ -384,7 +381,7 @@ const size_t data_chars = len - idx;
       {
         if (nc != NC_UNUSED_USER_NAME && nc != NC_VARIABLE)
            {
-             more = UCS_string("symbol cannot be assigned in 1 ⎕TF N record");
+             MORE_ERROR() << "symbol cannot be assigned in 1 ⎕TF N record";
              return Value_P();
            }
 
@@ -393,12 +390,12 @@ const size_t data_chars = len - idx;
         Token_string tos;
         if (tokenizer.tokenize(data, tos) != E_NO_ERROR)
            {
-             more = UCS_string("tokenization failed in 1 ⎕TF N record");
+             MORE_ERROR() << "tokenization failed in 1 ⎕TF N record";
              return Value_P();
            }
         if (tos.size() != shape.get_volume())
            {
-             more = UCS_string("item count mismatch in 1 ⎕TF N record");
+             MORE_ERROR() << "item count mismatch in 1 ⎕TF N record";
              return Value_P();
            }
 
@@ -410,7 +407,7 @@ const size_t data_chars = len - idx;
              if (tag == TOK_INTEGER)   continue;
              if (tag == TOK_REAL)      continue;
              if (tag == TOK_COMPLEX)   continue;
-             more = UCS_string( "Non-number in 1 ⎕TF N record");
+             MORE_ERROR() <<  "Non-number in 1 ⎕TF N record";
              return Value_P();
            }
 
@@ -787,11 +784,11 @@ ShapeItem skipped = 0;
              for (; s < tos.size() && tos[s].get_Class() == TC_VALUE; ++s)
                 {
                   tos[s].get_apl_val()->toggle_UCS();
-                  move_1(tos[s - skipped], tos[s], LOC);
+                  tos[s - skipped].move_1(tos[s], LOC);
                 }
            }
 
-        if (skipped)   move_1(tos[s - skipped], tos[s], LOC);
+        if (skipped)   tos[s - skipped].move_1(tos[s], LOC);
 
       }
 
@@ -814,7 +811,7 @@ ShapeItem skipped = 0;
             tos[s + 2].get_Class() != TC_VALUE)                     // B 
            {
              if (skipped)   // dont copy to itself
-                move_1(tos[s - skipped], tos[s], LOC);
+                tos[s - skipped].move_1(tos[s], LOC);
              continue;
            }
 
@@ -829,14 +826,14 @@ ShapeItem skipped = 0;
         Value_P bval = tos[s].get_apl_val();
         if (sh.get_volume() == bval->element_count())   // same volume
            {
-             move_1(tos[s - skipped], tos[s], LOC);
+             tos[s - skipped].move_1(tos[s], LOC);
              tos[s - skipped].ChangeTag(TOK_APL_VALUE1);
              bval->set_shape(sh);
            }
         else
            {
              Token t = Bif_F12_RHO::do_reshape(sh, *bval);
-             move_1(tos[s - skipped], t, LOC);
+             tos[s - skipped].move_1(t, LOC);
            }
       }
 
@@ -865,7 +862,7 @@ ShapeItem skipped = 0;
                 {
                   // TOK_R_PARENT will become a value: just copy it for now
                   //
-                  if (skipped)   move_1(tos[s - skipped], tos[s], LOC);
+                  if (skipped)   tos[s - skipped].move_1(tos[s], LOC);
                   continue;
                 }
 
@@ -883,7 +880,7 @@ ShapeItem skipped = 0;
 
                   tos[s + 1].clear(LOC);
                   Token tok_AB(TOK_APL_VALUE1, Z);
-                  move_2(tos[d_1], tok_AB, LOC);
+                  tos[d_1].move_2(tok_AB, LOC);
                   s += 1;   skipped += 2;   // skip , (of A , B) but not B
                   continue;   // don't move_1() below
                 }
@@ -896,11 +893,11 @@ ShapeItem skipped = 0;
              bval->set_shape(sh);
 
              tos[s].ChangeTag(TOK_APL_VALUE1);
-             move_1(tos[s - skipped], tos[s], LOC);
+             tos[s - skipped].move_1(tos[s], LOC);
              continue;
            }
 
-        if (skipped)   move_1(tos[s - skipped], tos[s], LOC);
+        if (skipped)   tos[s - skipped].move_1(tos[s], LOC);
       }
 
    if (skipped)
@@ -924,7 +921,7 @@ ShapeItem skipped = 0;
             tos[s + 2].get_Class() != TC_VALUE)              // not B 
            {
              if (skipped)   // dont copy to itself
-                move_1(tos[s - skipped], tos[s], LOC);
+                tos[s - skipped].move_1(tos[s], LOC);
              continue;
            }
 
@@ -961,7 +958,7 @@ ShapeItem skipped = 0;
            !tos[s + 5].get_apl_val()->is_int_scalar())      // K not integer
            {
              if (skipped)   // dont copy to itself
-                move_1(tos[s - skipped], tos[s], LOC);
+                tos[s - skipped].move_1(tos[s], LOC);
              continue;
            }
 
@@ -976,7 +973,7 @@ ShapeItem skipped = 0;
         loop(k, K)   new (sequence->next_ravel())   IntCell(N + k);
         sequence->check_value(LOC);
         Token tok(TOK_APL_VALUE1, sequence);
-        move_2(tos[s - skipped], tok, LOC);
+        tos[s - skipped].move_2(tok, LOC);
         s += 5;   skipped += 5;
       }
 
@@ -1013,7 +1010,7 @@ ShapeItem skipped = 0;
            !tos[s + 7].get_apl_val()->is_int_scalar())      // K not integer
            {
              if (skipped)   // dont copy to itself
-                move_1(tos[s - skipped], tos[s], LOC);
+                tos[s - skipped].move_1(tos[s], LOC);
              continue;
            }
 
@@ -1030,7 +1027,7 @@ ShapeItem skipped = 0;
         loop(k, K)   new (sequence->next_ravel())   IntCell(M * (N + k));
         sequence->check_value(LOC);
         Token tok(TOK_APL_VALUE1, sequence);
-        move_2(tos[s - skipped], tok, LOC);
+        tos[s - skipped].move_2(tok, LOC);
         s += 7;   skipped += 7;
       }
 
@@ -1057,21 +1054,21 @@ ShapeItem skipped = 0;
             tos[s + 3].get_tag()   != TOK_R_PARENT)          // not )
            {
              if (skipped)   // dont copy to itself
-                move_1(tos[s - skipped], tos[s], LOC);
+                tos[s - skipped].move_1(tos[s], LOC);
              continue;
            }
 
         Value_P B = tos[s + 2].get_apl_val();
         if (B->is_scalar())
            {
-             move_1(tos[s - skipped], tos[s + 2], LOC);
+             tos[s - skipped].move_1(tos[s + 2], LOC);
            }
         else
            {
              Value_P enc_B(LOC);
              new (enc_B->next_ravel()) PointerCell(B, enc_B.getref());
              Token tok(TOK_APL_VALUE1, enc_B);
-             move_2(tos[s - skipped], tok, LOC);
+             tos[s - skipped].move_2(tok, LOC);
              tos[s + 2].clear(LOC);   // B
            }
         s += 3;   skipped += 3;
@@ -1099,24 +1096,24 @@ ShapeItem skipped = 0;
             tos[s + 2].get_Class() != TC_VALUE)              // not B 
            {
              if (skipped)   // dont copy to itself
-                move_1(tos[s - skipped], tos[s], LOC);
+                tos[s - skipped].move_1(tos[s], LOC);
              continue;
            }
 
         if (skipped)   // dont copy to itself
-           move_1(tos[s - skipped], tos[s], LOC);   // move ⍴
+           tos[s - skipped].move_1(tos[s], LOC);   // move ⍴
 
         Value_P B = tos[s + 2].get_apl_val();
         if (B->is_scalar())
            {
-             move_1(tos[s - skipped + 1], tos[s + 2], LOC);
+             tos[s - skipped + 1].move_1(tos[s + 2], LOC);
            }
         else
            {
              Value_P enc_B(LOC);
              new (enc_B->next_ravel()) PointerCell(B, enc_B.getref());
              Token tok(TOK_APL_VALUE1, enc_B);
-             move_2(tos[s - skipped + 1], tok, LOC);
+             tos[s - skipped + 1].move_2(tok, LOC);
              tos[s + 2].clear(LOC);   // B
            }
         s += 2;   ++skipped;
@@ -1143,13 +1140,13 @@ ShapeItem skipped = 0;
             tos[s + 1].get_Class() == TC_VALUE     &&
             tos[s + 2].get_tag()   == TOK_R_PARENT)   // ( B )
            {
-             move_1(tos[s - skipped], tos[s + 1], LOC);  // override ( of (B)
+             tos[s - skipped].move_1(tos[s + 1], LOC);  // override ( of (B)
              s += 2;   skipped += 2;
              continue;
            }
 
         if (skipped)   // dont copy to itself
-           move_1(tos[s - skipped], tos[s], LOC);
+           tos[s - skipped].move_1(tos[s], LOC);
       }
 
    if (skipped)
@@ -1169,7 +1166,7 @@ ShapeItem skipped = 0;
         // the first value (if any) survives
         //
         if (skipped)   // dont copy to itself
-           move_1(tos[s - skipped], tos[s], LOC);
+           tos[s - skipped].move_1(tos[s], LOC);
 
         if (tos[s - skipped].get_Class() != TC_VALUE)   continue;
 
@@ -1179,7 +1176,7 @@ ShapeItem skipped = 0;
          while ((s + 1) < tos.size() && tos[s + 1].get_Class() == TC_VALUE)
             {
                Token t;
-               move_1(t, tos[s - skipped], LOC);
+               t.move_1(tos[s - skipped], LOC);
                Value::glue(tos[s - skipped], t, tos[s + 1], LOC);
                s += 1;   skipped += 1;
             }
@@ -1222,7 +1219,7 @@ const UCS_string text = fun.canonical(false);
         return;
       }
 
-vector<UCS_string> lines;
+UCS_string_vector lines;
    text.to_vector(lines);
 
    ucs.append_utf8("⎕FX");
