@@ -94,7 +94,13 @@ struct PJob_scalar_B
 {
    /// default constructor
    PJob_scalar_B()
-   : value_Z(*(Value *)0)
+   : value_Z(*(Value *)0),
+     len_Z(0),
+     cZ(0),
+     cB(0),
+     error(E_NO_ERROR),
+     fun(0),
+     fun1(0)
    {}
 
    /// assign \b other to \b this
@@ -107,7 +113,9 @@ struct PJob_scalar_B
      len_Z(Z.nz_element_count()),
      cZ(&Z.get_ravel(0)),
      cB(&B.get_ravel(0)),
-     error(E_NO_ERROR)
+     error(E_NO_ERROR),
+     fun(0),
+     fun1(0)
    {}
 
    /// the value being computed
@@ -141,14 +149,23 @@ struct PJob_scalar_B
 };
 
 /// all monadic scalar jobs
-static Parallel_job_list<PJob_scalar_B> joblist_B;
+static Parallel_job_list<PJob_scalar_B, false> joblist_B;
 
 /// one dyadic scalar job
 struct PJob_scalar_AB
 {
    /// default constructor
    PJob_scalar_AB()
-   : value_Z(*(Value *)0)
+   : value_Z(*(Value *)0),
+     len_Z(0),
+     cZ(0),
+     cA(0),
+     inc_A(0),
+     cB(0),
+     inc_B(0),
+     error(E_NO_ERROR),
+     fun(0),
+     fun2(0)
    {}
 
    /// assign \b other to \b this
@@ -164,7 +181,9 @@ struct PJob_scalar_AB
      inc_A(iA),
      cB(_cB),
      inc_B(iB),
-     error(E_NO_ERROR)
+     error(E_NO_ERROR),
+     fun(0),
+     fun2(0)
    {}
 
    /// A value (e.g parallel ~Value())
@@ -211,7 +230,7 @@ struct PJob_scalar_AB
 };
 
 /// all dyadic scalar jobs
-static Parallel_job_list<PJob_scalar_AB> joblist_AB;
+static Parallel_job_list<PJob_scalar_AB, false> joblist_AB;
 
 //-----------------------------------------------------------------------------
 Token
@@ -289,6 +308,7 @@ CELL_PERFORMANCE_END(get_statistics_B(), start_2, z)
                      }
                 }
            }
+        job->value_Z.check_value(LOC);
       }
 
    Z->check_value(LOC);
@@ -409,6 +429,7 @@ const Shape * shape_Z = 0;
    else if (A->same_shape(*B))   shape_Z = &B->get_shape();
    else 
       {
+        joblist_AB.cancel_jobs();
         if (!A->same_rank(*B))   RANK_ERROR;
         else                     LENGTH_ERROR;
       }
@@ -477,6 +498,7 @@ Value_P Z(*shape_Z, LOC);
 
                           if (inc_A1 && inc_B1 && !A1->same_shape(*B1))
                              {
+                               joblist_AB.cancel_jobs();
                                if (!A1->same_rank(*B1))   RANK_ERROR;
                                else                       LENGTH_ERROR;
                              }
@@ -574,6 +596,7 @@ CELL_PERFORMANCE_END(get_statistics_AB(), start_2, z)
                         }
                 }
            }
+        job->value_Z.check_value(LOC);
       }
 
    joblist_AB.cancel_jobs();
