@@ -98,6 +98,21 @@ StateIndicator::retry(const char * loc)
       CERR << endl << "RETRY " << loc << ")" << endl;
 }
 //-----------------------------------------------------------------------------
+bool
+StateIndicator::uses_function(const UserFunction * ufun) const
+{
+const Executable * uexec = ufun;
+
+   // case 1: ufun is the currently executing function
+   //
+   if (uexec == get_executable())   return true;
+
+   // case 2: ufun is used on the prefix parser stack
+   //
+   if (current_stack.uses_function(ufun))   return true;
+   return false;
+}
+//-----------------------------------------------------------------------------
 UCS_string
 StateIndicator::function_name() const
 {
@@ -123,7 +138,8 @@ StateIndicator::function_name() const
              }
       }
 
-   Assert(0);
+   FIXME;
+   return UCS_string();
 } 
 //-----------------------------------------------------------------------------
 void
@@ -138,16 +154,16 @@ StateIndicator::print(ostream & out) const
    switch(get_parse_mode())
       {
         case PM_FUNCTION:
-             out << "Pmode:    ∇ "
+             out << "Pmode:      ∇ "
                  << executable->get_ufun()->get_name_and_line(get_PC());
              break;
 
         case PM_STATEMENT_LIST:
-             out << "Pmode:    ◊ " << " " << executable->get_text(0);
+             out << "Pmode:      ◊ " << " " << executable->get_text(0);
              break;
 
         case PM_EXECUTE:
-             out << "Pmode:    ⍎ " << " " << executable->get_text(0);
+             out << "Pmode:      ⍎ " << " " << executable->get_text(0);
              break;
 
         default:
@@ -155,16 +171,18 @@ StateIndicator::print(ostream & out) const
       }
    out << endl;
 
-   out << "PC:       " << get_PC();
+   out << "PC:         " << get_PC() << " (" << executable->get_body().size()
+                       << ")";
    out << " " << executable->get_body()[get_PC()] << endl;
-   out << "Stat:     " << executable->statement_text(get_PC());
+   out << "Stat:       " << executable->statement_text(get_PC());
    out << endl;
 
-   out << "err_code: " << HEX(error.error_code) << endl;
-   out << "thrown:   at " << error.throw_loc << endl;
-   out << "e_msg_1:  '" << error.get_error_line_1() << "'" << endl;
-   out << "e_msg_2:  '" << error.get_error_line_2() << "'" << endl;
-   out << "e_msg_3:  '" << error.get_error_line_3() << "'" << endl;
+   out << "err_code:   " << HEX(error.error_code) << endl;
+   if (error.error_code)
+      out << "thrown at:  " << error.throw_loc << endl
+       << "e_msg_1:    '" << error.get_error_line_1() << "'" << endl
+       << "e_msg_2:    '" << error.get_error_line_2() << "'" << endl
+       << "e_msg_3:    '" << error.get_error_line_3() << "'" << endl;
 
    out << endl;
 }
@@ -172,8 +190,6 @@ StateIndicator::print(ostream & out) const
 void
 StateIndicator::list(ostream & out, SI_mode mode) const
 {
-   Assert(this);
-
    if (mode & SIM_debug)   // command ]SI or ]SIS
       {
         print(out);
@@ -453,7 +469,7 @@ Token * tok_L = current_stack.locate_L();
    if (tok_L == 0)   return;
 
 Value_P old_value = tok_L->get_apl_val();   // so that 
-   move_2(*tok_L, Token(tok_L->get_tag(), new_value), LOC);
+   tok_L->move_2(Token(tok_L->get_tag(), new_value), LOC);
 }
 //-----------------------------------------------------------------------------
 void
@@ -463,7 +479,7 @@ Token * tok_R = current_stack.locate_R();
    if (tok_R == 0)   return;
 
 Value_P old_value = tok_R->get_apl_val();   // so that 
-   move_2(*tok_R, Token(tok_R->get_tag(), new_value), LOC);
+   tok_R->move_2(Token(tok_R->get_tag(), new_value), LOC);
 }
 //-----------------------------------------------------------------------------
 void
@@ -536,7 +552,7 @@ bool print_value = tag == TOK_APL_VALUE1 || tag == TOK_APL_VALUE3;
    Quad_QUOTE::done(false, LOC);
 
 const int boxing_format = Command::get_boxing_format();
-   if (boxing_format == -1)
+   if (boxing_format == 0)   // no boxing
       {
         if (Quad_SYL::print_length_limit &&
             B->element_count() >= Quad_SYL::print_length_limit)
@@ -565,11 +581,20 @@ const int boxing_format = Command::get_boxing_format();
                      "⎕SYL[⎕IO + " << Quad_SYL::SYL_PRINT_LIMIT
                   << "] reached)  ***" << endl;
            }
-        else
+        else   // no print length limit or small B
            {
              B->print(COUT);
            }
       }
+   else if (boxing_format < 0)
+      {
+        const PrintContext pctx = Workspace::get_PrintContext(PST_NONE);
+        Value_P B1 = Quad_CR::do_CR(-boxing_format, B.get(), pctx);
+        if (B1->get_cols() >= Workspace::get_PW())   // too large
+           B->print(COUT);   // don't box
+        else
+           B1->print(COUT);   // do box
+      }
    else
       {
         const PrintContext pctx = Workspace::get_PrintContext(PST_NONE);
@@ -589,7 +614,8 @@ StateIndicator::get_parse_mode_name() const
      }
 
    CERR << "pmode = " << get_parse_mode() << endl;
-   Assert(0 && "bad pmode");
+   FIXME;
+   return Invalid_Unicode;
 }
 //-----------------------------------------------------------------------------
 
