@@ -43,8 +43,7 @@
 Symbol::Symbol(ID::Id id)
    : NamedObject(id),
      next(0),
-     name(UCS_string(UTF8_string(ID::name(id)))),
-     erased(false),
+     name(ID::get_name(id)),
      monitor_callback(0)
 {
    push();
@@ -54,7 +53,6 @@ Symbol::Symbol(const UCS_string & ucs, ID::Id id)
    : NamedObject(id),
      next(0),
      name(ucs),
-     erased(false),
      monitor_callback(0)
 {
    push();
@@ -135,7 +133,7 @@ Symbol::assign(Value_P new_value, bool clone, const char * loc)
         Assert(0);
       }
 
-ValueStackItem & vs = value_stack.back();
+ValueStackItem & vs = value_stack.last();
 
    switch(vs.name_class)
       {
@@ -312,7 +310,7 @@ const int incr_B = (ec_B == 1) ? 0 : 1;
 void
 Symbol::assign_named_lambda(Function * lambda, const char * loc)
 {
-ValueStackItem & vs = value_stack.back();
+ValueStackItem & vs = value_stack.last();
 UserFunction * ufun = lambda->get_ufun1();
    Assert(ufun);
 
@@ -344,25 +342,29 @@ Symbol::pop()
 {
    if (value_stack.size() == 0)
       {
-       CERR << "Symbol is: '" << get_name() << "' at " << LOC << endl;
+        CERR << "Symbol is: '" << get_name() << "' at " << LOC << endl;
         Assert(0 && LOC);
       }
 
-const ValueStackItem & vs = value_stack.back();
+const ValueStackItem & vs = value_stack.last();
 
    if (vs.name_class == NC_VARIABLE)
       {
-        Value_P ret = vs.apl_val;
         Log(LOG_SYMBOL_push_pop)
            {
+             Value_P ret = vs.apl_val;
              CERR << "-pop-value " << name
                   << " flags " << ret->get_flags() << " ";
              if (value_stack.size() == 0)   CERR << " (last)";
              CERR << " addr " << (const void *)ret.get() << endl;
            }
-        ptr_clear(ret, LOC);
 
-        value_stack.pop_back();
+        // unlike vector<ValueStackItem>, Simple_string<ValueStackItem>.pop()
+        // does not destruct vs.apl_val, so we do it here.
+        //
+        ptr_clear(value_stack.last().apl_val, LOC);
+
+        value_stack.pop();
         if (monitor_callback)   monitor_callback(*this, SEV_POPED);
       }
    else
@@ -374,7 +376,7 @@ const ValueStackItem & vs = value_stack.back();
              if (value_stack.size() == 0)   CERR << " (last)";
              CERR << endl;
            }
-        value_stack.pop_back();
+        value_stack.pop();
         if (monitor_callback)   monitor_callback(*this, SEV_POPED);
       }
 }
@@ -389,7 +391,7 @@ Symbol::push()
         CERR << endl;
       }
 
-   value_stack.push_back(ValueStackItem());
+   value_stack.append(ValueStackItem());
    if (monitor_callback)   monitor_callback(*this, SEV_PUSHED);
 }
 //-----------------------------------------------------------------------------
@@ -403,7 +405,7 @@ Symbol::push_label(Function_Line label)
         CERR << endl;
       }
 
-   value_stack.push_back(ValueStackItem(label));
+   value_stack.append(ValueStackItem(label));
    if (monitor_callback)   monitor_callback(*this, SEV_PUSHED);
 }
 //-----------------------------------------------------------------------------
@@ -421,7 +423,7 @@ ValueStackItem vs;
    if (function->is_operator())   vs.name_class = NC_OPERATOR;
    else                           vs.name_class = NC_FUNCTION;
    vs.sym_val.function = function;
-   value_stack.push_back(vs);
+   value_stack.append(vs);
    if (monitor_callback)   monitor_callback(*this, SEV_PUSHED);
 }
 //-----------------------------------------------------------------------------
@@ -429,7 +431,7 @@ void
 Symbol::push_value(Value_P value)
 {
 ValueStackItem vs;
-   value_stack.push_back(vs);
+   value_stack.append(vs);
    if (monitor_callback)   monitor_callback(*this, SEV_PUSHED);
    assign(value, true, LOC);
 
@@ -450,7 +452,7 @@ const int sym_stack_size = value_stack_size();
 
    loop(s, sym_stack_size)
       {
-        const ValueStackItem & vsi = (*this)[s];
+        const ValueStackItem & vsi = value_stack[s];
         if (vsi.name_class != NC_FUNCTION)   continue;
         if (fun != vsi.sym_val.function)     continue;
 
@@ -466,7 +468,7 @@ const int sym_stack_size = value_stack_size();
 Value_P
 Symbol::get_value()
 {
-const ValueStackItem & vs = value_stack.back();
+const ValueStackItem & vs = value_stack.last();
 
    if (vs.name_class == NC_VARIABLE)   return vs.apl_val;
 
@@ -479,9 +481,9 @@ Symbol::cant_be_defined() const
 // if (value_stack.size() > 1)         return "symbol was localized";
    if (Workspace::is_called(name))   return "function is called";
 
-   if (value_stack.back().name_class == NC_UNUSED_USER_NAME)   return 0;   // OK
-   if (value_stack.back().name_class == NC_FUNCTION)           return 0;   // OK
-   if (value_stack.back().name_class == NC_OPERATOR)           return 0;   // OK
+   if (value_stack.last().name_class == NC_UNUSED_USER_NAME)   return 0;   // OK
+   if (value_stack.last().name_class == NC_FUNCTION)           return 0;   // OK
+   if (value_stack.last().name_class == NC_OPERATOR)           return 0;   // OK
    return "bad name class";
 }
 //-----------------------------------------------------------------------------
@@ -489,24 +491,24 @@ Value_P
 Symbol::get_apl_value() const
 {
    Assert(value_stack.size() > 0);
-   if (value_stack.back().name_class != NC_VARIABLE)
+   if (value_stack.last().name_class != NC_VARIABLE)
       throw_symbol_error(get_name(), LOC);
 
-   return value_stack.back().apl_val;
+   return value_stack.last().apl_val;
 }
 //-----------------------------------------------------------------------------
 const Cell *
 Symbol::get_first_cell() const
 {
    Assert(value_stack.size() > 0);
-   if (value_stack.back().name_class != NC_VARIABLE)   return 0;
-   return &value_stack.back().apl_val->get_ravel(0);
+   if (value_stack.last().name_class != NC_VARIABLE)   return 0;
+   return &value_stack.last().apl_val->get_ravel(0);
 }
 //-----------------------------------------------------------------------------
 bool
 Symbol::can_be_assigned() const
 {
-   switch (value_stack.back().name_class)
+   switch (value_stack.last().name_class)
       {
         case NC_UNUSED_USER_NAME:
         case NC_VARIABLE:
@@ -520,26 +522,26 @@ Symbol::get_SV_key() const
 {
    Assert(value_stack.size() > 0);
 
-   if (value_stack.back().name_class != NC_SHARED_VAR)   return SV_key(0);
+   if (value_stack.last().name_class != NC_SHARED_VAR)   return SV_key(0);
 
-   return value_stack.back().sym_val.sv_key;
+   return value_stack.last().sym_val.sv_key;
 }
 //-----------------------------------------------------------------------------
 void
 Symbol::set_SV_key(SV_key key)
 {
-   value_stack.back().name_class = NC_SHARED_VAR;
-   value_stack.back().sym_val.sv_key = key;
+   value_stack.last().name_class = NC_SHARED_VAR;
+   value_stack.last().sym_val.sv_key = key;
 }
 //-----------------------------------------------------------------------------
 const Function *
 Symbol::get_function() const
 {
    Assert(value_stack.size() > 0);
-   switch(value_stack.back().name_class)
+   switch(value_stack.last().name_class)
       {
         case NC_FUNCTION:
-        case NC_OPERATOR: return value_stack.back().sym_val.function;
+        case NC_OPERATOR: return value_stack.last().sym_val.function;
         default:          return 0;
       }
 }
@@ -547,7 +549,7 @@ Symbol::get_function() const
 Function *
 Symbol::get_function()
 {
-const ValueStackItem & vs = value_stack.back();
+const ValueStackItem & vs = value_stack.last();
 
    if (vs.name_class == NC_FUNCTION)   return vs.sym_val.function;
    if (vs.name_class == NC_OPERATOR)   return vs.sym_val.function;
@@ -558,7 +560,7 @@ const ValueStackItem & vs = value_stack.back();
 void
 Symbol::get_attributes(int mode, Cell * dest) const
 {
-const ValueStackItem & vs = value_stack.back();
+const ValueStackItem & vs = value_stack.last();
 bool has_result = false;
 int fun_valence = 0;
 int oper_valence = 0;
@@ -655,7 +657,7 @@ Symbol::resolve(Token & tok, bool left_sym)
 
    Assert1(value_stack.size());
 
-const ValueStackItem & vs = value_stack.back();
+const ValueStackItem & vs = value_stack.last();
    switch(vs.name_class)
       {
         case NC_UNUSED_USER_NAME:
@@ -669,7 +671,7 @@ const ValueStackItem & vs = value_stack.back();
                IntCell lab(vs.sym_val.label);
                Value_P value(lab, LOC);
                Token t(TOK_APL_VALUE1, value);
-               move_1(tok, t, LOC);
+               tok.move_1(t, LOC);
              }
              return;
 
@@ -679,7 +681,7 @@ const ValueStackItem & vs = value_stack.back();
              // if we resolve a variable. the value is considered grouped.
              {
                Token t(TOK_APL_VALUE1, get_apl_value()->clone(LOC));
-               move_1(tok, t, LOC);
+               tok.move_1(t, LOC);
              }
              return;
 
@@ -691,7 +693,7 @@ const ValueStackItem & vs = value_stack.back();
                   //
                   return;
                 }
-             move_2(tok, vs.sym_val.function->get_token(), LOC);
+             tok.move_2(vs.sym_val.function->get_token(), LOC);
              return;
 
         case NC_SHARED_VAR:
@@ -714,16 +716,16 @@ Symbol::resolve_lv(const char * loc)
    Assert(value_stack.size());
 
    // if this is not a variable, then re-use the error handling of resolve().
-   if (value_stack.back().name_class != NC_VARIABLE)
+   if (value_stack.last().name_class != NC_VARIABLE)
       {
         CERR << "Symbol '" << get_name()
              << "' has changed type from variable to name class "
-             << value_stack.back().name_class << endl
+             << value_stack.last().name_class << endl
              << " while executing an assignment" << endl;
         throw_apl_error(E_LEFT_SYNTAX_ERROR, loc);
       }
 
-Value_P val = value_stack.back().apl_val;
+Value_P val = value_stack.last().apl_val;
    return Token(TOK_APL_VALUE1, val->get_cellrefs(loc));
 }
 //-----------------------------------------------------------------------------
@@ -732,7 +734,7 @@ Symbol::resolve_class(bool left)
 {
    Assert1(value_stack.size());
 
-   switch(value_stack.back().name_class)
+   switch(value_stack.last().name_class)
       {
         case NC_LABEL:
         case NC_VARIABLE:
@@ -741,7 +743,7 @@ Symbol::resolve_class(bool left)
 
         case NC_FUNCTION:
              {
-               const int valence = value_stack.back().sym_val.function
+               const int valence = value_stack.last().sym_val.function
                                  ->get_fun_valence();
                if (valence == 2)   return TC_FUN2;
                if (valence == 1)   return TC_FUN1;
@@ -750,7 +752,7 @@ Symbol::resolve_class(bool left)
 
         case NC_OPERATOR:
              {
-               const int valence = value_stack.back().sym_val.function
+               const int valence = value_stack.last().sym_val.function
                                  ->get_oper_valence();
                return (valence == 2) ? TC_OPER2 : TC_OPER1;
              }
@@ -764,7 +766,7 @@ Symbol::expunge()
 {
    if (value_stack.size() == 0)   return 1;   // empty stack
 
-ValueStackItem & vs = value_stack.back();
+ValueStackItem & vs = value_stack.last();
 
    if (vs.name_class == NC_VARIABLE)
       {
@@ -812,7 +814,7 @@ ValueStackItem & vs = value_stack.back();
 void
 Symbol::set_nc(NameClass nc)
 {
-ValueStackItem & vs = value_stack.back();
+ValueStackItem & vs = value_stack.last();
 
    if (vs.name_class == NC_UNUSED_USER_NAME)
       {
@@ -826,7 +828,7 @@ ValueStackItem & vs = value_stack.back();
 void
 Symbol::share_var(SV_key key)
 {
-ValueStackItem & vs = value_stack.back();
+ValueStackItem & vs = value_stack.last();
 
    if (vs.name_class == NC_UNUSED_USER_NAME)   // new shared variable
       {
@@ -860,7 +862,7 @@ Symbol::unshare_var()
 {
    if (value_stack.size() == 0)   return NO_COUPLING;
 
-ValueStackItem & vs = value_stack.back();
+ValueStackItem & vs = value_stack.last();
    if (vs.name_class != NC_SHARED_VAR)   return NO_COUPLING;
 
 const SV_key key = get_SV_key();
@@ -877,7 +879,7 @@ const SV_Coupling old_coupling = Svar_DB::get_coupling(key);
 void
 Symbol::set_nc(NameClass nc, Function * fun)
 {
-ValueStackItem & vs = value_stack.back();
+ValueStackItem & vs = value_stack.last();
 
 const bool can_set = (vs.name_class == NC_FUNCTION) ||
                      (vs.name_class == NC_OPERATOR) ||
@@ -899,9 +901,9 @@ Symbol::list(ostream & out)
 
    for (int s = name.size(); s < 32; ++s)   out << " ";
 
-   if (erased)   out << "   ERASED";
+   if (is_erased())   out << "   ERASED";
    Assert(value_stack.size());
-const NameClass nc = value_stack.back().name_class;
+const NameClass nc = value_stack.last().name_class;
    if      (nc == NC_INVALID)            out << "   INVALID NC";
    else if (nc == NC_UNUSED_USER_NAME)   out << "   Unused";
    else if (nc == NC_LABEL)              out << "   Label";
@@ -1052,17 +1054,18 @@ int count = 0;
 }
 //-----------------------------------------------------------------------------
 void
-Symbol::vector_assignment(Symbol * * symbols, int sym_count, Value_P values)
+Symbol::vector_assignment(Simple_string<Symbol *, false> & symbols,
+                          Value_P values)
 {
    if (values->get_rank() > 1)   RANK_ERROR;
    if (!values->is_scalar() &&
-       values->element_count() != sym_count)   LENGTH_ERROR;
+       values->element_count() != symbols.size())   LENGTH_ERROR;
 
 const int incr = values->is_scalar() ? 0 : 1;
 const Cell * cV = &values->get_ravel(0);
-   loop(s, sym_count)
+   loop(s, symbols.size())
       {
-        Symbol * sym = symbols[sym_count - s - 1];
+        Symbol * sym = symbols[symbols.size() - s - 1];
         if (cV->is_pointer_cell())
            {
              sym->assign(cV->get_pointer_value(), true, LOC);
@@ -1082,10 +1085,10 @@ const Cell * cV = &values->get_ravel(0);
 void
 Symbol::dump(ostream & out) const
 {
-const ValueStackItem & vs = (*this)[0];
+const ValueStackItem & vs = value_stack[0];
    if (vs.name_class == NC_VARIABLE)
       {
-        vector<UCS_string> CR10;
+        UCS_string_vector CR10;
         Quad_CR::do_CR10_var(CR10, get_name(), *vs.apl_val.get());
 
         loop(l, CR10.size())
@@ -1193,6 +1196,7 @@ ValueStackItem & tos = value_stack[0];
         case NC_FUNCTION:
         case NC_OPERATOR:
              tos.sym_val.function->destroy();
+             tos.name_class = NC_UNUSED_USER_NAME;
              break;
 
         default: break;
