@@ -567,7 +567,7 @@ const APL_Integer ps = value->get_sole_integer();
 Quad_PW::Quad_PW()
    : SystemVariable(ID::Quad_PW)
 {
-   Symbol::assign(IntScalar(DEFAULT_Quad_PW, LOC), false, LOC);
+   Symbol::assign(IntScalar(uprefs.initial_pw, LOC), false, LOC);
 }
 //-----------------------------------------------------------------------------
 void
@@ -615,7 +615,7 @@ UCS_string line;
 
    line.remove_leading_and_trailing_whitespaces();
 
-   move_2(token, Bif_F1_EXECUTE::execute_statement(line), LOC);
+   token.move_2(Bif_F1_EXECUTE::execute_statement(line), LOC);
 }
 //=============================================================================
 Quad_QUOTE::Quad_QUOTE()
@@ -638,7 +638,7 @@ Quad_QUOTE::done(bool with_LF, const char * loc)
    if (prompt.size())
       {
         if (with_LF)   COUT << endl;
-        prompt.clear();
+        prompt.shrink(0);
       }
 }
 //-----------------------------------------------------------------------------
@@ -686,7 +686,7 @@ Quad_QUOTE::get_apl_value() const
 
    // get_quad_cr_line() may call done(), so we save the current prompt.
    //
-const UCS_string old_prompt(prompt);
+const UCS_string old_prompt = prompt.no_pad();
 
 bool eof = false;
 UCS_string line;
