@@ -2,7 +2,7 @@
     This file is part of GNU APL, a free implementation of the
     ISO/IEC Standard 13751, "Programming Language APL, Extended"
 
-    Copyright (C) 2008-2015  Dr. Jürgen Sauermann
+    Copyright (C) 2008-2016  Dr. Jürgen Sauermann
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,13 +31,11 @@
 Token::Token(const Token & other)
    : tag(TOK_VOID)
 {
-   copy_1(*this, other, "Token::Token(other)");
+   copy_1(other, "Token::Token(other)");
 }
 //-----------------------------------------------------------------------------
 Token::Token(TokenTag tg, IndexExpr & idx)
 {
-   Assert(&idx);
-
    if (tg == TOK_PINDEX)
       {
         // this token is a partial index in the prefix parser.
@@ -56,11 +54,11 @@ Token::Token(TokenTag tg, IndexExpr & idx)
              tag = TOK_AXES;
              if (idx.value_count() == 0)   // []
                 {
-                  new (&value._apl_val()) Value_P;
+                  new (&value.apl_val) Value_P;
                 }
              else                          // [x]
                 {
-                  new (&value._apl_val())   Value_P(idx.extract_value(0));
+                  new (&value.apl_val)   Value_P(idx.extract_value(0));
                 }
            }
         else                         // [idx; ...]
@@ -139,7 +137,7 @@ operator << (ostream & out, const Token & token)
         else if (token.get_tag() == TOK_APL_VALUE3)   out << "VALUE3";
         else                                          out << "VALUE???";
         Value_P value = token.get_apl_val();
-        Assert(value);
+        Assert(!!value);
         const Depth depth = value->compute_depth();
         out << "«";
         for (Depth d = 0; d < depth; ++d)   out << "≡";
@@ -247,15 +245,25 @@ Token::ChangeTag(TokenTag new_tag)
 int
 Token::value_use_count() const
 {
-   if (!is_apl_val())       return 0;
-   if (!value._apl_val())   return -98;
-   return value._apl_val()->get_owner_count();
+   if (!is_apl_val())    return 0;
+   if (!value.apl_val)   return -98;
+   return value.apl_val->get_owner_count();
 }
 //-----------------------------------------------------------------------------
 void
-Token::extract_apl_val(const char * loc) const
+Token::extract_apl_val(const char * loc)
 {
-   if (is_apl_val())   ptr_clear(value._apl_val(), loc);
+   if (is_apl_val())   value.apl_val.reset();
+}
+//-----------------------------------------------------------------------------
+Value *
+Token::extract_and_keep(const char * loc)
+{
+   if (!is_apl_val())   return 0;
+
+Value * ret = value.apl_val.get();
+   value.apl_val.clear_pointer(loc);
+   return ret;
 }
 //-----------------------------------------------------------------------------
 ostream &
@@ -390,7 +398,7 @@ Token::print_value(ostream & out) const
         case TOK_APL_VALUE1:
         case TOK_APL_VALUE3:
              {
-               Value_P v = value._apl_val();
+               const Value * v = value.apl_val.get();
                if (v->get_rank() == 0)   out << "''";
                loop(r, v->get_rank())
                    {
@@ -583,14 +591,14 @@ UCS_string ucs;
         case TC_L_PARENT:
         case TC_R_CURLY:
         case TC_L_CURLY:
-             return UCS_string(UTF8_string(ID::name(get_Id())));
+             return ID::get_name(get_Id());
 
         case TC_FUN0:
         case TC_FUN12:
         case TC_OPER1:
         case TC_OPER2:
              if (get_Id() == ID::No_ID)   return get_function()->get_name();
-             return UCS_string(UTF8_string(ID::name(get_Id())));
+             return ID::get_name(get_Id());
 
         default:
              CERR << "Token: " << HEX4(tag) << " " << *this
@@ -680,7 +688,7 @@ Token::short_class_name(TokenClass cls)
 }
 //-----------------------------------------------------------------------------
 void
-Token::print_token_list(ostream & out, const Source<Token> & src)
+Token::print_token_list(ostream & out, const Source<Token, true> & src)
 {
 const int len = src.rest();
    loop(t, len)   out << "`" << src[t] << "  ";
@@ -766,12 +774,12 @@ Token::copy_N(const Token & src)
 }
 //-----------------------------------------------------------------------------
 void
-copy_1(Token & dst, const Token & src, const char * loc)
+Token::copy_1(const Token & src, const char * loc)
 {
-   dst.clear(loc);
+   clear(loc);
    if (src.is_apl_val())
       {
-        Value * val = src.get_apl_val_pointer();
+        const Value * val = src.value.apl_val.get();
         if (val)
            {
              ADD_EVENT(val, VHE_TokCopy1, src.value_use_count(), loc);
@@ -782,42 +790,48 @@ copy_1(Token & dst, const Token & src, const char * loc)
            }
       }
 
-   dst.copy_N(src);
+   copy_N(src);
 }
 //-----------------------------------------------------------------------------
 void
-move_1(Token & dst, Token & src, const char * loc)
+Token::move_1(Token & src, const char * loc)
 {
-   dst.clear(loc);
-   dst.copy_N(src);
+   clear(loc);
+   copy_N(src);
 
    if (src.is_apl_val())
       {
-        Value * val = src.get_apl_val_pointer();
-	ADD_EVENT(val, VHE_TokMove1, src.value_use_count() - 1, loc);
-        src.clear(loc);
+        const Value * val = src.value.apl_val.get();
+        if (val)
+           {
+	     ADD_EVENT(val, VHE_TokMove1, src.value_use_count() - 1, loc);
+             src.clear(loc);
+           }
       }
 }
 //-----------------------------------------------------------------------------
 void
-move_2(Token & dst, const Token & src, const char * loc)
+Token::move_2(const Token & src, const char * loc)
 {
-   dst.clear(loc);
-   dst.copy_N(src);
+   clear(loc);
+   copy_N(src);
 
    if (src.is_apl_val())
       {
-        Value * val = src.get_apl_val_pointer();
-        ADD_EVENT(val, VHE_TokMove2, src.value_use_count() - 1, loc);
-        ((Token &)src).clear(loc);
+        const Value * val = src.value.apl_val.get();
+        if (val)
+           {
+             ADD_EVENT(val, VHE_TokMove2, src.value_use_count() - 1, loc);
+             ((Token &)src).clear(loc);
+           }
       }
 }
 //-----------------------------------------------------------------------------
 void
 Token_string::reverse_from_to(ShapeItem from, ShapeItem to)
 {
-Token * t1 = items + from;
-Token * t2 = items + to;
+Token * t1 = &at(from);
+Token * t2 = &at(to);
    Assert(0 <= from);
    Assert(from <= to);
    Assert(to <= size());
@@ -836,7 +850,7 @@ Token_string::print(ostream & out) const
 {
    loop(t, size())
        {
-         const Token & tok = (*this)[t];
+         const Token & tok = at(t);
          out << "`" << tok << "  ";
        }
 
