@@ -78,7 +78,7 @@ Tokenizer::do_tokenize(const UCS_string & input, Token_string & tos)
 Source<Unicode> src(input);
    while ((rest_1 = rest_2 = src.rest()) != 0)
       {
-        const Unicode uni = *src;
+        Unicode uni = *src;
         if (uni == UNI_COMMENT)             break;   // ⍝ comment
         if (uni == UNI_ASCII_NUMBER_SIGN)   break;   // # comment
 
@@ -206,7 +206,7 @@ Source<Unicode> src(input);
                         if (src.rest() == 1)   // syntax error
                            throw_parse_error(E_SYNTAX_ERROR, LOC, loc);
 
-                        const Unicode uni_1 = src[1];
+                        Unicode uni_1 = src[1];
                         const Token tok_1 = Avec::uni_to_token(uni_1, LOC);
                         if ((tok_1.get_tag() & TC_MASK) == TC_NUMERIC)
                            tokenize_number(src, tos);
@@ -510,6 +510,8 @@ Tokenizer::tokenize_string2(Source<Unicode> & src, Token_string & tos)
    // skip the leading "
    {
      const Unicode uni = src.get();
+     if (uni)   { /* do nothing, needed for -Wall */ }
+
      Assert1(uni == UNI_ASCII_DOUBLE_QUOTE);
    }
 
@@ -599,7 +601,7 @@ const bool real_valid = tokenize_real(src, real_need_float, real_flt, real_int);
         bool imag_need_float = false;
         const bool imag_valid = tokenize_real(src, imag_need_float,
                                               imag_flt, imag_int);
-        if (!imag_need_float)   imag_flt = imag_flt;
+        if (!imag_need_float)   imag_flt = imag_int;
 
         if (!imag_valid)
            {
@@ -632,7 +634,7 @@ const bool real_valid = tokenize_real(src, real_need_float, real_flt, real_int);
         bool imag_need_float = false;
         const bool imag_valid = tokenize_real(src, imag_need_float,
                                               degrees_flt, degrees_int);
-        if (!imag_need_float)   degrees_flt = degrees_flt;
+        if (!imag_need_float)   degrees_flt = degrees_int;
 
         if (!imag_valid)
            {
@@ -816,7 +818,7 @@ int expo = 0;
    //
    while (digits.size() && digits[0] == '0')
       {
-        digits.drop_leading(1);   // discard '0'
+        digits.erase(0);   // discard '0'
         --expo;
       }
 
