@@ -2,7 +2,7 @@
     This file is part of GNU APL, a free implementation of the
     ISO/IEC Standard 13751, "Programming Language APL, Extended"
 
-    Copyright (C) 2008-2015  Dr. Jürgen Sauermann
+    Copyright (C) 2008-2016  Dr. Jürgen Sauermann
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,10 +31,49 @@
 #include "UTF8_string.hh"
 #include "Value.icc"
 
+ShapeItem UCS_string::total_count = 0;
+ShapeItem UCS_string::total_id = 0;
+
+//-----------------------------------------------------------------------------
+UCS_string::UCS_string()
+{
+   create(LOC);
+}
+//-----------------------------------------------------------------------------
+UCS_string::UCS_string(Unicode uni)
+   : Simple_string<Unicode, false>(1, uni)
+{
+  create(LOC);
+}
+//-----------------------------------------------------------------------------
+UCS_string::UCS_string(const Unicode * data, size_t len)
+   : Simple_string<Unicode, false>(data, len)
+{
+   create(LOC);
+}
+//-----------------------------------------------------------------------------
+UCS_string::UCS_string(size_t len, Unicode uni)
+   : Simple_string<Unicode, false>(len, uni)
+{
+   create(LOC);
+}
+//-----------------------------------------------------------------------------
+UCS_string::UCS_string(const UCS_string & ucs)
+   : Simple_string<Unicode, false>(ucs)
+{
+   create(LOC);
+}
+//-----------------------------------------------------------------------------
+UCS_string::UCS_string(const UCS_string & ucs, size_t pos, size_t len)
+   : Simple_string<Unicode, false>(ucs, pos, len)
+{
+   create(LOC);
+}
 //-----------------------------------------------------------------------------
 UCS_string::UCS_string(const char * cstring)
-   : Simple_string<Unicode>(0, 0)
 {
+   create(LOC);
+
 const int len = strlen(cstring);
    extend(len + 1);
 
@@ -43,8 +82,9 @@ const int len = strlen(cstring);
 }
 //-----------------------------------------------------------------------------
 UCS_string::UCS_string(const UTF8_string & utf)
-   : Simple_string<Unicode>(0, 0)
 {
+   create(LOC);
+
    Log(LOG_char_conversion)
       CERR << "UCS_string::UCS_string(): utf = " << utf << endl;
 
@@ -76,7 +116,7 @@ UCS_string::UCS_string(const UTF8_string & utf)
                       << " len " << utf.size() << " at " << LOC <<  endl;
                    if (utf.size() >= 40)
                       {
-                         UTF8_string end(utf.get_items() + utf.size() - 10, 10);
+                         const UTF8_string end(&utf[utf.size() - 10], 10);
                          end.dump_hex(CERR << endl << "(ending with : ", 20)
                                            << ")" << endl;
                       }
@@ -90,7 +130,7 @@ UCS_string::UCS_string(const UTF8_string & utf)
                       << " len " << utf.size() << " at " << LOC <<  endl;
                    if (utf.size() >= 40)
                       {
-                         UTF8_string end(utf.get_items() + utf.size() - 10, 10);
+                         const UTF8_string end(&utf[utf.size() - 10], 10);
                          end.dump_hex(CERR << endl << "(ending with : ", 20)
                                            << ")" << endl;
                       }
@@ -112,8 +152,9 @@ UCS_string::UCS_string(const UTF8_string & utf)
 //-----------------------------------------------------------------------------
 UCS_string::UCS_string(APL_Float value, bool & scaled,
                        const PrintContext & pctx)
-   : Simple_string<Unicode>(0, 0)
 {
+   create(LOC);
+
 int quad_pp = pctx.get_PP();
    if (quad_pp > MAX_Quad_PP)   quad_pp = MAX_Quad_PP;
    if (quad_pp < MIN_Quad_PP)   quad_pp = MIN_Quad_PP;
@@ -283,13 +324,14 @@ const Unicode last = digits.last();
 }
 //-----------------------------------------------------------------------------
 UCS_string::UCS_string(const PrintBuffer & pb, Rank rank, int quad_PW)
-   : Simple_string<Unicode>(0, 0)
 {
+   create(LOC);
+
    if (pb.get_height() == 0)   return;      // empty PrintBuffer
 
 const int total_width = pb.get_width(0);
 
-vector<int> breakpoints;
+Simple_string<int, false> breakpoints;
    breakpoints.reserve(2*total_width/quad_PW);
 
    // print rows, breaking at breakpoints
@@ -306,7 +348,7 @@ vector<int> breakpoints;
               if (row == 0)   // first row: set up breakpoints
                  {
                    chunk_len = pb.get_line(0).compute_chunk_length(quad_PW,col);
-                   breakpoints.push_back(chunk_len);
+                   breakpoints.append(chunk_len);
                  }
               else
                  {
@@ -326,7 +368,7 @@ vector<int> breakpoints;
    //
    loop(u, size())
        {
-         if (is_iPAD_char((*this)[u]))   (*this)[u] = UNI_ASCII_SPACE;
+         if (is_iPAD_char(at(u)))   at(u) = UNI_ASCII_SPACE;
        }
 }
 //-----------------------------------------------------------------------------
@@ -342,7 +384,7 @@ int pos = col + chunk_len;
 
    while (--pos > col)
       {
-         const Unicode uni = (*this)[pos];
+         const Unicode uni = at(pos);
          if (uni == UNI_iPAD_U2 || uni == UNI_iPAD_U3)
             {
                chunk_len = pos - col + 1;
@@ -366,14 +408,13 @@ UCS_string::remove_trailing_padchars()
    //
    loop(u, size())
        {
-         if ((*this)[u] == UNI_LINE_VERT)    break;
-         if ((*this)[u] == UNI_LINE_VERT2)   break;
-         if ((*this)[u] == UNI_iPAD_L0)
+         if (at(u) == UNI_LINE_VERT)    break;
+         if (at(u) == UNI_LINE_VERT2)   break;
+         if (at(u) == UNI_iPAD_L0)
             {
-              clear();
+              shrink(0);
               return;
             }
-
        }
 
    while (size())
@@ -394,7 +435,7 @@ UCS_string::remove_trailing_padchars()
 void
 UCS_string::remove_trailing_whitespaces()
 {
-   while (size() && Avec::is_white(last()))   pop();
+   while (size() && last() <= UNI_ASCII_SPACE)   pop();
 }
 //-----------------------------------------------------------------------------
 void
@@ -403,11 +444,13 @@ UCS_string::remove_leading_whitespaces()
 int count = 0;
    loop(s, size())
       {
-        if (Avec::is_white((*this)[s]))   ++count;
-        else                                 break;
+        if (at(s) <= UNI_ASCII_SPACE)   ++count;
+        else                            break;
       }
 
-   drop_leading(count);
+   if (count == 0)        return;      // no leading whitspaces
+   if (count == size())   shrink(0);   // only whitespaces
+   else                   *this = UCS_string(*this, count, size() - count);
 }
 //-----------------------------------------------------------------------------
 void
@@ -417,11 +460,11 @@ UCS_string::split_ws(UCS_string & rest)
 
    loop(clen, size())
        {
-         if (Avec::is_white((*this)[clen]))   // whilespace: end of command
+         if (Avec::is_white(at(clen)))   // whilespace: end of command
             {
               ShapeItem arg = clen;
-              while (arg < size() && Avec::is_white((*this)[arg]))   ++arg;
-              while (arg < size())   rest.append((*this)[arg++]);
+              while (arg < size() && Avec::is_white(at(arg)))   ++arg;
+              while (arg < size())   rest.append(at(arg++));
               shrink(clen);
               return;
             }
@@ -430,8 +473,10 @@ UCS_string::split_ws(UCS_string & rest)
 //-----------------------------------------------------------------------------
 /// constructor
 UCS_string::UCS_string(const Value & value)
-   : Simple_string<Unicode>(0, 0)
+   : Simple_string<Unicode, false>(0, 0)
 {
+   create(LOC);
+
    if (value.get_rank() > 1) RANK_ERROR;
 
 const ShapeItem ec = value.element_count();
@@ -441,8 +486,10 @@ const ShapeItem ec = value.element_count();
 }
 //-----------------------------------------------------------------------------
 UCS_string::UCS_string(istream & in)
-   : Simple_string<Unicode>(0, 0)
+   : Simple_string<Unicode, false>(0, 0)
 {
+   create(LOC);
+
    for (;;)
       {
         const Unicode uni = UTF8_string::getc(in);
@@ -460,38 +507,14 @@ UCS_string::copy_black(UCS_string & dest, int & idx) const
    while (idx < size() && operator[](idx) <= ' ')   ++idx;
 }
 //-----------------------------------------------------------------------------
-void
-UCS_string::copy_black_list(vector<UCS_string> & dest) const
-{
-   for (int idx = 0; ; )
-      {
-        UCS_string next;
-        copy_black(next, idx);
-        if (next.size() == 0)   break;
-
-        dest.push_back(next);
-      }
-}
-//-----------------------------------------------------------------------------
 ShapeItem
 UCS_string::LF_count() const
 {
 ShapeItem count = 0;
-   loop(u, size())   if ((*this)[u] == UNI_ASCII_LF)   ++count;
+   loop(u, size())   if (at(u) == UNI_ASCII_LF)   ++count;
    return count;
 }
 //-----------------------------------------------------------------------------
-bool
-UCS_string::contained_in(const vector<UCS_string> & list) const
-{
-   loop(l, list.size())
-      {
-        if (*this == list[l])   return true;
-      }
-
-   return false;
-}
-//-----------------------------------------------------------------------------
 ShapeItem
 UCS_string::substr_pos(const UCS_string & sub) const
 {
@@ -501,7 +524,7 @@ const ShapeItem start_positions = 1 + size() - sub.size();
         bool mismatch = false;
         loop(u, sub.size())
            {
-             if ((*this)[start + u] != sub[u])
+             if (at(start + u) != sub[u])
                 {
                   mismatch = true;
                   break;
@@ -517,7 +540,7 @@ const ShapeItem start_positions = 1 + size() - sub.size();
 bool 
 UCS_string::has_black() const
 {
-   loop(s, size())   if (!Avec::is_white((*this)[s]))   return true;
+   loop(s, size())   if (!Avec::is_white(at(s)))   return true;
    return false;
 }
 //-----------------------------------------------------------------------------
@@ -529,7 +552,7 @@ UCS_string::starts_with(const char * prefix) const
         const char pc = *prefix++;
         if (pc == 0)   return true;   // prefix matches this string.
 
-        const Unicode uni = (*this)[s];
+        const Unicode uni = at(s);
         if (uni != Unicode(pc))   return false;
       }
 
@@ -543,7 +566,7 @@ UCS_string::starts_with(const UCS_string & prefix) const
 {
    if (prefix.size() > size())   return false;
 
-   loop(p, prefix.size())   if ((*this)[p] != prefix[p])   return false;
+   loop(p, prefix.size())   if (at(p) != prefix[p])   return false;
 
    return true;
 }
@@ -557,7 +580,7 @@ UCS_string::starts_iwith(const char * prefix) const
         if (pc == 0)   return true;   // prefix matches this string.
         if (pc >= 'a' && pc <= 'z')   pc -= 'a' - 'A';
 
-        int uni = (*this)[s];
+        int uni = at(s);
         if (uni >= 'a' && uni <= 'z')   uni -= 'a' - 'A';
 
         if (uni != Unicode(pc))   return false;
@@ -573,7 +596,7 @@ UCS_string::starts_iwith(const UCS_string & prefix) const
 
    loop(p, prefix.size())
       {
-        int c1 = (*this)[p];
+        int c1 = at(p);
         int c2 = prefix[p];
         if (c1 >= 'a' && c1 <= 'z')   c1 -= 'a' - 'A';
         if (c2 >= 'a' && c2 <= 'z')   c2 -= 'a' - 'A';
@@ -589,7 +612,7 @@ UCS_string::no_pad() const
 UCS_string ret;
    loop(s, size())
       {
-        Unicode uni = (*this)[s];
+        Unicode uni = at(s);
         if (is_iPAD_char(uni))   uni = UNI_ASCII_SPACE;
         ret.append(uni);
       }
@@ -602,7 +625,7 @@ UCS_string::map_pad()
 {
    loop(s, size())
       {
-        if (is_iPAD_char((*this)[s]))   (*this)[s] = UNI_ASCII_SPACE;
+        if (is_iPAD_char(at(s)))   at(s) = UNI_ASCII_SPACE;
       }
 }
 //-----------------------------------------------------------------------------
@@ -612,52 +635,31 @@ UCS_string::remove_pad() const
 UCS_string ret;
    loop(s, size())
       {
-        Unicode uni = (*this)[s];
+        Unicode uni = at(s);
         if (!is_iPAD_char(uni))   ret.append(uni);
       }
 
    return ret;
 }
 //-----------------------------------------------------------------------------
-void
-UCS_string::remove_lt_spaces()
-{
-   // remove trailing spaces
-   //
-   while (size() && (*this)[size() - 1] <= ' ')   shrink(size() - 1);
-
-   // count leading spaces
-   //
-int count = 0;
-   loop(s, size())
-      {
-        if ((*this)[s] <= ' ')   ++count;
-        else                     break;
-      }
-
-   // remove leading spaces
-   //
-   erase(0, count);
-}
-//-----------------------------------------------------------------------------
 UCS_string
 UCS_string::reverse() const
 {
 UCS_string ret;
-   for (int s = size(); s > 0;)   ret.append((*this)[--s]);
+   for (int s = size(); s > 0;)   ret.append(at(--s));
    return ret;
 }
 //-----------------------------------------------------------------------------
 bool
 UCS_string::is_comment_or_label() const
 {
-   if (size() == 0)                               return false;
-   if ((*this)[0] == UNI_ASCII_NUMBER_SIGN)       return true;   // comment
-   if ((*this)[0] == UNI_COMMENT)                 return true;   // comment
+   if (size() == 0)                          return false;
+   if (at(0) == UNI_ASCII_NUMBER_SIGN)       return true;   // comment
+   if (at(0) == UNI_COMMENT)                 return true;   // comment
    loop(t, size())
        {
-         if ((*this)[t] == UNI_ASCII_COLON)       return true;   // label
-         if (!Avec::is_symbol_char((*this)[t]))   return false;
+         if (at(t) == UNI_ASCII_COLON)       return true;   // label
+         if (!Avec::is_symbol_char(at(t)))   return false;
        }
 
    return false;
@@ -746,24 +748,24 @@ char cc[60];
 }
 //-----------------------------------------------------------------------------
 size_t
-UCS_string::to_vector(vector<UCS_string> & result) const
+UCS_string::to_vector(UCS_string_vector & result) const
 {
 size_t max_len = 0;
 
-   result.clear();
+   result.shrink(0);
    if (size() == 0)   return max_len;
 
-   result.push_back(UCS_string());
+   result.append(UCS_string());
    loop(s, size())
       {
-        const Unicode uni = (*this)[s];
+        const Unicode uni = at(s);
         if (uni == UNI_ASCII_LF)    // line done
            {
-             const size_t len = result.back().size();
+             const size_t len = result.last().size();
              if (max_len < len)   max_len = len;
 
              if (s < size() - 1)   // more coming
-                result.push_back(UCS_string());
+                result.append(UCS_string());
            }
         else
            {
@@ -773,7 +775,7 @@ size_t max_len = 0;
       }
 
    // if the last line lacked a \n we check max_len here again.
-const size_t len = result.back().size();
+const size_t len = result.last().size();
    if (max_len < len)   max_len = len;
 
    return max_len;
@@ -783,13 +785,20 @@ int
 UCS_string::atoi() const
 {
 int ret = 0;
+bool negative = false;
 
    loop(s, size())
       {
-        const Unicode uni = (*this)[s];
+        const Unicode uni = at(s);
 
         if (!ret && Avec::is_white(uni))   continue;   // leading whitespace
 
+        if (uni == UNI_ASCII_MINUS || uni == UNI_OVERBAR)
+           {
+             negative = true;
+             continue;
+           }
+
         if (uni < UNI_ASCII_0)                break;      // non-digit
         if (uni > UNI_ASCII_9)                break;      // non-digit
 
@@ -797,7 +806,7 @@ int ret = 0;
         ret += uni - UNI_ASCII_0;
       }
 
-   return ret;
+   return negative ? -ret : ret;
 }
 //-----------------------------------------------------------------------------
 ostream &
@@ -856,8 +865,8 @@ UCS_string::lexical_before(const UCS_string other) const
    loop(u, size())
       {
         if (u >= other.size())   return false;   // other is a prefix of this
-        if ((*this)[u] < other[u])   return true;
-        if ((*this)[u] > other[u])   return false;
+        if (at(u) < other.at(u))   return true;
+        if (at(u) > other.at(u))   return false;
       }
 
    // at this point the common part of this and other is equal, If other
@@ -871,7 +880,7 @@ UCS_string::dump(ostream & out) const
    out << right << hex << uppercase << setfill('0');
    loop(s, size())
       {
-        out << " U+" << setw(4) << (int)((*this)[s]);
+        out << " U+" << setw(4) << (int)at(s);
       }
 
    return out << left << dec << nouppercase << setfill(' ');
@@ -1082,21 +1091,21 @@ UCS_string::round_last_digit()
       {
         for (int q = size() - 2; q >= 0; --q)
             {
-              const Unicode cc = items[q];
+              const Unicode cc = at(q);
               if (cc < UNI_ASCII_0)   continue;   // not a digit
               if (cc > UNI_ASCII_9)   continue;   // not a digit
 
-              items[q] = Unicode(cc + 1);   // round up
+              at(q) = Unicode(cc + 1);   // round up
               if (cc != UNI_ASCII_9)   break;    // 0-8 rounded up: stop
 
-              items[q] = UNI_ASCII_0;    // 9 rounded up: say 0 and repeat
+              at(q) = UNI_ASCII_0;    // 9 rounded up: say 0 and repeat
               if (q)   continue;   // not first difit
 
               // something like 9.xxx has been rounded up to, say, 0.xxx
               // but should be 10.xxx Fix it.
               //
-              for (int d = size() - 1; d > 0; --d)   items[d] = items[d - 1];
-              items[0] = UNI_ASCII_1;
+              for (int d = size() - 1; d > 0; --d)  at(d) = at(d - 1);
+              at(0) = UNI_ASCII_1;
             }
       }
 
@@ -1107,123 +1116,34 @@ UCS_string::round_last_digit()
 bool
 UCS_string::contains(Unicode uni)
 {
-   loop(u, size())   if ((*this)[u] == uni)   return true;
+   loop(u, size())   if (at(u) == uni)   return true;
    return false;
 }
 //----------------------------------------------------------------------------
-void
-UCS_string::sort_names(const UCS_string ** names, int count)
+UCS_string
+UCS_string::sort() const
 {
-   if (count < 2)   return;
-   Heapsort<const UCS_string *>::sort(names, count, 0, compare_names);
+UCS_string ret(*this);
+   Heapsort<Unicode>::sort(&ret[0], ret.size(), 0, greater_uni);
+   return ret;
 }
 //----------------------------------------------------------------------------
-void
-UCS_string::sort_names(vector<UCS_string> names)
+UCS_string
+UCS_string::unique() const
 {
-   if (names.size() < 2)   return;
+   if (size() <= 1)   return UCS_string(*this);
 
-   for (size_t h = 0; h < (names.size() - 1); ++h)
-       {
-         // find smallest above (including) h
-         //
-         int smallest = h;
-         for (size_t j = h + 1; j < names.size(); ++j)
-             {
-               if (names[smallest].compare(names[j]) == COMP_GT)
-                  {
-                    names[smallest].swap(names[j]);
-                  }
-             }
-       }
-}
-//----------------------------------------------------------------------------
-void
-UCS_string::compute_column_width(vector<int> & result,
-                                 const UCS_string ** names, int name_count,
-                                 int tab_size, int quad_PW)
-{
-   if (name_count < 2)
-      {
-        result.clear();
-        if (name_count)   result.push_back(names[0]->size());
-        else              result.push_back(quad_PW);
-        return;
-      }
+UCS_string sorted = sort();
+UCS_string ret;
+   ret.reserve(sorted.size());
 
-   // compute block counts (one block having tab_size characters)
-   //
-const int max_blocks = (quad_PW + 1) / tab_size;
-DynArray(int, name_blocks, name_count);
-   loop(n, name_count)
+   ret.append(sorted[0]);
+   for (ShapeItem j = 1; j < size(); ++j)
        {
-         name_blocks[n] = 1 + (1 + names[n]->size()) / tab_size;
+         if (sorted[j] != ret.back())   ret.append(sorted[j]);
        }
 
-   // compute max number of column blocks based on first line blocks
-   //
-int max_col = -1;
-   {
-     int blocks = 0;
-     loop(n, name_count)
-         {
-           if ((blocks + name_blocks[n]) < max_blocks)   // name_blocks[n] fits
-              blocks += name_blocks[n];
-           else                                          // max_blocks exceeded
-             {
-               max_col = n - 1;
-               break;
-             }
-         }
-
-     if (max_col == -1)   // all blocks fit
-        {
-          result.clear();
-          loop(n, name_count)
-              {
-                result.push_back(name_blocks[n]);
-              }
-          return;
-        }
-   }
-
-   // decrease max_col until all names fit...
-   //
-   for (;max_col > 1; --max_col)
-      {
-        result.clear();
-        int free_blocks = max_blocks;
-        loop(n, name_count)   // try to fit blocks into result
-            {
-              const int col_n = n % max_col;
-              const int bn = name_blocks[n];
-              if (n < max_col)   // first row: append column
-                 {
-                   free_blocks -= bn;
-                   if (free_blocks < 0)   break;
-                   result.push_back(bn);
-                 }
-              else if (bn > result[col_n])   
-                 {
-                   free_blocks -= bn - result[col_n];
-                   if (free_blocks < 0)   break;
-                   result[col_n] = bn;
-                 }
-            }
-
-        if (free_blocks >= 0)   return;   // success
-      }
-
-   // single colums
-   //
-   result.clear();
-int max_nb = 0;
-   loop(n, name_count)
-      {
-        if (max_nb < name_blocks[n])   max_nb = name_blocks[n];
-      }
-   result.push_back(max_nb);
+   Heapsort<Unicode>::sort(&ret[0], ret.size(), 0, greater_uni);
+   return ret;
 }
 //----------------------------------------------------------------------------
-
-//----------------------------------------------------------------------------
