@@ -126,6 +126,7 @@ UserFunction::UserFunction(Fun_signature sig, int lambda_num,
    parse_body_line(Function_Line_0, lambda_body, false, false, LOC);
    setup_lambdas();
    line_starts.append(Function_PC(lambda_body.size() - 1));
+   line_starts.append(Function_PC_0);
    error_line = -1;   // no error
    error_info = 0;
 }
@@ -262,12 +263,12 @@ UserFunction::eval_LB(Token & LO, Value_P B)
 
    Workspace::push_SI(this, LOC);
 
-   if (header.Z())         header.Z()->push();
-   if (header.A())         header.A()->push();
+   if (header.Z())         header.Z() ->push();
+   if (header.A())         header.A() ->push();
    if (LO.is_function())   header.LO()->push_function(LO.get_function());
    else                    header.LO()->push_value(LO.get_apl_val());
-   if (header.X())         header.X()->push();
-                           header.B()->push_value(B);
+   if (header.X())         header.X() ->push();
+   header                        .B() ->push_value(B);
 
    header.eval_common();
 
@@ -294,7 +295,7 @@ UserFunction::eval_LXB(Token & LO, Value_P X, Value_P B)
    if (LO.is_function())   header.LO()->push_function(LO.get_function());
    else                    header.LO()->push_value(LO.get_apl_val());
    if (header.X())         header.X()->push_value(X);
-                           header.B()->push_value(B);
+   header                        .B()->push_value(B);
 
    header.eval_common();
 
@@ -320,10 +321,10 @@ UserFunction::eval_ALB(Value_P A, Token & LO, Value_P B)
    if (header.X())         header.X()->push();
 
    if (header.Z())         header.Z()->push();
-                           header.A()->push_value(A);
+   header                        .A()->push_value(A);
    if (LO.is_function())   header.LO()->push_function(LO.get_function());
    else                    header.LO()->push_value(LO.get_apl_val());
-                           header.B()->push_value(B);
+   header                        .B()->push_value(B);
 
    header.eval_common();
 
@@ -348,11 +349,11 @@ UserFunction::eval_ALXB(Value_P A, Token & LO, Value_P X, Value_P B)
    Workspace::push_SI(this, LOC);
 
    if (header.Z())         header.Z()->push();
-                           header.A()->push_value(A);
+   header                        .A()->push_value(A);
    if (LO.is_function())   header.LO()->push_function(LO.get_function());
    else                    header.LO()->push_value(LO.get_apl_val());
    if (header.X())         header.X()->push_value(X);
-                           header.B()->push_value(B);
+   header                        .B()->push_value(B);
 
    header.eval_common();
 
@@ -381,8 +382,8 @@ UserFunction::eval_LRB(Token & LO, Token & RO, Value_P B)
    else                    header.LO()->push_value(LO.get_apl_val());
    if (RO.is_function())   header.RO()->push_function(RO.get_function());
    else                    header.RO()->push_value(RO.get_apl_val());
-   if (header.X())         header.X()->push();
-                           header.B()->push_value(B);
+   if (header.X())         header.X() ->push();
+   header                        .B() ->push_value(B);
 
    header.eval_common();
 
@@ -412,7 +413,7 @@ UserFunction::eval_LRXB(Token & LO, Token & RO, Value_P X, Value_P B)
    if (RO.is_function())   header.RO()->push_function(RO.get_function());
    if (header.X())         header.X()->push_value(X);
    else                    header.RO()->push_value(RO.get_apl_val());
-                           header.B()->push_value(B);
+   header                        .B()->push_value(B);
 
    header.eval_common();
 
@@ -438,13 +439,13 @@ UserFunction::eval_ALRB(Value_P A, Token & LO, Token & RO, Value_P B)
 
    if (header.Z())         header.Z()->push();
 
-                           header.A()->push_value(A);
+   header                        .A()->push_value(A);
    if (LO.is_function())   header.LO()->push_function(LO.get_function());
    else                    header.LO()->push_value(LO.get_apl_val());
    if (RO.is_function())   header.RO()->push_function(RO.get_function());
    else                    header.RO()->push_value(RO.get_apl_val());
    if (header.X())         header.X()->push();
-                           header.B()->push_value(B);
+   header                        .B()->push_value(B);
 
    header.eval_common();
 
@@ -471,13 +472,13 @@ UserFunction::eval_ALRXB(Value_P A, Token & LO, Token & RO,
    Workspace::push_SI(this, LOC);
 
    if (header.Z())         header.Z()->push();
-                           header.A()->push_value(A);
+   header                        .A()->push_value(A);
    if (LO.is_function())   header.LO()->push_function(LO.get_function());
    else                    header.LO()->push_value(LO.get_apl_val());
    if (RO.is_function())   header.RO()->push_function(RO.get_function());
    else                    header.RO()->push_value(RO.get_apl_val());
    if (header.X())         header.X()->push_value(X);
-                           header.B()->push_value(B);
+   header                        .B()->push_value(B);
 
    header.eval_common();
 
@@ -533,13 +534,16 @@ UCS_string & message_2 = error.error_message_2;
 }
 //-----------------------------------------------------------------------------
 void
-UserFunction::set_trace_stop(Function_Line * lines, int line_count, bool stop)
+UserFunction::set_trace_stop(Simple_string<Function_Line, false> & lines,
+                             bool stop)
 {
    // Sort lines
    //
-DynArray(bool, ts_lines, line_starts.size());
-   loop(ts, line_starts.size())   ts_lines[ts] = false;
-   loop(ll, line_count)
+Simple_string<bool, false> ts_lines;
+   ts_lines.reserve(line_starts.size());
+
+   loop(ts, line_starts.size())   ts_lines.append(false);
+   loop(ll, lines.size())
       {
         Function_Line line = lines[ll];
         if (line >= 1 && line < (int)line_starts.size())  ts_lines[line] = true;
@@ -547,7 +551,7 @@ DynArray(bool, ts_lines, line_starts.size());
 
    if (stop)
       {
-        stop_lines.clear();
+        stop_lines.shrink(0);
         loop(ts, line_starts.size())
            {
              if (ts_lines[ts])   stop_lines.append((Function_Line)ts);
@@ -555,7 +559,7 @@ DynArray(bool, ts_lines, line_starts.size());
       }
    else
       {
-        trace_lines.clear();
+        trace_lines.shrink(0);
         loop(ts, line_starts.size())
            {
              if (ts_lines[ts])   trace_lines.append((Function_Line)ts);
@@ -568,7 +572,7 @@ DynArray(bool, ts_lines, line_starts.size());
 void
 UserFunction::parse_body(const char * loc, bool tolerant, bool macro)
 {
-   line_starts.clear();
+   line_starts.shrink(0);
    line_starts.append(Function_PC_0);   // will be set later.
 
    clear_body();
@@ -713,8 +717,7 @@ UTF8_string utf((const UTF8 *)start, len);
 
    // skip trailing \r and \n.
    //
-   while (len > 0 && (utf[len - 1] == '\r' || utf[len - 1] == '\n'))   --len;
-   utf.erase(len);
+   while (utf.size() && (utf.last() == '\r' || utf.last() == '\n'))   utf.pop();
 
 UCS_string ucs(utf);
    munmap((char *)start, st.st_size);
@@ -752,21 +755,18 @@ const char * info = ufun->get_error_info();
         if (info)
            {
              Log(LOG_UserFunction__fix)   CERR << "Error: " << info << endl;
-             Workspace::more_error() = UCS_string(info);
+             MORE_ERROR() << info;
            }
 
          if (err_line == 0)
            {
-             Workspace::more_error().append_utf8(" (function header)");
+             MORE_ERROR() << " (function header)";
              Log(LOG_UserFunction__fix) CERR << "Bad header line" <<  endl;
            }
          else if (err_line > 0)
            {
-             Workspace::more_error().append_utf8(" (function line [");
-             Workspace::more_error().append_number(err_line);
-             Workspace::more_error().append_utf8("] of:\n");
-             Workspace::more_error().append(text);
-
+             MORE_ERROR() << " (function line [" << err_line << "] of:\n"
+                          << text;
              Log(LOG_UserFunction__fix)
                 CERR << "Bad function line: " << err_line << endl;
            }
@@ -982,10 +982,11 @@ UserFunction::adjust_line_starts()
    // this function is called from Executable::setup_lambdas() just before
    // Parser::remove_void_token(body) in order to adjust line_starts
    //
-DynArray(ShapeItem, gaps, line_starts.size());   // count TOK_VOID in every line
+Simple_string<ShapeItem, false> gaps;
+   gaps.reserve(line_starts.size());   // count TOK_VOID in every line
    loop(ls, line_starts.size())
       {
-         gaps[ls] = 0;
+         gaps.append(0);
          if (ls == 0)   continue;   // function header (has no TOK_VOID)
 
          const ShapeItem from = line_starts[ls];
@@ -994,9 +995,8 @@ DynArray(ShapeItem, gaps, line_starts.size());   // count TOK_VOID in every line
 
         for (ShapeItem b = from; b < to; ++b)
             {
-             if (body[b].get_tag() == TOK_VOID)   ++gaps[ls];
+             if (body[b].get_tag() == TOK_VOID)   ++gaps.last();
             }
-
       }
 
 int total_gaps = 0;
@@ -1007,6 +1007,19 @@ int total_gaps = 0;
        }
 }
 //-----------------------------------------------------------------------------
+Function_PC
+UserFunction::line_start(Function_Line line) const
+{
+   if (line < 0 || line >= line_starts.size())
+      {
+        Q1(line)
+        Q1(line_starts.size())
+        Assert(0);
+      }
+      
+   return line_starts[line];
+}
+//-----------------------------------------------------------------------------
 ostream &
 UserFunction::print_val_or_fun(ostream & out, Token & tok)
 {
