@@ -40,6 +40,8 @@ extern const char * configure_args;
 #include "Output.hh"
 #include "ScalarFunction.hh"
 #include "UserPreferences.hh"
+#include "Workspace.hh"
+
 #include "Value.icc"
 
 UserPreferences uprefs;
@@ -83,18 +85,19 @@ char cc[4000];
 
    snprintf(cc, sizeof(cc),
 "    --cfg                show ./configure options used and exit\n"
-"    --noCIN              do not echo input(for scripting)\n"
+"    --noCIN              do not echo input (for scripting)\n"
 "    --echoCIN            echo (final) input to COUT\n"
 "    --rawCIN             do not emit escape sequences\n"
 "    --[no]Color          start with ]XTERM ON [OFF])\n"
-"    --noCONT             do not )LOAD CONTINUE workspace on startup)\n"
+"    --noCONT             do not )LOAD CONTINUE or SETUP workspace on startup\n"
 "    --emacs              run in (classical) emacs mode\n"
 "    --emacs_arg arg      run in emacs mode with argument arg\n"
 "    --gpl                show license (GPL) and exit\n"
-"    -L wsname            )LOAD wsname on startup\n"
+"    -L wsname            )LOAD wsname (and not SETUP or CONTINUE) on startup\n"
 "    --LX expr            execute APL expression expr first\n"
 "    -p N                 use profile N in preferences files\n"
 "    --par proc           use processor parent ID proc (default: no parent)\n"
+"    --PW value           initial value of ⎕PW\n"
 "    -q, --silent         do not print the welcome banner\n"
 "    -s, --script         same as --silent --noCIN --noCONT --noColor\n"
 "    --safe               safe mode (no shared vars, no native functions)\n"
@@ -179,7 +182,7 @@ bool
 UserPreferences::parse_argv_1()
 {
 bool log_startup = false;
-   for (size_t a = 1; a < expanded_argv.size(); )
+   for (int a = 1; a < expanded_argv.size(); )
        {
          const char * opt = expanded_argv[a++];
          const char * val = (a < expanded_argv.size()) ? expanded_argv[a] : 0;
@@ -236,7 +239,7 @@ UserPreferences::parse_argv_2(bool logit)
          }
    }
 
-   for (size_t a = 1; a < expanded_argv.size(); )
+   for (int a = 1; a < expanded_argv.size(); )
        {
          if ((int)a == script_argc)   { ++a;   continue; }   // skip scriptname
 
@@ -313,7 +316,8 @@ UserPreferences::parse_argv_2(bool logit)
 
               const UTF8_string & filename(val);
               InputFile fam(filename, 0, false, !do_not_echo, true, no_LX);
-              InputFile::files_todo.push_back(fam);
+              InputFile::files_todo.append(fam);
+              InputFile::files_orig.append(fam);
               continue;
             }
 
@@ -439,6 +443,27 @@ UserPreferences::parse_argv_2(bool logit)
               continue;
             }
 
+         if (!strcmp(opt, "--PW"))
+            {
+              ++a;
+              if (!val)
+                {
+                  CERR << "--PW without screen width" << endl;
+                  exit(a);
+                }
+
+              initial_pw = atoi(val);
+              if (initial_pw < MIN_Quad_PW || initial_pw > MAX_Quad_PW)
+                {
+                  CERR << "bad --PW value (ignored)" << endl; 
+                }
+              else
+                {
+                  Workspace::set_PW(initial_pw, LOC);
+                }
+              continue;
+            }
+
          if (!strcmp(opt, "--rawCIN"))
             {
               raw_cin = true;
@@ -537,7 +562,8 @@ UserPreferences::parse_argv_2(bool logit)
                        }
                     const UTF8_string & filename = expanded_argv[a];
                     InputFile fam(filename, 0, true, true, false, no_LX);
-                    InputFile::files_todo.push_back(fam);
+                    InputFile::files_todo.append(fam);
+                    InputFile::files_orig.append(fam);
                   }
 
               // 
@@ -612,7 +638,7 @@ UserPreferences::parse_argv_2(bool logit)
       {
         const UTF8_string & filename = expanded_argv[script_argc];
         InputFile fam(filename, 0, false, !do_not_echo, true, no_LX);
-        InputFile::files_todo.insert(InputFile::files_todo.begin(), fam);
+        InputFile::files_todo.insert_before(0, fam);
       }
 
    // count number of testfiles
@@ -631,7 +657,7 @@ UserPreferences::parse_argv_2(bool logit)
     3. as a script with arguments on the first script line, e.g.
        /usr/bin/apl 'line 1 arguments' SCRIPT.apl [ scriptargs... ]
 
-    4. On a non-GNU/linux OS; the saemantics of the first script line
+    4. On a non-GNU/linux OS; the semantics of the first script line
        is unknown.
 
     To make this work, 2, should be avoided and 4. should use exactly one
@@ -655,8 +681,8 @@ UserPreferences::expand_argv(int argc, const char ** argv)
 {
    loop(a, argc)
        {
-         original_argv.push_back(argv[a]);
-         expanded_argv.push_back(argv[a]);
+         original_argv.append(argv[a]);
+         expanded_argv.append(argv[a]);
        }
 
    if (argc <= 1)   // no args at all
@@ -679,33 +705,52 @@ UserPreferences::expand_argv(int argc, const char ** argv)
    //
    // expand argv[1], stopping at --
    //
-const char * apl_args = argv[1];
+const char * apl_args = argv[1];   // the args after e.g. /usr/bin/apl
    script_argc = 2;
-   if (!strchr(apl_args, ' '))   return;   // single option
+   if (!strchr(apl_args, ' '))   // single option
+      {
+        return;
+      }
 
-   expanded_argv.erase(expanded_argv.begin() + 1);
+   expanded_argv.erase(1);
    --script_argc;
    for (int index = 1;;)
        {
+         // skip leading whitespace
+         //
          while (*apl_args && *apl_args <= ' ')   ++apl_args;
+
          if (*apl_args == 0)   break;
 
-         if (!strcmp(apl_args, "--"))        break;
-         if (!strncmp(apl_args, "-- ", 3))   break;
+         if (!strcmp(apl_args, "--"))        // "--" at end of apl_args
+            {
+              expanded_argv.insert_before(index++, "--");
+              ++script_argc;
+              break;   // done
+            }
+
+         if (!strncmp(apl_args, "-- ", 2) &&
+               apl_args[2] <= ' ')   // "--" somewhere in apl_args
+            {
+              expanded_argv.insert_before(index++, "--");
+              ++script_argc;
+              apl_args += 2;   // skip "--"
+              continue;
+            }
 
          const char * arg_end = strchr(apl_args, ' ');
          if (arg_end)   // more arguments
             {
               const int arg_len = arg_end - apl_args;
               const char * arg = strndup(apl_args, arg_len);
-              expanded_argv.insert(expanded_argv.begin() + index++, arg);
+              expanded_argv.insert_before(index++, arg);
               ++script_argc;
               apl_args += arg_len;
             }
          else           // last argument
             {
               const char * arg = strdup(apl_args);
-              expanded_argv.insert(expanded_argv.begin() + index++, arg);
+              expanded_argv.insert_before(index++, arg);
               ++script_argc;
               break;
             }
@@ -960,6 +1005,7 @@ int file_profile = 0;   // the current profile in the preferences file
          if (!strcasecmp(opt, "Profile"))   // Never ignore Profile entries
             {
               file_profile = atoi(arg);
+              continue;
             }
          if (file_profile && (file_profile != user_profile))   continue;
 
@@ -1169,6 +1215,23 @@ int file_profile = 0;   // the current profile in the preferences file
             {
               control_Ds_to_exit = atoi(arg);
             }
+         else if (!strcasecmp(opt, "INITIAL-⎕PW"))
+            {
+              if (initial_pw >= MIN_Quad_PW && initial_pw <= MAX_Quad_PW)
+                 initial_pw = atoi(arg);
+              else
+                 CERR << "bad value " << arg << " for INITIAL-⎕PW (ignored)"
+                      << endl;
+            }
+         else if (!strcasecmp(opt, "WINCH-SETS-⎕PW"))
+            {
+              WINCH_sets_pw = yes;
+            }
+         else
+            {
+              CERR << "Invalid option '" << opt << " in preferences file "
+                   << filename << " (ignored)" << endl;
+            }
        }
 
    fclose(f);
@@ -1218,17 +1281,21 @@ int line = 0;
          //
          {
            const char * param = strchr(p, '(');
-           if (param == 0)   goto param_error;   skip_spaces(++param);
+           if (param == 0)   goto param_error;
+           skip_spaces(++param);
 
            const char * p_ab = strchr(param, ',');
-           if (p_ab == 0)   goto param_error;   skip_spaces(++p_ab);
+           if (p_ab == 0)   goto param_error;
+           skip_spaces(++p_ab);
            const int i_ab = p_ab[1] == 'A' ? 2 : 1;
 
            const char * p_txt = strchr(p_ab, ',');
-           if (p_txt == 0)   goto param_error;   skip_spaces(++p_txt);
+           if (p_txt == 0)   goto param_error;
+           skip_spaces(++p_txt);
 
            const char * p_be = strchr(p_txt, ',');
-           if (p_be == 0)   goto param_error;   skip_spaces(++p_be);
+           if (p_be == 0)   goto param_error;
+           skip_spaces(++p_be);
 
            const ShapeItem value = strtoll(p_be, 0, 0);
 
