@@ -64,9 +64,9 @@ Value::init_ravel()
    if (Quad_SYL::value_count_limit &&
        Quad_SYL::value_count_limit < value_count)
       {
-        Workspace::more_error() = UCS_string(
+        MORE_ERROR() <<
 "the system limit on the APL value count (as set in ⎕SYL) was reached\n"
-"(and to avoid lock-up, the limit in ⎕SYL was automatically cleared).");
+"(and to avoid lock-up, the limit in ⎕SYL was automatically cleared).";
 
         // reset the limit so that we don't get stuck here.
         //
@@ -91,10 +91,10 @@ const ShapeItem length = shape.get_volume();
              new (&shape) Shape();
              new (ravel)   IntCell(42);
 
-             Workspace::more_error() = UCS_string(
+             MORE_ERROR() <<
              "the system limit on the total ravel size (as set in ⎕SYL) "
              "was reached\n(and to avoid lock-up, the limit in ⎕SYL "
-             "was automatically cleared).");
+             "was automatically cleared).";
 
              // reset the limit so that we don't get stuck here.
              //
@@ -784,14 +784,18 @@ Value::is_int_vector() const
 }
 //-----------------------------------------------------------------------------
 bool
-Value::is_complex() const
+Value::is_complex(bool check_numeric) const
 {
 const ShapeItem ec = nz_element_count();
 
    loop(e, ec)
       {
         const Cell & cell = get_ravel(e);
-        if (!cell.is_numeric())        DOMAIN_ERROR;
+        if (!cell.is_numeric())
+           {
+             if (check_numeric)    DOMAIN_ERROR;
+             else                  continue;
+           }
         if (!cell.is_near_real())   return true;
       }
 
@@ -799,6 +803,24 @@ const ShapeItem ec = nz_element_count();
 }
 //-----------------------------------------------------------------------------
 bool
+Value::can_be_compared() const
+{
+const ShapeItem count = nz_element_count();
+   loop(c, count)
+      {
+       const Cell & cell = get_ravel(c);
+       const CellType ctype = cell.get_cell_type();
+       if (ctype & (CT_CHAR | CT_INT | CT_FLOAT))   continue;
+       if (cell.is_near_real())                     continue;
+       if (cell.is_pointer_cell() &&
+           cell.get_pointer_value()->can_be_compared())   continue;
+       return false;
+      }
+
+   return true;
+}
+//-----------------------------------------------------------------------------
+bool
 Value::is_simple() const
 {
 const ShapeItem count = element_count();
@@ -1034,14 +1056,8 @@ const Cell * cI = &X->get_ravel(0);
          const ShapeItem idx = cI++->get_near_int() - qio;
          if (idx < 0 || idx >= max_idx)
             {
-              UCS_string & t4 = Workspace::more_error();
-              t4.clear();
-              t4.append_utf8("⎕IO=");
-              t4.append_number(qio);
-              t4.append_utf8(" offending index=");
-              t4.append_number(idx);
-              t4.append_utf8(" max index=");
-              t4.append_number(max_idx);
+              MORE_ERROR() << "⎕IO=" << qio << " offending index=" << idx
+                           << " max index=" << max_idx;
               Z->rollback(Z->valid_ravel_items, LOC);
               INDEX_ERROR;
             }
@@ -1534,90 +1550,6 @@ const ShapeItem ec = element_count();
 }
 //-----------------------------------------------------------------------------
 void
-Value::to_varnames(vector<UCS_string> & result, bool last) const
-{
-const ShapeItem var_count = get_rows();
-const ShapeItem name_len = get_cols();
-
-   loop(v, var_count)
-      {
-        ShapeItem nidx = v*name_len;
-        const ShapeItem end = nidx + name_len;
-        UCS_string & name = result[v];
-        loop(n, name_len)
-           {
-             const Unicode uni = get_ravel(nidx++).get_char_value();
-
-             if (n == 0 && Avec::is_quad(uni))   // leading ⎕
-                {
-                  name.append(uni);
-                  continue;
-                }
-
-             if (Avec::is_symbol_char(uni))   // valid symbol char
-                {
-                  name.append(uni);
-                  continue;
-                }
-
-             // end of (first) name reached. At this point we expect either
-             // spaces until 'end' or some spaces and another name.
-             //
-             if (uni != UNI_ASCII_SPACE)
-                {
-                  name.clear();
-                  break;
-                }
-
-             // we have reached the end of the first name. At this point
-             // there could bei:
-             //
-             // 1. spaces until 'end' (= one name), or
-             // 2. a second name (alias)
-
-             // skip spaces from nidx and subsequent spaces
-             //
-             while (nidx < end &&
-                    get_ravel(nidx).get_char_value() == UNI_ASCII_SPACE)
-                   ++nidx;
-
-             if (nidx == end)   break;   // only spaces (no second name)
-
-             // at this point we maybe have the start of a second name, which
-             // is an error (if last is false) or not. In both cases the first
-             // name can be ignored.
-             //
-             name.clear();
-             if (!last)   break;   // error
-
-             // 'last' is true thus to_varnames() was called from ⎕SVO and
-             // the line may contains two variable names.
-             // Return the second i.e. last) one)
-             //
-             last = false;
-             while (nidx < end)
-                {
-                  const Unicode uni = get_ravel(nidx++).get_char_value();
-                  if (Avec::is_symbol_char(uni))   // valid symbol char
-                     {
-                       name.append(uni);
-                     }
-                  else if (uni == UNI_ASCII_SPACE)
-                     {
-                       break;
-                     }
-                  else
-                     {
-                       name.clear();   // error
-                       break;
-                     }
-                }
-             break;
-           }
-      }
-}
-//-----------------------------------------------------------------------------
-void
 Value::to_proto()
 {
 const ShapeItem ec = nz_element_count();
@@ -1774,7 +1706,7 @@ const ShapeItem rows = ec/cols;
 int
 Value::print_incomplete(ostream & out)
 {
-vector<Value *> incomplete;
+Simple_string<Value *, false> incomplete;
 bool goon = true;
 
    for (const DynamicObject * dob = all_values.get_prev();
@@ -1786,7 +1718,7 @@ bool goon = true;
          if (val->is_complete())   continue;
 
          out << "incomplete value at " << (const void *)val << endl;
-         incomplete.push_back(val);
+         incomplete.append(val);
 
          if (!goon)
             {
@@ -1810,8 +1742,8 @@ bool goon = true;
 int
 Value::print_stale(ostream & out)
 {
-vector<Value *> stale_vals;
-vector<const DynamicObject *> stale_dobs;
+Simple_string<Value *, false> stale_vals;
+Simple_string<const DynamicObject *, false> stale_dobs;
 bool goon = true;
 int count = 0;
 
@@ -1826,8 +1758,8 @@ int count = 0;
          if (val->owner_count)   continue;
 
          out << "stale value at " << (const void *)val << endl;
-         stale_vals.push_back(val);
-         stale_dobs.push_back(dob);
+         stale_vals.append(val);
+         stale_dobs.append(dob);
 
          if (!goon)
             {
