@@ -2,7 +2,7 @@
     This file is part of GNU APL, a free implementation of the
     ISO/IEC Standard 13751, "Programming Language APL, Extended"
 
-    Copyright (C) 2008-2016  Dr. Jürgen Sauermann
+    Copyright (C) 2008-2017  Dr. Jürgen Sauermann
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -33,25 +33,13 @@ using namespace std;
 #include "LibPaths.hh"
 #include "Output.hh"
 #include "Quad_FX.hh"
+#include "Quad_SQL.hh"
 #include "Quad_TF.hh"
 #include "SystemVariable.hh"
 #include "UserFunction.hh"
 #include "UserPreferences.hh"
 #include "Workspace.hh"
 
-/* define 'all_values' and 'all_index_exprs' here so that they will be
-   constructed BEFORE Workspace::the_workspace !
-
-   See also 3.6.2 of "ISO standard Programming Languages — C++"
-*/
-
-DynamicObject DynamicObject::all_values(LOC);
-DynamicObject DynamicObject::all_index_exprs(LOC);
-
-// now Workspace::the_workspace can be constructed
-//
-Workspace Workspace::the_workspace;
-
 //-----------------------------------------------------------------------------
 Workspace::Workspace()
    : WS_name("CLEAR WS"),
@@ -81,9 +69,9 @@ Workspace::push_SI(Executable * fun, const char * loc)
    if (Quad_SYL::si_depth_limit && SI_top() &&
        Quad_SYL::si_depth_limit <= SI_top()->get_level())
       {
-        Workspace::more_error() = UCS_string(
+        MORE_ERROR() <<
         "the system limit on SI depth (as set in ⎕SYL) was reached\n"
-        "(and to avoid lock-up, the limit in ⎕SYL was automatically cleared).");
+        "(and to avoid lock-up, the limit in ⎕SYL was automatically cleared).";
 
         Quad_SYL::si_depth_limit = 0;
         set_attention_raised(LOC);
@@ -336,18 +324,16 @@ const Function * fun = current_referent->get_function();
 const UserFunction * ufun = fun->get_ufun1();
    if (!ufun)   return false;         // not a defined function
 
-const Executable * uexec = ufun;
-
    for (const StateIndicator * si = SI_top(); si; si = si->get_parent())
       {
-       if (uexec == si->get_executable())   return true;
+        if (si->uses_function(ufun))   return true;
       }
 
    return false;
 }
 //-----------------------------------------------------------------------------
 void
-Workspace::write_OUT(FILE * out, uint64_t & seq, const vector<UCS_string>
+Workspace::write_OUT(FILE * out, uint64_t & seq, const UCS_string_vector
                      & objects)
 {
    // if objects is empty then write all user define objects and some system
@@ -471,8 +457,8 @@ const int ret = the_workspace.expunged_functions.size();
 
    while(the_workspace.expunged_functions.size())
        {
-         const UserFunction * ufun = the_workspace.expunged_functions.back();
-         the_workspace.expunged_functions.pop_back();
+         const UserFunction * ufun = the_workspace.expunged_functions.last();
+         the_workspace.expunged_functions.pop();
          out << "finally deleting " << ufun->get_name() << "...";
          delete ufun;
          out << " OK" << endl;
@@ -485,21 +471,30 @@ const int ret = the_workspace.expunged_functions.size();
 void
 Workspace::clear_WS(ostream & out, bool silent)
 {
+   // remove user-defined commands
+   //
+   get_user_commands().shrink(0);
+
    // clear the SI (pops all localized symbols)
    //
    clear_SI(out);
 
-   // clear the symbol table
+   // clear the symbol tables
    //
    the_workspace.symbol_table.clear(out);
 
    // clear the )MORE error info
    //
-   more_error().clear();
+   more_error().shrink(0);
+
+   // ⎕PW and ⎕TZ shall survive )CLEAR (lrm p. 260);
+   //
+const int pw = the_workspace.v_Quad_PW[0].apl_val->get_sole_integer();
+const int tz = the_workspace.v_Quad_TZ.get_offset();
 
    // clear the value stacks of read/write system variables...
    //
-#define rw_sv_def(x, _str, _txt) get_v_ ## x().destruct();
+#define rw_sv_def(x, _str, _txt) get_v_ ## x().clear_vs();
 #define ro_sv_def(x, _str, _txt)
 #include "SystemVariable.def"
 
@@ -508,11 +503,14 @@ Workspace::clear_WS(ostream & out, bool silent)
    //
 // Value::erase_all(out);
 
-#define rw_sv_def(x, _str, _txt) new  (&get_v_ ##x()) x;
+#define rw_sv_def(x, _str, _txt) \
+   { get_v_ ##x().~x();   new (&get_v_ ##x()) x; }
 #define ro_sv_def(x, _str, _txt)
 #include "SystemVariable.def"
 
    get_v_Quad_RL().reset_seed();
+   Workspace::set_PW(pw, LOC);
+   the_workspace.v_Quad_TZ.set_offset(tz);
 
    // close open files in ⎕FIO
    Quad_FIO::fun->clear();
@@ -542,7 +540,7 @@ Workspace::list_SI(ostream & out, SI_mode mode)
 }
 //-----------------------------------------------------------------------------
 void
-Workspace::save_WS(ostream & out, vector<UCS_string> & lib_ws)
+Workspace::save_WS(ostream & out, UCS_string_vector & lib_ws)
 {
    // )SAVE
    // )SAVE wsname
@@ -552,29 +550,29 @@ bool name_from_WSID = false;
    if (lib_ws.size() == 0)   // no argument: use )WSID value
       {
         name_from_WSID = true;
-        lib_ws.push_back(the_workspace.WS_name);
+        lib_ws.append(the_workspace.WS_name);
       }
    else if (lib_ws.size() > 2)   // too many arguments
       {
         out << "BAD COMMAND+" << endl;
-        more_error() = UCS_string("too many parameters in command )SAVE");
+        MORE_ERROR() << "too many parameters in command )SAVE";
         return;
       }
 
    // at this point, lib_ws.size() is 1 or 2.
 
 LibRef libref = LIB_NONE;
-UCS_string wname = lib_ws.back();
-   if (lib_ws.size() == 2)   libref = (LibRef)(lib_ws.front().atoi());
+UCS_string wname = lib_ws.last();
+   if (lib_ws.size() == 2)   libref = (LibRef)(lib_ws[0].atoi());
 UTF8_string filename = LibPaths::get_lib_filename(libref, wname, false,
                                                   ".xml", 0);
 
    if (wname.compare(UCS_string("CLEAR WS")) == 0)   // don't save CLEAR WS
       {
         COUT << "NOT SAVED: THIS WS IS CLEAR WS" << endl;
-        more_error() = UCS_string(
+        MORE_ERROR() <<
         "the workspace was not saved because 'CLEAR WS' is a special \n"
-        "workspace name that cannot be saved. Use )WSID <name> first.");
+        "workspace name that cannot be saved. Use )WSID <name> first.";
         return;
       }
 
@@ -588,14 +586,12 @@ const bool file_exists = access(filename.c_str(), W_OK) == 0;
              COUT << "NOT SAVED: THIS WS IS "
                   << the_workspace.WS_name << endl;
 
-             UTF8_ostream more;
-             more << "the workspace was not saved because:\n"
+             MORE_ERROR() << "the workspace was not saved because:\n"
                   << "   the workspace name '" << the_workspace.WS_name
                   << "' of )WSID\n   does not match the name '" << wname
                   << "' used in the )SAVE command\n"
                   << "   and the workspace file\n   " << filename
                   << "\n   already exists. Use )WSID " << wname << " first."; 
-             more_error() = UCS_string(more.get_data());
              return;
            }
       }
@@ -695,7 +691,7 @@ const int err = rename(filename, backup_filename.c_str());
 void
 Workspace::load_DUMP(ostream & out, const UTF8_string & filename, int fd,
                      LX_mode with_LX, bool silent,
-                     vector<UCS_string> * object_filter)
+                     UCS_string_vector * object_filter)
 {
    Log(LOG_command_IN)
       CERR << "loading )DUMP file " << filename << "..." << endl;
@@ -727,18 +723,21 @@ FILE * file = fdopen(fd, "r");
       {
         if (filename == InputFile::files_todo[f].filename)   // same filename
            {
-             CERR << ")COPY " << filename << " causes recursion" << endl;
+             CERR << "*** )COPY " << filename
+                  << " would cause recursion (NOT COPIED)" << endl;
              return;
            }
       }
 
 InputFile fam(filename, file, false, false, true, with_LX);
-   if (object_filter)
+   if (object_filter)   // therefore )COPY, not )LOAD
       {
         loop(o, object_filter->size())
             fam.add_filter_object((*object_filter)[o]);
+        fam.set_COPY();
+        ++Bif_F1_EXECUTE::copy_pending;
       }
-   InputFile::files_todo.insert(InputFile::files_todo.begin(), fam);
+   InputFile::files_todo.insert_before(0, fam);
 }
 //-----------------------------------------------------------------------------
 /// a streambuf that escapes certain HTML characters
@@ -755,7 +754,7 @@ public:
       {
         switch(c & 0xFF)
            {
-              case '#':  out_file << "&#35";    break;
+              case '#':  out_file << "&#35;";    break;
               case '%':  out_file << "&#37;";   break;
               case '&':  out_file << "&#38;";   break;
               case '<':  out_file << "&lt;";    break;
@@ -781,7 +780,7 @@ public:
 };
 //-----------------------------------------------------------------------------
 void
-Workspace::dump_WS(ostream & out, vector<UCS_string> & lib_ws, bool html,
+Workspace::dump_WS(ostream & out, UCS_string_vector & lib_ws, bool html,
                    bool silent)
 {
    // )DUMP
@@ -790,29 +789,29 @@ Workspace::dump_WS(ostream & out, vector<UCS_string> & lib_ws, bool html,
 
    if (lib_ws.size() == 0)   // no argument: use )WSID value
       {
-         lib_ws.push_back(the_workspace.WS_name);
+         lib_ws.append(the_workspace.WS_name);
       }
    else if (lib_ws.size() > 2)   // too many arguments
       {
         out << "BAD COMMAND+" << endl;
-        more_error() = UCS_string("too many parameters in command )DUMP");
+        MORE_ERROR() << "too many parameters in command )DUMP";
         return;
       }
 
    // at this point, lib_ws.size() is 1 or 2.
 
 LibRef libref = LIB_NONE;
-const UCS_string wname = lib_ws.back();
-   if (lib_ws.size() == 2)   libref = (LibRef)(lib_ws.front().atoi());
+const UCS_string wname = lib_ws.last();
+   if (lib_ws.size() == 2)   libref = (LibRef)(lib_ws[0].atoi());
 const char * extension = html ? ".html" : ".apl";
 UTF8_string filename = LibPaths::get_lib_filename(libref, wname, false,
                                                   extension, 0);
    if (wname.compare(UCS_string("CLEAR WS")) == 0)   // don't save CLEAR WS
       {
         COUT << "NOT DUMPED: THIS WS IS " << wname << endl;
-        more_error() = UCS_string(
+        MORE_ERROR() <<
         "the workspace was not dumped because 'CLEAR WS' is a special\n"
-        "workspace name that cannot be dumped. Use )WSID <name> first.");
+        "workspace name that cannot be dumped. Use )WSID <name> first.";
         return;
       }
 
@@ -909,6 +908,7 @@ const char * tz_sign = (offset < 0) ? "" : "+";
 int function_count = 0;
 int variable_count = 0;
    the_workspace.symbol_table.dump(*sout, function_count, variable_count);
+   the_workspace.dump_commands(*sout);
 
    // system variables
    //
@@ -939,21 +939,34 @@ int variable_count = 0;
       }
 }
 //-----------------------------------------------------------------------------
+void
+Workspace::dump_commands(ostream & out)
+{
+Simple_string<Command::user_command, false> & cmds = get_user_commands();
+
+   loop(c, cmds.size())
+      out << "]USERCMD " << cmds[c].prefix
+          << " " << cmds[c].apl_function
+          << " " << cmds[c].mode << endl;
+
+   if (cmds.size())   out << endl;
+}
+//-----------------------------------------------------------------------------
 // )LOAD WS, set ⎕LX of loaded WS on success
 void
-Workspace::load_WS(ostream & out, const vector<UCS_string> & lib_ws,
+Workspace::load_WS(ostream & out, const UCS_string_vector & lib_ws,
                    UCS_string & quad_lx, bool silent)
 {
    if (lib_ws.size() < 1 || lib_ws.size() > 2)   // no or too many argument(s)
       {
         out << "BAD COMMAND+" << endl;
-        more_error() = UCS_string("too many parameters in command )LOAD");
+        MORE_ERROR() << "too many parameters in command )LOAD";
         return;
       }
 
 LibRef libref = LIB_NONE;
-   if (lib_ws.size() == 2)   libref = (LibRef)(lib_ws.front().atoi());
-UCS_string wname = lib_ws.back();
+   if (lib_ws.size() == 2)   libref = (LibRef)(lib_ws[0].atoi());
+UCS_string wname = lib_ws.last();
 UTF8_string filename = LibPaths::get_lib_filename(libref, wname, true,
                                                   ".xml", ".apl");
 
@@ -1011,7 +1024,7 @@ XML_Loading_Archive in(filename.c_str(), dump_fd);
 }
 //-----------------------------------------------------------------------------
 void
-Workspace::copy_WS(ostream & out, vector<UCS_string> & lib_ws_objects,
+Workspace::copy_WS(ostream & out, UCS_string_vector & lib_ws_objects,
                    bool protection)
 {
    // )COPY wsname                              wsname /absolute or relative
@@ -1022,28 +1035,26 @@ Workspace::copy_WS(ostream & out, vector<UCS_string> & lib_ws_objects,
    if (lib_ws_objects.size() < 1)   // at least workspace name is required
       {
         out << "BAD COMMAND+" << endl;
-        more_error() = UCS_string(
-                       "missing parameter(s) in command )COPY or )PCOPY");
+        MORE_ERROR() << "missing parameter(s) in command )COPY or )PCOPY";
         return;
       }
 
 LibRef libref = LIB_NONE;
-   if (Avec::is_digit(lib_ws_objects.front()[0]))
+   if (Avec::is_digit(lib_ws_objects[0][0]))
       {
-        libref = (LibRef)(lib_ws_objects.front().atoi());
-        lib_ws_objects.erase(lib_ws_objects.begin());
+        libref = (LibRef)(lib_ws_objects[0].atoi());
+        lib_ws_objects.erase(0);
       }
 
    if (lib_ws_objects.size() < 1)   // at least workspace name is required
       {
         out << "BAD COMMAND+" << endl;
-        more_error() = UCS_string(
-                       "missing parameter(s) in command )COPY or )PCOPY");
+        MORE_ERROR() << "missing parameter(s) in command )COPY or )PCOPY";
         return;
       }
 
-UCS_string wname = lib_ws_objects.front();
-   lib_ws_objects.erase(lib_ws_objects.begin());
+UCS_string wname = lib_ws_objects[0];
+   lib_ws_objects.erase(0);
 
 UTF8_string filename = LibPaths::get_lib_filename(libref, wname, true,
                                                   ".xml", ".apl");
@@ -1075,8 +1086,8 @@ XML_Loading_Archive in(filename.c_str(), dump_fd);
 void
 Workspace::wsid(ostream & out, UCS_string arg, bool silent)
 {
-   while (arg.size() && arg[0] <= ' ')       arg.remove_front();
-   while (arg.size() && arg.back() <= ' ')   arg.erase(arg.size() - 1);
+   while (arg.size() && arg[0] <= ' ')       arg.erase(0);
+   while (arg.size() && arg.back() <= ' ')   arg.pop();
 
    if (arg.size() == 0)   // inquire workspace name
       {
